@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VBox_tm_h
@@ -55,17 +51,20 @@ typedef enum TMCLOCK
      * This clock ticks all the time, so use with care. */
     TMCLOCK_REAL = 0,
     /** Virtual guest time.
-     * This clock only ticks when the guest is running. It's implemented
-     * as an offset to real time. */
+     * This clock only ticks when the guest is running.  It's implemented
+     * as an offset to monotonic real time (GIP). */
     TMCLOCK_VIRTUAL,
     /** Virtual guest synchronized timer time.
-     * This is a special clock and timer queue for synchronizing virtual timers and
-     * virtual time sources. This clock is trying to keep up with TMCLOCK_VIRTUAL,
-     * but will wait for timers to be executed. If it lags too far behind TMCLOCK_VIRTUAL,
-     * it will try speed up to close the distance.
-     * @remarks Do not use this unless you *must*. */
+     * This is a special clock and timer queue for synchronizing virtual timers
+     * and virtual time sources.  This clock is trying to keep up with
+     * TMCLOCK_VIRTUAL, but will wait for timers to be executed.  If it lags
+     * too far behind TMCLOCK_VIRTUAL, it will try speed up to close the
+     * distance.
+     * @remarks Do not use this unless you really *must*. */
     TMCLOCK_VIRTUAL_SYNC,
-    /** Virtual CPU timestamp. (Running only when we're executing guest code.) */
+    /** Virtual CPU timestamp.
+     * By default this is a function of TMCLOCK_VIRTUAL_SYNC and the virtual
+     * CPU frequency. */
     TMCLOCK_TSC,
     /** Number of clocks. */
     TMCLOCK_MAX
@@ -74,8 +73,7 @@ typedef enum TMCLOCK
 
 /** @defgroup grp_tm_timer_flags Timer flags.
  * @{ */
-/** Use the default critical section for the class of timers.
- * Only devices have one at the moment. */
+/** Use the default critical section for the class of timers. */
 #define TMTIMER_FLAGS_DEFAULT_CRIT_SECT 0
 /** No critical section needed or a custom one is set using
  *  TMR3TimerSetCritSect(). */
@@ -150,6 +148,18 @@ typedef DECLCALLBACK(void) FNTMTIMERDEV(PPDMDEVINS pDevIns, PTMTIMER pTimer, voi
 /** Pointer to a device timer callback function. */
 typedef FNTMTIMERDEV *PFNTMTIMERDEV;
 
+/**
+ * Device timer callback function.
+ *
+ * @param   pUsbIns         The USB device instance the timer is associated
+ *                          with.
+ * @param   pTimer          The timer handle.
+ * @param   pvUser          User argument specified upon timer creation.
+ */
+typedef DECLCALLBACK(void) FNTMTIMERUSB(PPDMUSBINS pUsbIns, PTMTIMER pTimer, void *pvUser);
+/** Pointer to a timer callback for a USB device. */
+typedef FNTMTIMERUSB *PFNTMTIMERUSB;
+
 /**
  * Driver timer callback function.
  *
@@ -245,7 +255,7 @@ VMMR3DECL(int)          TMR3TimerLoad(PTMTIMERR3 pTimer, PSSMHANDLE pSSM);
 VMMR3DECL(int)          TMR3TimerSetCritSect(PTMTIMERR3 pTimer, PPDMCRITSECT pCritSect);
 VMMR3DECL(void)         TMR3TimerQueuesDo(PVM pVM);
 VMM_INT_DECL(void)      TMR3VirtualSyncFF(PVM pVM, PVMCPU pVCpu);
-VMM_INT_DECL(PRTTIMESPEC) TMR3UTCNow(PVM pVM, PRTTIMESPEC pTime);
+VMM_INT_DECL(PRTTIMESPEC) TMR3UtcNow(PVM pVM, PRTTIMESPEC pTime);
 /** @} */
 #endif /* IN_RING3 */
 
