@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___iprt_uni_h
@@ -104,13 +100,16 @@ typedef const RTUNICASERANGE *PCRTUNICASERANGE;
 /** @name Unicode Code Point Flags.
  * @internal
  * @{ */
-#define RTUNI_UPPER  RT_BIT(0)
-#define RTUNI_LOWER  RT_BIT(1)
-#define RTUNI_ALPHA  RT_BIT(2)
-#define RTUNI_XDIGIT RT_BIT(3)
-#define RTUNI_DDIGIT RT_BIT(4)
-#define RTUNI_WSPACE RT_BIT(5)
+#define RTUNI_UPPER         RT_BIT(0)
+#define RTUNI_LOWER         RT_BIT(1)
+#define RTUNI_ALPHA         RT_BIT(2)
+#define RTUNI_XDIGIT        RT_BIT(3)
+#define RTUNI_DDIGIT        RT_BIT(4)
+#define RTUNI_WSPACE        RT_BIT(5)
 /*#define RTUNI_BSPACE RT_BIT(6) - later */
+/** When set, the codepoint requires further checking wrt NFC and NFD
+ * normalization. I.e. set when either of QC_NFD and QC_NFC are not Y. */
+#define RTUNI_QC_NFX        RT_BIT(7)
 /** @} */
 
 
