@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2007-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -29,11 +25,16 @@
 # include <unistd.h>
 #endif
 #include <errno.h>
+#ifndef RT_OS_WINDOWS
+# include <signal.h>
+#endif
 
+#include "product-generated.h"
 #include <iprt/asm.h>
 #include <iprt/buildconfig.h>
 #include <iprt/initterm.h>
 #include <iprt/path.h>
+#include <iprt/semaphore.h>
 #include <iprt/string.h>
 #include <iprt/stream.h>
 #include <iprt/thread.h>
@@ -53,8 +54,12 @@ char *g_pszProgName =  (char *)"";
 int g_cVerbosity = 0;
 /** The default service interval (the -i | --interval) option). */
 uint32_t g_DefaultInterval = 0;
-/** Shutdown the main thread. (later, for signals.) */
-bool volatile g_fShutdown;
+#ifdef RT_OS_WINDOWS
+/** Signal shutdown to the Windows service thread. */
+static bool volatile g_fWindowsServiceShutdown;
+/** Event the Windows service thread waits for shutdown. */
+static RTSEMEVENT g_hEvtWindowsService;
+#endif
 
 /**
  * The details of the services that has been compiled in.
@@ -87,8 +92,17 @@ static struct
 #ifdef VBOXSERVICE_VMINFO
     { &g_VMInfo,    NIL_RTTHREAD, false, false, false, true },
 #endif
-#ifdef VBOXSERVICE_EXEC
-    { &g_Exec,      NIL_RTTHREAD, false, false, false, true },
+#ifdef VBOXSERVICE_CPUHOTPLUG
+    { &g_CpuHotPlug, NIL_RTTHREAD, false, false, false, true },
+#endif
+#ifdef VBOXSERVICE_MANAGEMENT
+# ifdef VBOX_WITH_MEMBALLOON
+    { &g_MemBalloon, NIL_RTTHREAD, false, false, false, true },
+# endif
+    { &g_VMStatistics, NIL_RTTHREAD, false, false, false, true },
+#endif
+#ifdef VBOX_WITH_PAGE_SHARING
+    { &g_PageSharing, NIL_RTTHREAD, false, false, false, true },
 #endif
 };
 
@@ -100,35 +114,38 @@ static struct
  */
 static int VBoxServiceUsage(void)
 {
-    RTPrintf("usage: %s [-f|--foreground] [-v|--verbose] [-i|--interval <seconds>]\n"
-             "           [--disable-<service>] [--enable-<service>] [-h|-?|--help]\n", g_pszProgName);
+    RTPrintf("Usage:\n"
+             " %-12s [-f|--foreground] [-v|--verbose] [-i|--interval <seconds>]\n"
+             "              [--disable-<service>] [--enable-<service>] [-h|-?|--help]\n", g_pszProgName);
 #ifdef RT_OS_WINDOWS
-    RTPrintf("           [-r|--register] [-u|--unregister]\n");
+    RTPrintf("              [-r|--register] [-u|--unregister]\n");
 #endif
     for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
-        RTPrintf("           %s\n", g_aServices[j].pDesc->pszUsage);
+        if (g_aServices[j].pDesc->pszUsage)
+            RTPrintf("%s\n", g_aServices[j].pDesc->pszUsage);
     RTPrintf("\n"
              "Options:\n"
-             "    -i | --interval          The default interval.\n"
-             "    -f | --foreground        Don't daemonzie the program. For debugging.\n"
-             "    -v | --verbose           Increment the verbosity level. For debugging.\n"
-             "    -h | -? | --help         Show this message and exit with status 1.\n"
+             "    -i | --interval         The default interval.\n"
+             "    -f | --foreground       Don't daemonzie the program. For debugging.\n"
+             "    -v | --verbose          Increment the verbosity level. For debugging.\n"
+             "    -h | -? | --help        Show this message and exit with status 1.\n"
              );
 #ifdef RT_OS_WINDOWS
-    RTPrintf("    -r | --register          Installs the service.\n"
-             "    -u | --unregister        Uninstall service.\n");
+    RTPrintf("    -r | --register         Installs the service.\n"
+             "    -u | --unregister       Uninstall service.\n");
 #endif
 
     RTPrintf("\n"
-             "Service specific options:\n");
+             "Service-specific options:\n");
     for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
     {
-        RTPrintf("    --enable-%-10s Enables the %s service. (default)\n", g_aServices[j].pDesc->pszName, g_aServices[j].pDesc->pszName);
-        RTPrintf("    --disable-%-9s Disables the %s service.\n", g_aServices[j].pDesc->pszName, g_aServices[j].pDesc->pszName);
-        RTPrintf("%s", g_aServices[j].pDesc->pszOptions);
+        RTPrintf("    --enable-%-14s Enables the %s service. (default)\n", g_aServices[j].pDesc->pszName, g_aServices[j].pDesc->pszName);
+        RTPrintf("    --disable-%-13s Disables the %s service.\n", g_aServices[j].pDesc->pszName, g_aServices[j].pDesc->pszName);
+        if (g_aServices[j].pDesc->pszOptions)
+            RTPrintf("%s", g_aServices[j].pDesc->pszOptions);
     }
     RTPrintf("\n"
-             " Copyright (C) 2009 Sun Microsystems, Inc.\n");
+             " Copyright (C) 2009-" VBOX_C_YEAR " " VBOX_VENDOR "\n");
 
     return 1;
 }
@@ -137,11 +154,11 @@ static int VBoxServiceUsage(void)
 /**
  * Displays a syntax error message.
  *
- * @returns 1
+ * @returns RTEXITCODE_SYNTAX.
  * @param   pszFormat   The message text.
  * @param   ...         Format arguments.
  */
-int VBoxServiceSyntax(const char *pszFormat, ...)
+RTEXITCODE VBoxServiceSyntax(const char *pszFormat, ...)
 {
     RTStrmPrintf(g_pStdErr, "%s: syntax error: ", g_pszProgName);
 
@@ -150,18 +167,18 @@ int VBoxServiceSyntax(const char *pszFormat, ...)
     RTStrmPrintfV(g_pStdErr, pszFormat, va);
     va_end(va);
 
-    return 1;
+    return RTEXITCODE_SYNTAX;
 }
 
 
 /**
  * Displays an error message.
  *
- * @returns 1
+ * @returns RTEXITCODE_FAILURE.
  * @param   pszFormat   The message text.
  * @param   ...         Format arguments.
  */
-int VBoxServiceError(const char *pszFormat, ...)
+RTEXITCODE VBoxServiceError(const char *pszFormat, ...)
 {
     RTStrmPrintf(g_pStdErr, "%s: error: ", g_pszProgName);
 
@@ -174,7 +191,7 @@ int VBoxServiceError(const char *pszFormat, ...)
     LogRel(("%s: Error: %N", g_pszProgName, pszFormat, &va));
     va_end(va);
 
-    return 1;
+    return RTEXITCODE_FAILURE;
 }
 
 
@@ -246,6 +263,16 @@ int VBoxServiceArgUInt32(int argc, char **argv, const char *psz, int *pi, uint32
 static DECLCALLBACK(int) VBoxServiceThread(RTTHREAD ThreadSelf, void *pvUser)
 {
     const unsigned i = (uintptr_t)pvUser;
+
+#ifndef RT_OS_WINDOWS
+    /*
+     * Block all signals for this thread. Only the main thread will handle signals.
+     */
+    sigset_t signalMask;
+    sigfillset(&signalMask);
+    pthread_sigmask(SIG_BLOCK, &signalMask, NULL);
+#endif
+
     int rc = g_aServices[i].pDesc->pfnWorker(&g_aServices[i].fShutdown);
     ASMAtomicXchgBool(&g_aServices[i].fShutdown, true);
     RTThreadUserSignal(ThreadSelf);
@@ -253,28 +280,25 @@ static DECLCALLBACK(int) VBoxServiceThread(RTTHREAD ThreadSelf, void *pvUser)
 }
 
 
-unsigned VBoxServiceGetStartedServices(void)
+/**
+ * Check if at least one service should be started.
+ */
+static bool VBoxServiceCheckStartedServices(void)
 {
-    unsigned iMain = ~0U;
     for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
         if (g_aServices[j].fEnabled)
-        {
-            iMain = j;
-            break;
-        }
+            return true;
 
-   return iMain; /* Return the index of the main service (must always come last!). */
+   return false;
 }
 
+
 /**
  * Starts the service.
  *
  * @returns VBox status code, errors are fully bitched.
- *
- * @param   iMain           The index of the service that belongs to the main
- *                          thread. Pass ~0U if none does.
  */
-int VBoxServiceStartServices(unsigned iMain)
+int VBoxServiceStartServices(void)
 {
     int rc;
 
@@ -288,9 +312,16 @@ int VBoxServiceStartServices(unsigned iMain)
             rc = g_aServices[j].pDesc->pfnInit();
             if (RT_FAILURE(rc))
             {
-                VBoxServiceError("Service '%s' failed to initialize: %Rrc\n",
-                                 g_aServices[j].pDesc->pszName, rc);
-                return rc;
+                if (rc != VERR_SERVICE_DISABLED)
+                {
+                    VBoxServiceError("Service '%s' failed to initialize: %Rrc\n",
+                                     g_aServices[j].pDesc->pszName, rc);
+                    return rc;
+                }
+                g_aServices[j].fEnabled = false;
+                VBoxServiceVerbose(0, "Service '%s' was disabled because of missing functionality\n",
+                                   g_aServices[j].pDesc->pszName);
+
             }
         }
 
@@ -301,8 +332,7 @@ int VBoxServiceStartServices(unsigned iMain)
     rc = VINF_SUCCESS;
     for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
     {
-        if (    !g_aServices[j].fEnabled
-            ||  j == iMain)
+        if (!g_aServices[j].fEnabled)
             continue;
 
         VBoxServiceVerbose(2, "Starting service     '%s' ...\n", g_aServices[j].pDesc->pszName);
@@ -323,16 +353,7 @@ int VBoxServiceStartServices(unsigned iMain)
             rc = VERR_GENERAL_FAILURE;
         }
     }
-    if (RT_SUCCESS(rc))
-    {
-        /* The final service runs in the main thread. */
-        VBoxServiceVerbose(1, "Starting '%s' in the main thread\n", g_aServices[iMain].pDesc->pszName);
-        rc = g_aServices[iMain].pDesc->pfnWorker(&g_fShutdown);
-        if (rc != VINF_SUCCESS) /* Only complain if service returned an error. Otherwise the service is a one-timer. */
-        {
-            VBoxServiceError("Service '%s' stopped unexpected; rc=%Rrc\n", g_aServices[iMain].pDesc->pszName, rc);
-        }
-    }
+
     return rc;
 }
 
@@ -347,40 +368,124 @@ int VBoxServiceStopServices(void)
 {
     int rc = VINF_SUCCESS;
 
+    /*
+     * Signal all the services.
+     */
     for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
-        ASMAtomicXchgBool(&g_aServices[j].fShutdown, true);
+        ASMAtomicWriteBool(&g_aServices[j].fShutdown, true);
+
+    /*
+     * Do the pfnStop callback on all running services.
+     */
     for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
         if (g_aServices[j].fStarted)
-            g_aServices[j].pDesc->pfnStop();
-    for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
-        if (g_aServices[j].fEnabled)
         {
-            if (g_aServices[j].Thread != NIL_RTTHREAD)
-            {
-                int rc;
-                VBoxServiceVerbose(2, "Waiting for service '%s' to stop ...\n", g_aServices[j].pDesc->pszName);
-                for (int i=0; i<30; i++) /* Wait 30 seconds in total */
-                {
-                    rc = RTThreadWait(g_aServices[j].Thread, 1000 /* Wait 1 second */, NULL);
-                    if (RT_SUCCESS(rc))
-                        break;
-#ifdef RT_OS_WINDOWS
-                    /* Notify SCM that it takes a bit longer ... */
-                    VBoxServiceWinSetStatus(SERVICE_STOP_PENDING, i);
-#endif
-                }
-                if (RT_FAILURE(rc))
-                    VBoxServiceError("Service '%s' failed to stop. (%Rrc)\n", g_aServices[j].pDesc->pszName, rc);
-            }
-            VBoxServiceVerbose(3, "Terminating service '%s' (%d) ...\n", g_aServices[j].pDesc->pszName, j);
-            g_aServices[j].pDesc->pfnTerm();
+            VBoxServiceVerbose(3, "Calling stop function for service '%s' ...\n", g_aServices[j].pDesc->pszName);
+            g_aServices[j].pDesc->pfnStop();
         }
 
+    /*
+     * Wait for all the service threads to complete.
+     */
+    for (unsigned j = 0; j < RT_ELEMENTS(g_aServices); j++)
+    {
+        if (!g_aServices[j].fEnabled) /* Only stop services which were started before. */
+            continue;
+        if (g_aServices[j].Thread != NIL_RTTHREAD)
+        {
+            VBoxServiceVerbose(2, "Waiting for service '%s' to stop ...\n", g_aServices[j].pDesc->pszName);
+            for (int i = 0; i < 30; i++) /* Wait 30 seconds in total */
+            {
+                rc = RTThreadWait(g_aServices[j].Thread, 1000 /* Wait 1 second */, NULL);
+                if (RT_SUCCESS(rc))
+                    break;
+#ifdef RT_OS_WINDOWS
+                /* Notify SCM that it takes a bit longer ... */
+                VBoxServiceWinSetStopPendingStatus(i + j*32);
+#endif
+            }
+            if (RT_FAILURE(rc))
+                VBoxServiceError("Service '%s' failed to stop. (%Rrc)\n", g_aServices[j].pDesc->pszName, rc);
+        }
+        VBoxServiceVerbose(3, "Terminating service '%s' (%d) ...\n", g_aServices[j].pDesc->pszName, j);
+        g_aServices[j].pDesc->pfnTerm();
+    }
+
+#ifdef RT_OS_WINDOWS
+    /*
+     * Wake up and tell the main() thread that we're shutting down (it's
+     * sleeping in VBoxServiceMainWait).
+     */
+    if (g_hEvtWindowsService != NIL_RTSEMEVENT)
+    {
+        VBoxServiceVerbose(3, "Stopping the main thread...\n");
+        ASMAtomicWriteBool(&g_fWindowsServiceShutdown, true);
+        rc = RTSemEventSignal(g_hEvtWindowsService);
+        AssertRC(rc);
+    }
+#endif
+
     VBoxServiceVerbose(2, "Stopping services returned: rc=%Rrc\n", rc);
     return rc;
 }
 
 
+/**
+ * Block the main thread until the service shuts down.
+ */
+void VBoxServiceMainWait(void)
+{
+    int rc;
+
+#ifdef RT_OS_WINDOWS
+    /*
+     * Wait for the semaphore to be signalled.
+     */
+    VBoxServiceVerbose(1, "Waiting in main thread\n");
+    rc = RTSemEventCreate(&g_hEvtWindowsService);
+    AssertRC(rc);
+    while (!ASMAtomicReadBool(&g_fWindowsServiceShutdown))
+    {
+        rc = RTSemEventWait(g_hEvtWindowsService, RT_INDEFINITE_WAIT);
+        AssertRC(rc);
+    }
+    RTSemEventDestroy(g_hEvtWindowsService);
+    g_hEvtWindowsService = NIL_RTSEMEVENT;
+
+#else
+    /*
+     * Wait explicitly for a HUP, INT, QUIT, ABRT or TERM signal, blocking
+     * all important signals.
+     *
+     * The annoying EINTR/ERESTART loop is for the benefit of Solaris where
+     * sigwait returns when we receive a SIGCHLD.  Kind of makes sense since
+     */
+    sigset_t signalMask;
+    sigemptyset(&signalMask);
+    sigaddset(&signalMask, SIGHUP);
+    sigaddset(&signalMask, SIGINT);
+    sigaddset(&signalMask, SIGQUIT);
+    sigaddset(&signalMask, SIGABRT);
+    sigaddset(&signalMask, SIGTERM);
+    pthread_sigmask(SIG_BLOCK, &signalMask, NULL);
+
+    int iSignal;
+    do
+    {
+        iSignal = -1;
+        rc = sigwait(&signalMask, &iSignal);
+    }
+    while (   rc == EINTR
+# ifdef ERESTART
+           || rc == ERESTART
+# endif
+          );
+
+    VBoxServiceVerbose(3, "VBoxServiceMainWait: Received signal %d (rc=%d)\n", iSignal, rc);
+#endif /* !RT_OS_WINDOWS */
+}
+
+
 int main(int argc, char **argv)
 {
     int rc = VINF_SUCCESS;
@@ -548,36 +653,21 @@ int main(int argc, char **argv)
     /*
      * Check that at least one service is enabled.
      */
-    unsigned iMain = VBoxServiceGetStartedServices();
-    if (iMain == ~0U)
+    if (!VBoxServiceCheckStartedServices())
         return VBoxServiceSyntax("At least one service must be enabled.\n");
 
     VBoxServiceVerbose(0, "%s r%s started. Verbose level = %d\n",
-        RTBldCfgVersion(), RTBldCfgRevisionStr(), g_cVerbosity);
+                       RTBldCfgVersion(), RTBldCfgRevisionStr(), g_cVerbosity);
 
     /*
      * Daemonize if requested.
      */
+    RTEXITCODE rcExit;
     if (fDaemonize && !fDaemonized)
     {
 #ifdef RT_OS_WINDOWS
-        /** @todo Should do something like VBoxSVC here, OR automatically re-register
-         *        the service and start it. Involving VbglR3Daemonize isn't an option
-         *        here.
-         *
-         *        Also, the idea here, IIRC, was to map the sub service to windows
-         *        services. The todo below is for mimicking windows services on
-         *        non-windows systems. Not sure if this is doable or not, but in anycase
-         *        this code can be moved into -win.
-         *
-         *        You should return when StartServiceCtrlDispatcher, btw., not
-         *        continue.
-         */
         VBoxServiceVerbose(2, "Starting service dispatcher ...\n");
-        if (!StartServiceCtrlDispatcher(&g_aServiceTable[0]))
-            return VBoxServiceError("StartServiceCtrlDispatcher: %u. Please start %s with option -f (foreground)!",
-                                    GetLastError(), g_pszProgName);
-        /* Service now lives in the control dispatcher registered above. */
+        rcExit = VBoxServiceWinEnterCtrlDispatcher();
 #else
         VBoxServiceVerbose(1, "Daemonizing...\n");
         rc = VbglR3Daemonize(false /* fNoChDir */, false /* fNoClose */);
@@ -588,19 +678,22 @@ int main(int argc, char **argv)
     }
 #ifdef RT_OS_WINDOWS
     else
+#endif
     {
-        /* Run the app just like a console one if not daemonized. */
-#endif
-        /** @todo Make the main thread responsive to signal so it can shutdown/restart the threads on non-SIGKILL signals. */
-
         /*
-         * Start the service, enter the main threads run loop and stop them again when it returns.
+         * Windows: We're running the service as a console application now. Start the
+         *          services, enter the main thread's run loop and stop them again
+         *          when it returns.
+         *
+         * POSIX:   This is used for both daemons and console runs. Start all services
+         *          and return immediately.
          */
-        rc = VBoxServiceStartServices(iMain);
+        rc = VBoxServiceStartServices();
+        rcExit = RT_SUCCESS(rc) ? RTEXITCODE_SUCCESS : RTEXITCODE_FAILURE;
+        if (RT_SUCCESS(rc))
+            VBoxServiceMainWait();
         VBoxServiceStopServices();
-#ifdef RT_OS_WINDOWS
     }
-#endif
 
 #ifdef RT_OS_WINDOWS
     /*
@@ -614,6 +707,6 @@ int main(int argc, char **argv)
 #endif
 
     VBoxServiceVerbose(0, "Ended.\n");
-    return RT_SUCCESS(rc) ? 0 : 1;
+    return rcExit;
 }
 
