@@ -1,5 +1,5 @@
 /** @file
- * vboxvfs -- VirtualBox Guest Additions for Linux: mount(8) helper
+ * vboxsf -- VirtualBox Guest Additions for Linux: mount(8) helper
  *
  * Parses options provided by mount (or user directly)
  * Packs them into struct vbsfmount and passes to mount(2)
@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,10 +16,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -141,7 +137,8 @@ process_mount_opts (const char *s, struct opts *opts)
         HONOSUID,
         HOSUID,
         HOREMOUNT,
-        HONOAUTO
+        HONOAUTO,
+        HONIGNORE
     } handler_opt;
     struct
     {
@@ -172,6 +169,7 @@ process_mount_opts (const char *s, struct opts *opts)
         {"suid",      HOSUID,      0, 0 },
         {"remount",   HOREMOUNT,   0, 0 },
         {"noauto",    HONOAUTO,    0, 0 },
+        {"_netdev",   HONIGNORE,   0, 0 },
         {NULL, 0, 0, NULL}
     }, *handler;
 
@@ -299,6 +297,7 @@ process_mount_opts (const char *s, struct opts *opts)
                     opts->convertcp[val_len] = 0;
                     break;
                 case HONOAUTO:
+                case HONIGNORE:
                     break;
                 }
                 break;
