@@ -2,7 +2,7 @@
 # Sun VirtualBox
 # VirtualBox postinstall script for Solaris.
 #
-# Copyright (C) 2008-2009 Sun Microsystems, Inc.
+# Copyright (C) 2008-2009 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -12,10 +12,6 @@
 # VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 # hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 #
-# Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
-# Clara, CA 95054 USA or visit http://www.sun.com if you need
-# additional information or have any questions.
-#
 
 uncompress_files()
 {
@@ -284,10 +280,18 @@ if test ! -z "$xorgbin"; then
         /usr/sbin/installf -c none $PKGINST $vboxclient_dest/1099.vboxclient=$vboxadditions_path/1099.vboxclient s
         clientinstalled=1
     fi
+
+    # Try other autostart locations if none of the above ones work
     if test $clientinstalled -eq 0; then
-        echo "*** Failed to configure client, couldn't find any autostart directory!"
-        # Exit as partially failed installation
-        retval=2
+        vboxclient_dest="/etc/xdg/autostart"
+        if test -d "$vboxclient_dest"; then
+            /usr/sbin/installf -c none $PKGINST $vboxclient_dest/1099.vboxclient=$vboxadditions_path/1099.vboxclient s
+            clientinstalled=1
+        else
+            echo "*** Failed to configure client, couldn't find any autostart directory!"
+            # Exit as partially failed installation
+            retval=2
+        fi
     fi
 else
     echo "(*) X.Org not found, skipped configuring X.Org guest additions."
