@@ -1,4 +1,23 @@
+/* $Id: bootp.c $ */
+/** @file
+ * NAT - BOOTP/DHCP server emulation.
+ */
+
 /*
+ * Copyright (C) 2006-2010 Oracle Corporation
+ *
+ * This file is part of VirtualBox Open Source Edition (OSE), as
+ * available from http://www.virtualbox.org. This file is free software;
+ * you can redistribute it and/or modify it under the terms of the GNU
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ */
+
+/*
+ * This code is based on:
+ *
  * QEMU BOOTP/DHCP server
  *
  * Copyright (c) 2004 Fabrice Bellard
@@ -87,7 +106,7 @@ static BOOTPClient *get_new_addr(PNATState pData, struct in_addr *paddr)
     if (!bc)
         return NULL;
 
-    paddr->s_addr = htonl(ntohl(pData->special_addr.s_addr) | (bc->number + START_ADDR));
+    paddr->s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | (bc->number + START_ADDR));
     bc->addr.s_addr = paddr->s_addr;
     return bc;
 }
@@ -166,7 +185,7 @@ static BOOTPClient *find_addr(PNATState pData, struct in_addr *paddr, const uint
 
             bc = &bootp_clients[i];
             bc->allocated = 1;
-            paddr->s_addr = htonl(ntohl(pData->special_addr.s_addr) | (i + START_ADDR));
+            paddr->s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | (i + START_ADDR));
             return bc;
         }
     }
@@ -191,8 +210,8 @@ static struct mbuf *dhcp_create_msg(PNATState pData, struct bootp_t *bp, struct
     rbp->bp_flags = bp->bp_flags; /* figure 2 of rfc2131 */
     rbp->bp_giaddr.s_addr = bp->bp_giaddr.s_addr;
 #if 0 /*check flags*/
-    saddr.sin_port = htons(BOOTP_SERVER);
-    daddr.sin_port = htons(BOOTP_CLIENT);
+    saddr.sin_port = RT_H2N_U16_C(BOOTP_SERVER);
+    daddr.sin_port = RT_H2N_U16_C(BOOTP_CLIENT);
 #endif
     rbp->bp_htype = 1;
     rbp->bp_hlen = 6;
@@ -219,8 +238,8 @@ static int dhcp_do_ack_offer(PNATState pData, struct mbuf *m, BOOTPClient *bc, i
     struct dns_domain_entry *dd = NULL;
     int added = 0;
     uint8_t *q_dns_header = NULL;
-    uint32_t lease_time = htonl(LEASE_TIME);
-    uint32_t netmask = htonl(pData->netmask);
+    uint32_t lease_time = RT_H2N_U32_C(LEASE_TIME);
+    uint32_t netmask = RT_H2N_U32(pData->netmask);
 
     rbp = mtod(m, struct bootp_t *);
     q = &rbp->bp_vend[0];
@@ -243,7 +262,7 @@ static int dhcp_do_ack_offer(PNATState pData, struct mbuf *m, BOOTPClient *bc, i
         rbp->bp_ciaddr.s_addr = bc->addr.s_addr; /* Client IP address */
     }
 #ifndef VBOX_WITH_NAT_SERVICE
-    saddr.s_addr = htonl(ntohl(pData->special_addr.s_addr) | CTL_ALIAS);
+    saddr.s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | CTL_ALIAS);
 #else
     saddr.s_addr = pData->special_addr.s_addr;
 #endif
@@ -271,9 +290,9 @@ static int dhcp_do_ack_offer(PNATState pData, struct mbuf *m, BOOTPClient *bc, i
     FILL_BOOTP_EXT(q, RFC1533_NETMASK, 4, &netmask);
     FILL_BOOTP_EXT(q, RFC1533_GATEWAY, 4, &saddr);
 
-    if (pData->use_dns_proxy || pData->use_host_resolver)
+    if (pData->fUseDnsProxy || pData->fUseHostResolver)
     {
-        uint32_t addr = htonl(ntohl(pData->special_addr.s_addr) | CTL_DNS);
+        uint32_t addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | CTL_DNS);
         FILL_BOOTP_EXT(q, RFC1533_DNS, 4, &addr);
         goto skip_dns_servers;
     }
@@ -299,7 +318,7 @@ skip_dns_servers:
             /* dhcpcd client very sad if no domain name is passed */
             FILL_BOOTP_EXT(q, RFC1533_DOMAINNAME, 1, " ");
     }
-    if (pData->fPassDomain && !pData->use_host_resolver)
+    if (pData->fPassDomain && !pData->fUseHostResolver)
     {
         LIST_FOREACH(dd, &pData->pDomainList, dd_list)
         {
@@ -339,7 +358,6 @@ static int dhcp_send_nack(PNATState pData, struct bootp_t *bp, BOOTPClient *bc,
 
 static int dhcp_send_ack(PNATState pData, struct bootp_t *bp, BOOTPClient *bc, struct mbuf *m, int fDhcpRequest)
 {
-    struct bootp_t *rbp;
     int offReply = 0; /* boot_reply will fill general options and add END before sending response */
 
     dhcp_create_msg(pData, bp, m, DHCPACK);
@@ -383,7 +401,6 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, const uint8_
     BOOTPClient *bc = NULL;
     struct in_addr daddr;
     int offReply;
-    uint8_t *opt;
     uint8_t *req_ip = NULL;
     uint8_t *server_ip = NULL;
     uint32_t ui32;
@@ -440,7 +457,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, const uint8_
             }
             else
             {
-               if ((bp->bp_ciaddr.s_addr & htonl(pData->netmask)) != pData->special_addr.s_addr)
+               if ((bp->bp_ciaddr.s_addr & RT_H2N_U32(pData->netmask)) != pData->special_addr.s_addr)
                {
                    LogRel(("NAT: Client %R[IP4] requested IP -- sending NAK\n", &bp->bp_ciaddr));
                    offReply = dhcp_send_nack(pData, bp, bc, m);
@@ -464,7 +481,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, const uint8_
             Assert(server_ip == NULL);
             Assert(req_ip != NULL);
             ui32 = *(uint32_t *)(req_ip + 2);
-            if ((ui32 & htonl(pData->netmask)) != pData->special_addr.s_addr)
+            if ((ui32 & RT_H2N_U32(pData->netmask)) != pData->special_addr.s_addr)
             {
                 LogRel(("NAT: address %R[IP4] has been requested -- sending NAK\n", &ui32));
                 offReply = dhcp_send_nack(pData, bp, bc, m);
@@ -618,7 +635,7 @@ static void dhcp_decode(PNATState pData, struct bootp_t *bp, const uint8_t *buf,
     int pmsg_type;
     struct in_addr req_ip;
     int fDhcpDiscover = 0;
-    int len, tag;
+    uint8_t *parameter_list = NULL;
     struct mbuf *m = NULL;
 
     pmsg_type = 0;
@@ -634,6 +651,27 @@ static void dhcp_decode(PNATState pData, struct bootp_t *bp, const uint8_t *buf,
     Assert(p);
     if (p == NULL)
         return;
+    /*
+     * We're going update dns list at least once per DHCP transaction (!not on every operation
+     * within transaction), assuming that transaction can't be longer than 1 min.
+     */
+    if (   !pData->fUseHostResolver
+           && (   pData->dnsLastUpdate == 0
+               || curtime - pData->dnsLastUpdate > 60 * 1000)) /* one minute*/
+    {
+        uint8_t i = 2; /* i = 0 - tag, i == 1 - length */
+        parameter_list = dhcp_find_option(&bp->bp_vend[0], RFC2132_PARAM_LIST);
+        for (;parameter_list && i < parameter_list[1]; ++i)
+        {
+            if (parameter_list[i] == RFC1533_DNS)
+            {
+                slirp_release_dns_list(pData);
+                slirp_init_dns_list(pData);
+                pData->dnsLastUpdate = curtime;
+                break;
+            }
+        }
+    }
 
 #ifndef VBOX_WITH_SLIRP_BSD_MBUF
     if ((m = m_get(pData)) == NULL)
@@ -663,7 +701,7 @@ static void dhcp_decode(PNATState pData, struct bootp_t *bp, const uint8_t *buf,
             break;
 
         case DHCPRELEASE:
-            rc = dhcp_decode_release(pData, bp, buf, size); 
+            rc = dhcp_decode_release(pData, bp, buf, size);
             /* no reply required */
             break;
 
@@ -689,7 +727,7 @@ static void dhcp_decode(PNATState pData, struct bootp_t *bp, const uint8_t *buf,
     }
     Assert(m);
     /*silently ignore*/
-    m_free(pData, m);
+    m_freem(pData, m);
     return;
 
 reply:
@@ -711,7 +749,7 @@ static void bootp_reply(PNATState pData, struct mbuf *m, int offReply, uint16_t
     q += offReply;
 
 #ifndef VBOX_WITH_NAT_SERVICE
-    saddr.sin_addr.s_addr = htonl(ntohl(pData->special_addr.s_addr) | CTL_ALIAS);
+    saddr.sin_addr.s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | CTL_ALIAS);
 #else
     saddr.sin_addr.s_addr = pData->special_addr.s_addr;
 #endif
@@ -733,8 +771,8 @@ static void bootp_reply(PNATState pData, struct mbuf *m, int offReply, uint16_t
         daddr.sin_addr.s_addr = INADDR_BROADCAST;
     else
         daddr.sin_addr.s_addr = rbp->bp_yiaddr.s_addr; /*unicast requested by client*/
-    saddr.sin_port = htons(BOOTP_SERVER);
-    daddr.sin_port = htons(BOOTP_CLIENT);
+    saddr.sin_port = RT_H2N_U16_C(BOOTP_SERVER);
+    daddr.sin_port = RT_H2N_U16_C(BOOTP_CLIENT);
     udp_output2(pData, NULL, m, &saddr, &daddr, IPTOS_LOWDELAY);
 }
 
