@@ -1,4 +1,24 @@
+/* $Id: debug.c $ */
+/** @file
+ * NAT - debug helpers.
+ */
+
 /*
+ * Copyright (C) 2006-2010 Oracle Corporation
+ *
+ * This file is part of VirtualBox Open Source Edition (OSE), as
+ * available from http://www.virtualbox.org. This file is free software;
+ * you can redistribute it and/or modify it under the terms of the GNU
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ */
+
+
+/*
+ * This code is based on:
+ *
  * Copyright (c) 1995 Danny Gasparovski.
  * Portions copyright (c) 2000 Kelly Price.
  *
@@ -23,7 +43,7 @@ void dump_packet(void *, int);
 void
 dump_packet(void *dat, int n)
 {
-    Log(("nat: PACKET DUMPED:\n%.*Vhxd\n", n, dat));
+    Log(("nat: PACKET DUMPED:\n%.*Rhxd\n", n, dat));
 }
 #endif
 
@@ -160,7 +180,7 @@ mbufstats(PNATState pData)
 {
 #ifndef VBOX_WITH_SLIRP_BSD_MBUF
     /*
-     * (vvl) this static code can't work with mbuf zone anymore 
+     * (vvl) this static code can't work with mbuf zone anymore
      * @todo: make statistic correct
      */
 #if 0
@@ -191,7 +211,7 @@ void
 sockstats(PNATState pData)
 {
     char buff[256];
-    int n;
+    size_t n;
     struct socket *so, *so_next;
 
     lprint(" \n");
@@ -206,9 +226,9 @@ sockstats(PNATState pData)
             buff[n++] = ' ';
         buff[17] = 0;
         lprint("%s %3d   %15s %5d ",
-               buff, so->s, inet_ntoa(so->so_laddr), ntohs(so->so_lport));
+               buff, so->s, inet_ntoa(so->so_laddr), RT_N2H_U16(so->so_lport));
         lprint("%15s %5d %5d %5d\n",
-                inet_ntoa(so->so_faddr), ntohs(so->so_fport),
+                inet_ntoa(so->so_faddr), RT_N2H_U16(so->so_fport),
                 so->so_rcv.sb_cc, so->so_snd.sb_cc);
     LOOP_LABEL(tcp, so, so_next);
     }
@@ -220,9 +240,9 @@ sockstats(PNATState pData)
             buff[n++] = ' ';
         buff[17] = 0;
         lprint("%s %3d  %15s %5d  ",
-                buff, so->s, inet_ntoa(so->so_laddr), ntohs(so->so_lport));
+                buff, so->s, inet_ntoa(so->so_laddr), RT_N2H_U16(so->so_lport));
         lprint("%15s %5d %5d %5d\n",
-                inet_ntoa(so->so_faddr), ntohs(so->so_fport),
+                inet_ntoa(so->so_faddr), RT_N2H_U16(so->so_fport),
                 so->so_rcv.sb_cc, so->so_snd.sb_cc);
     LOOP_LABEL(udp, so, so_next);
     }
@@ -239,7 +259,7 @@ print_ipv4_address(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput,
 
     AssertReturn(strcmp(pszType, "IP4") == 0, 0);
 
-    ip = ntohl(*(uint32_t*)pvValue);
+    ip = RT_N2H_U32(*(uint32_t*)pvValue);
     return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, IP4_ADDR_PRINTF_FORMAT,
            IP4_ADDR_PRINTF_DECOMP(ip));
 }
@@ -291,14 +311,14 @@ print_socket(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput,
     }
 
     in_addr = (struct sockaddr_in *)&addr;
-    ip = ntohl(so->so_faddr.s_addr);
+    ip = RT_N2H_U32(so->so_faddr.s_addr);
     return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "socket %4d:(proto:%u) "
             "state=%04x ip=" IP4_ADDR_PRINTF_FORMAT ":%d "
             "name=" IP4_ADDR_PRINTF_FORMAT ":%d",
             so->s, so->so_type, so->so_state, IP4_ADDR_PRINTF_DECOMP(ip),
-            ntohs(so->so_fport),
-            IP4_ADDR_PRINTF_DECOMP(ntohl(in_addr->sin_addr.s_addr)),
-            ntohs(in_addr->sin_port));
+            RT_N2H_U16(so->so_fport),
+            IP4_ADDR_PRINTF_DECOMP(RT_N2H_U32(in_addr->sin_addr.s_addr)),
+            RT_N2H_U16(in_addr->sin_port));
 }
 
 static DECLCALLBACK(size_t)
