@@ -1,11 +1,10 @@
+/* $Id: VBoxHeadless.cpp $ */
 /** @file
- *
- * VBox frontends: VBoxHeadless (headless frontend):
- * Headless server executable
+ * VBoxHeadless - The VirtualBox Headless frontend for running VMs on servers.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include <VBox/com/com.h>
@@ -38,6 +33,7 @@ using namespace com;
 #ifdef VBOX_WITH_VRDP
 # include <VBox/vrdpapi.h>
 #endif
+#include <iprt/buildconfig.h>
 #include <iprt/ctype.h>
 #include <iprt/initterm.h>
 #include <iprt/stream.h>
@@ -65,20 +61,24 @@ using namespace com;
 #ifdef VBOX_WITH_VRDP
 # include "Framebuffer.h"
 #endif
+#ifdef VBOX_WITH_VNC
+# include "FramebufferVNC.h"
+#endif
+
 
 ////////////////////////////////////////////////////////////////////////////////
 
 #define LogError(m,rc) \
     do { \
-        Log (("VBoxHeadless: ERROR: " m " [rc=0x%08X]\n", rc)); \
-        RTPrintf ("%s\n", m); \
+        Log(("VBoxHeadless: ERROR: " m " [rc=0x%08X]\n", rc)); \
+        RTPrintf("%s\n", m); \
     } while (0)
 
 ////////////////////////////////////////////////////////////////////////////////
 
 /* global weak references (for event handlers) */
-static ComPtr <ISession, ComWeakRef> gSession;
-static ComPtr <IConsole, ComWeakRef> gConsole;
+static ISession *gSession = NULL;
+static IConsole *gConsole = NULL;
 static EventQueue *gEventQ = NULL;
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -89,20 +89,186 @@ static EventQueue *gEventQ = NULL;
 class StateChangeEvent : public Event
 {
 public:
-    StateChangeEvent (MachineState_T state) : mState (state) {}
+    StateChangeEvent(MachineState_T state) : mState(state) {}
 protected:
     void *handler()
     {
-        LogFlow (("VBoxHeadless: StateChangeEvent: %d\n", mState));
+        LogFlow(("VBoxHeadless: StateChangeEvent: %d\n", mState));
         /* post the termination event if the machine has been PoweredDown/Saved/Aborted */
         if (mState < MachineState_Running)
-            gEventQ->postEvent (NULL);
+            gEventQ->postEvent(NULL);
         return 0;
     }
 private:
     MachineState_T mState;
 };
 
+/**
+ * Callback handler for VirtualBox events
+ */
+class VirtualBoxCallback :
+  VBOX_SCRIPTABLE_IMPL(IVirtualBoxCallback)
+{
+public:
+    VirtualBoxCallback()
+    {
+#ifndef VBOX_WITH_XPCOM
+        refcnt = 0;
+#endif
+        mfNoLoggedInUsers = true;
+    }
+
+    virtual ~VirtualBoxCallback()
+    {
+    }
+
+#ifndef VBOX_WITH_XPCOM
+    STDMETHOD_(ULONG, AddRef)()
+    {
+        return ::InterlockedIncrement(&refcnt);
+    }
+    STDMETHOD_(ULONG, Release)()
+    {
+        long cnt = ::InterlockedDecrement(&refcnt);
+        if (cnt == 0)
+            delete this;
+        return cnt;
+    }
+#endif
+    VBOX_SCRIPTABLE_DISPATCH_IMPL(IVirtualBoxCallback)
+
+    NS_DECL_ISUPPORTS
+
+    STDMETHOD(OnMachineStateChange)(IN_BSTR machineId, MachineState_T state)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnMachineDataChange)(IN_BSTR machineId)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnExtraDataCanChange)(IN_BSTR machineId, IN_BSTR key, IN_BSTR value,
+                                    BSTR *error, BOOL *changeAllowed)
+    {
+        /* we never disagree */
+        if (!changeAllowed)
+            return E_INVALIDARG;
+        *changeAllowed = TRUE;
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnExtraDataChange)(IN_BSTR machineId, IN_BSTR key, IN_BSTR value)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnMediumRegistered)(IN_BSTR mediaId, DeviceType_T mediaType,
+                                  BOOL registered)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnMachineRegistered)(IN_BSTR machineId, BOOL registered)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnSessionStateChange)(IN_BSTR machineId, SessionState_T state)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnSnapshotTaken)(IN_BSTR aMachineId, IN_BSTR aSnapshotId)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnSnapshotDeleted)(IN_BSTR aMachineId, IN_BSTR aSnapshotId)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnSnapshotChange)(IN_BSTR aMachineId, IN_BSTR aSnapshotId)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnGuestPropertyChange)(IN_BSTR machineId, IN_BSTR key, IN_BSTR value, IN_BSTR flags)
+    {
+#ifdef VBOX_WITH_GUEST_PROPS
+        Utf8Str utf8Key = key;
+        if (utf8Key == "/VirtualBox/GuestInfo/OS/NoLoggedInUsers")
+        {
+            /* Check if the "disconnect on logout feature" is enabled. */
+            BOOL fDisconnectOnGuestLogout = FALSE;
+            ComPtr <IMachine> machine;
+            HRESULT hrc = S_OK;
+
+            if (gConsole)
+            {
+                hrc = gConsole->COMGETTER(Machine)(machine.asOutParam());
+                if (SUCCEEDED(hrc) && machine)
+                {
+                    Bstr value1;
+                    hrc = machine->GetExtraData(Bstr("VRDP/DisconnectOnGuestLogout"), value1.asOutParam());
+                    if (SUCCEEDED(hrc) && value1 == "1")
+                    {
+                        fDisconnectOnGuestLogout = TRUE;
+                    }
+                }
+            }
+
+            if (fDisconnectOnGuestLogout)
+            {
+                Utf8Str utf8Value = value;
+                if (utf8Value == "true")
+                {
+                    if (!mfNoLoggedInUsers) /* Only if the property really changes. */
+                    {
+                        mfNoLoggedInUsers = true;
+
+                        /* If there is a VRDP connection, drop it. */
+                        ComPtr<IRemoteDisplayInfo> info;
+                        hrc = gConsole->COMGETTER(RemoteDisplayInfo)(info.asOutParam());
+                        if (SUCCEEDED(hrc) && info)
+                        {
+                            ULONG cClients = 0;
+                            hrc = info->COMGETTER(NumberOfClients)(&cClients);
+                            if (SUCCEEDED(hrc) && cClients > 0)
+                            {
+                                ComPtr <IVRDPServer> vrdpServer;
+                                hrc = machine->COMGETTER(VRDPServer)(vrdpServer.asOutParam());
+                                if (SUCCEEDED(hrc) && vrdpServer)
+                                {
+                                    vrdpServer->COMSETTER(Enabled)(FALSE);
+                                    vrdpServer->COMSETTER(Enabled)(TRUE);
+                                }
+                            }
+                        }
+                    }
+                }
+                else
+                {
+                    mfNoLoggedInUsers = false;
+                }
+            }
+        }
+        return S_OK;
+#else  /* !VBOX_WITH_GUEST_PROPS */
+        return VBOX_E_DONT_CALL_AGAIN;
+#endif /* !VBOX_WITH_GUEST_PROPS */
+    }
+
+private:
+#ifndef VBOX_WITH_XPCOM
+    long refcnt;
+#endif
+
+    bool mfNoLoggedInUsers;
+};
+
 /**
  *  Callback handler for machine events.
  */
@@ -110,7 +276,7 @@ class ConsoleCallback : VBOX_SCRIPTABLE_IMPL(IConsoleCallback)
 {
 public:
 
-    ConsoleCallback ()
+    ConsoleCallback()
     {
 #ifndef VBOX_WITH_XPCOM
         refcnt = 0;
@@ -137,13 +303,13 @@ public:
 #endif
     VBOX_SCRIPTABLE_DISPATCH_IMPL(IConsoleCallback)
 
-    STDMETHOD(OnMousePointerShapeChange) (BOOL visible, BOOL alpha, ULONG xHot, ULONG yHot,
-                                          ULONG width, ULONG height, BYTE *shape)
+    STDMETHOD(OnMousePointerShapeChange)(BOOL visible, BOOL alpha, ULONG xHot, ULONG yHot,
+                                         ULONG width, ULONG height, ComSafeArrayIn(BYTE,shape))
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
-    STDMETHOD(OnMouseCapabilityChange) (BOOL supportsAbsolute, BOOL needsHostCursor)
+    STDMETHOD(OnMouseCapabilityChange)(BOOL supportsAbsolute, BOOL supportsRelative, BOOL needsHostCursor)
     {
         /* Emit absolute mouse event to actually enable the host mouse cursor. */
         if (supportsAbsolute && gConsole)
@@ -160,43 +326,43 @@ public:
 
     STDMETHOD(OnKeyboardLedsChange)(BOOL fNumLock, BOOL fCapsLock, BOOL fScrollLock)
     {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnStateChange)(MachineState_T machineState)
+    {
+        gEventQ->postEvent(new StateChangeEvent(machineState));
         return S_OK;
     }
 
-    STDMETHOD(OnStateChange) (MachineState_T machineState)
+    STDMETHOD(OnExtraDataChange)(BSTR key)
     {
-        gEventQ->postEvent (new StateChangeEvent (machineState));
-        return S_OK;
-    }
-
-    STDMETHOD(OnExtraDataChange) (BSTR key)
-    {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
     STDMETHOD(OnAdditionsStateChange)()
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
-    STDMETHOD(OnNetworkAdapterChange) (INetworkAdapter *aNetworkAdapter)
+    STDMETHOD(OnNetworkAdapterChange)(INetworkAdapter *aNetworkAdapter)
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
-    STDMETHOD(OnSerialPortChange) (ISerialPort *aSerialPort)
+    STDMETHOD(OnSerialPortChange)(ISerialPort *aSerialPort)
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
-    STDMETHOD(OnParallelPortChange) (IParallelPort *aParallelPort)
+    STDMETHOD(OnParallelPortChange)(IParallelPort *aParallelPort)
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
     STDMETHOD(OnVRDPServerChange)()
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
     STDMETHOD(OnRemoteDisplayInfoChange)()
@@ -229,33 +395,38 @@ public:
 
     STDMETHOD(OnUSBControllerChange)()
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
     STDMETHOD(OnStorageControllerChange)()
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
     STDMETHOD(OnMediumChange)(IMediumAttachment * /* aMediumAttachment */)
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
-    STDMETHOD(OnUSBDeviceStateChange) (IUSBDevice *aDevice, BOOL aAttached,
+    STDMETHOD(OnCPUChange)(ULONG /*aCPU*/, BOOL /* aRemove */)
+    {
+        return VBOX_E_DONT_CALL_AGAIN;
+    }
+
+    STDMETHOD(OnUSBDeviceStateChange)(IUSBDevice *aDevice, BOOL aAttached,
                                       IVirtualBoxErrorInfo *aError)
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
-    STDMETHOD(OnSharedFolderChange) (Scope_T aScope)
+    STDMETHOD(OnSharedFolderChange)(Scope_T aScope)
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
     STDMETHOD(OnRuntimeError)(BOOL fatal, IN_BSTR id, IN_BSTR message)
     {
-        return S_OK;
+        return VBOX_E_DONT_CALL_AGAIN;
     }
 
     STDMETHOD(OnCanShowWindow)(BOOL *canShow)
@@ -267,7 +438,7 @@ public:
         return S_OK;
     }
 
-    STDMETHOD(OnShowWindow) (ULONG64 *winId)
+    STDMETHOD(OnShowWindow)(ULONG64 *winId)
     {
         /* OnCanShowWindow() always returns FALSE, so this call should never
          * happen. */
@@ -287,8 +458,10 @@ private:
 };
 
 #ifdef VBOX_WITH_XPCOM
-NS_DECL_CLASSINFO (ConsoleCallback)
-NS_IMPL_THREADSAFE_ISUPPORTS1_CI (ConsoleCallback, IConsoleCallback)
+NS_DECL_CLASSINFO(VirtualBoxCallback)
+NS_IMPL_THREADSAFE_ISUPPORTS1_CI(VirtualBoxCallback, IVirtualBoxCallback)
+NS_DECL_CLASSINFO(ConsoleCallback)
+NS_IMPL_THREADSAFE_ISUPPORTS1_CI(ConsoleCallback, IConsoleCallback)
 #endif
 
 #ifdef VBOX_WITH_SAVESTATE_ON_SIGNAL
@@ -356,6 +529,11 @@ static void show_usage()
 {
     RTPrintf("Usage:\n"
              "   -s, -startvm, --startvm <name|uuid>   Start given VM (required argument)\n"
+#ifdef VBOX_WITH_VNC
+             "   -n, --vnc                             Enable the built in VNC server\n"
+             "   -m, --vncport <port>                  TCP port number to use for the VNC server\n"
+             "   -o, --vncpass <pw>                    Set the VNC server password\n"
+#endif
 #ifdef VBOX_WITH_VRDP
              "   -v, -vrdp, --vrdp on|off|config       Enable (default) or disable the VRDP\n"
              "                                         server or don't change the setting\n"
@@ -425,12 +603,17 @@ static void parse_environ(unsigned long *pulFrameWidth, unsigned long *pulFrameH
 /**
  *  Entry point.
  */
-extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
+extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
 {
 #ifdef VBOX_WITH_VRDP
     const char *vrdpPort = NULL;
     const char *vrdpAddress = NULL;
     const char *vrdpEnabled = NULL;
+#endif
+#ifdef VBOX_WITH_VNC
+    bool        fVNCEnable      = false;
+    unsigned    uVNCPort        = 0;          /* default port */
+    char const *pszVNCPassword  = NULL;       /* no password */
 #endif
     unsigned fRawR0 = ~0U;
     unsigned fRawR3 = ~0U;
@@ -489,6 +672,11 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
         { "-vrdp", 'v', RTGETOPT_REQ_STRING },
         { "--vrdp", 'v', RTGETOPT_REQ_STRING },
 #endif /* VBOX_WITH_VRDP defined */
+#ifdef VBOX_WITH_VNC
+        { "--vncport", 'm', RTGETOPT_REQ_INT32 },
+        { "--vncpass", 'o', RTGETOPT_REQ_STRING },
+        { "--vnc", 'n', 0 },
+#endif /* VBOX_WITH_VNC */
         { "-rawr0", OPT_RAW_R0, 0 },
         { "--rawr0", OPT_RAW_R0, 0 },
         { "-norawr0", OPT_NO_RAW_R0, 0 },
@@ -509,7 +697,7 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
         { "-capture", 'c', 0 },
         { "--capture", 'c', 0 },
         { "--width", 'w', RTGETOPT_REQ_UINT32 },
-        { "--height", 'h', RTGETOPT_REQ_UINT32 },
+        { "--height", 'h', RTGETOPT_REQ_UINT32 }, /* great choice of short option! */
         { "--bitrate", 'r', RTGETOPT_REQ_UINT32 },
         { "--filename", 'f', RTGETOPT_REQ_STRING },
 #endif /* VBOX_FFMPEG defined */
@@ -529,7 +717,7 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
             case 's':
                 id = asGuidStr(ValueUnion.psz);
                 /* If the argument was not a UUID, then it must be a name. */
-                if (!id)
+                if (id.isEmpty())
                     name = ValueUnion.psz;
                 break;
 #ifdef VBOX_WITH_VRDP
@@ -543,6 +731,17 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
                 vrdpEnabled = ValueUnion.psz;
                 break;
 #endif /* VBOX_WITH_VRDP defined */
+#ifdef VBOX_WITH_VNC
+            case 'n':
+                fVNCEnable = true;
+                break;
+            case 'm':
+                uVNCPort = ValueUnion.i32;
+                break;
+            case 'o':
+                pszVNCPassword = ValueUnion.psz;
+                break;
+#endif /* VBOX_WITH_VNC */
             case OPT_RAW_R0:
                 fRawR0 = true;
                 break;
@@ -574,9 +773,6 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
             case 'w':
                 ulFrameWidth = ValueUnion.u32;
                 break;
-            case 'h':
-                ulFrameHeight = ValueUnion.u32;
-                break;
             case 'r':
                 ulBitRate = ValueUnion.u32;
                 break;
@@ -584,29 +780,26 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
                 pszFileNameParam = ValueUnion.psz;
                 break;
 #endif /* VBOX_FFMPEG defined */
-            case VINF_GETOPT_NOT_OPTION:
-                RTPrintf("Invalid parameter '%s'\n\n", ValueUnion.psz);
+            case 'h':
+#ifdef VBOX_FFMPEG
+                if ((GetState.pDef->fFlags & RTGETOPT_REQ_MASK) != RTGETOPT_REQ_NOTHING)
+                {
+                    ulFrameHeight = ValueUnion.u32;
+                    break;
+                }
+#endif
                 show_usage();
-                return -1;
+                return 0;
             case OPT_COMMENT:
                 /* nothing to do */
                 break;
+            case 'V':
+                RTPrintf("%sr%s\n", RTBldCfgVersion(), RTBldCfgRevisionStr());
+                return 0;
             default:
-                if (ch > 0)
-                {
-                    if (RT_C_IS_PRINT(ch))
-                        RTPrintf("Invalid option -%c\n\n", ch);
-                    else
-                        RTPrintf("Invalid option case %i\n\n", ch);
-                }
-                else if (ch == VERR_GETOPT_UNKNOWN_OPTION)
-                    RTPrintf("Unknown option: %s\n\n", ValueUnion.psz);
-                else if (ValueUnion.pDef)
-                    RTPrintf("%s: %Rrs\n\n", ValueUnion.pDef->pszLong, ch);
-                else
-                    RTPrintf("Error: %Rrs\n\n", ch);
+                ch = RTGetOptPrintError(ch, &ValueUnion);
                 show_usage();
-                return -1;
+                return ch;
         }
     }
 
@@ -614,38 +807,38 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
     if (ulFrameWidth < 512 || ulFrameWidth > 2048 || ulFrameWidth % 2)
     {
         LogError("VBoxHeadless: ERROR: please specify an even frame width between 512 and 2048", 0);
-        return -1;
+        return 1;
     }
     if (ulFrameHeight < 384 || ulFrameHeight > 1536 || ulFrameHeight % 2)
     {
         LogError("VBoxHeadless: ERROR: please specify an even frame height between 384 and 1536", 0);
-        return -1;
+        return 1;
     }
     if (ulBitRate < 300000 || ulBitRate > 1000000)
     {
         LogError("VBoxHeadless: ERROR: please specify an even bitrate between 300000 and 1000000", 0);
-        return -1;
+        return 1;
     }
     /* Make sure we only have %d or %u (or none) in the file name specified */
     char *pcPercent = (char*)strchr(pszFileNameParam, '%');
     if (pcPercent != 0 && *(pcPercent + 1) != 'd' && *(pcPercent + 1) != 'u')
     {
         LogError("VBoxHeadless: ERROR: Only %%d and %%u are allowed in the capture file name.", -1);
-        return -1;
+        return 1;
     }
     /* And no more than one % in the name */
     if (pcPercent != 0 && strchr(pcPercent + 1, '%') != 0)
     {
         LogError("VBoxHeadless: ERROR: Only one format modifier is allowed in the capture file name.", -1);
-        return -1;
+        return 1;
     }
     RTStrPrintf(&pszMPEGFile[0], RTPATH_MAX, pszFileNameParam, RTProcSelf());
 #endif /* defined VBOX_FFMPEG */
 
-    if (!id && !name)
+    if (id.isEmpty() && !name)
     {
         show_usage();
-        return -1;
+        return 1;
     }
 
     HRESULT rc;
@@ -654,12 +847,13 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
     if (FAILED(rc))
     {
         RTPrintf("VBoxHeadless: ERROR: failed to initialize COM!\n");
-        return rc;
+        return 1;
     }
 
     ComPtr<IVirtualBox> virtualBox;
     ComPtr<ISession> session;
     bool fSessionOpened = false;
+    VirtualBoxCallback *vboxCallback = NULL;
 
     do
     {
@@ -687,38 +881,38 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
         }
 
         /* find ID by name */
-        if (!id)
+        if (id.isEmpty())
         {
             ComPtr <IMachine> m;
-            rc = virtualBox->FindMachine (Bstr (name), m.asOutParam());
-            if (FAILED (rc))
+            rc = virtualBox->FindMachine(Bstr(name), m.asOutParam());
+            if (FAILED(rc))
             {
-                LogError ("Invalid machine name!\n", rc);
+                LogError("Invalid machine name!\n", rc);
                 break;
             }
-            m->COMGETTER(Id) (id.asOutParam());
-            AssertComRC (rc);
-            if (FAILED (rc))
+            m->COMGETTER(Id)(id.asOutParam());
+            AssertComRC(rc);
+            if (FAILED(rc))
                 break;
         }
 
-        Log (("VBoxHeadless: Opening a session with machine (id={%s})...\n",
+        Log(("VBoxHeadless: Opening a session with machine (id={%s})...\n",
               Utf8Str(id).raw()));
 
         // open a session
-        CHECK_ERROR_BREAK(virtualBox, OpenSession (session, id));
+        CHECK_ERROR_BREAK(virtualBox, OpenSession(session, id));
         fSessionOpened = true;
 
         /* get the console */
         ComPtr <IConsole> console;
-        CHECK_ERROR_BREAK(session, COMGETTER (Console) (console.asOutParam()));
+        CHECK_ERROR_BREAK(session, COMGETTER(Console)(console.asOutParam()));
 
         /* get the machine */
         ComPtr <IMachine> machine;
-        CHECK_ERROR_BREAK(console, COMGETTER(Machine) (machine.asOutParam()));
+        CHECK_ERROR_BREAK(console, COMGETTER(Machine)(machine.asOutParam()));
 
         ComPtr <IDisplay> display;
-        CHECK_ERROR_BREAK(console, COMGETTER(Display) (display.asOutParam()));
+        CHECK_ERROR_BREAK(console, COMGETTER(Display)(display.asOutParam()));
 
 #ifdef VBOX_FFMPEG
         IFramebuffer *pFramebuffer = 0;
@@ -751,7 +945,7 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
                     LogError("Failed to initialise video capturing - make sure that the file format\n"
                              "you wish to use is supported on your system\n", rcc);
             }
-            if (RT_SUCCESS(rrc) && (S_OK == rcc))
+            if (RT_SUCCESS(rrc) && (rcc == S_OK))
             {
                 Log2(("VBoxHeadless: Registering framebuffer\n"));
                 pFramebuffer->AddRef();
@@ -765,7 +959,25 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
             break;
         }
 #endif /* defined(VBOX_FFMPEG) */
+#ifdef VBOX_WITH_VNC
+        if (fVNCEnable)
+        {
+            VNCFB *pFramebufferVNC = new VNCFB(console, uVNCPort, pszVNCPassword);
+            rc = pFramebufferVNC->init();
+            if (rc != S_OK)
+            {
+                LogError("Failed to load the vnc server extension, possibly due to a damaged file\n", rc);
+                delete pFramebufferVNC;
+                break;
+            }
 
+            Log2(("VBoxHeadless: Registering VNC framebuffer\n"));
+            pFramebufferVNC->AddRef();
+            display->SetFramebuffer(VBOX_VIDEO_PRIMARY_SCREEN, pFramebufferVNC);
+        }
+        if (rc != S_OK)
+            break;
+#endif
         ULONG cMonitors = 1;
         machine->COMGETTER(MonitorCount)(&cMonitors);
 
@@ -780,7 +992,7 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
                 continue;
             }
 #endif /* defined(VBOX_FFMPEG) */
-            VRDPFramebuffer *pVRDPFramebuffer = new VRDPFramebuffer ();
+            VRDPFramebuffer *pVRDPFramebuffer = new VRDPFramebuffer();
             if (!pVRDPFramebuffer)
             {
                 RTPrintf("Error: could not create framebuffer object %d\n", uScreenId);
@@ -847,11 +1059,11 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
 
         /* register a callback for machine events */
         {
-            ConsoleCallback *callback = new ConsoleCallback ();
+            ConsoleCallback *callback = new ConsoleCallback();
             callback->AddRef();
-            CHECK_ERROR(console, RegisterCallback (callback));
+            CHECK_ERROR(console, RegisterCallback(callback));
             callback->Release();
-            if (FAILED (rc))
+            if (FAILED(rc))
                 break;
         }
 
@@ -860,8 +1072,8 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
         BOOL fVRDPEnable = true;
         BOOL fVRDPEnabled;
         ComPtr <IVRDPServer> vrdpServer;
-        CHECK_ERROR_BREAK(machine, COMGETTER (VRDPServer) (vrdpServer.asOutParam()));
-        CHECK_ERROR_BREAK(vrdpServer, COMGETTER(Enabled) (&fVRDPEnabled));
+        CHECK_ERROR_BREAK(machine, COMGETTER(VRDPServer)(vrdpServer.asOutParam()));
+        CHECK_ERROR_BREAK(vrdpServer, COMGETTER(Enabled)(&fVRDPEnabled));
 
         if (vrdpEnabled != NULL)
         {
@@ -882,7 +1094,7 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
 
         if (fVRDPEnable)
         {
-            Log (("VBoxHeadless: Enabling VRDP server...\n"));
+            Log(("VBoxHeadless: Enabling VRDP server...\n"));
 
             /* set VRDP port if requested by the user */
             if (vrdpPort != NULL)
@@ -898,7 +1110,7 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
             /* enable VRDP server (only if currently disabled) */
             if (!fVRDPEnabled)
             {
-                CHECK_ERROR_BREAK(vrdpServer, COMSETTER(Enabled) (TRUE));
+                CHECK_ERROR_BREAK(vrdpServer, COMSETTER(Enabled)(TRUE));
             }
         }
         else
@@ -906,17 +1118,17 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
             /* disable VRDP server (only if currently enabled */
             if (fVRDPEnabled)
             {
-                CHECK_ERROR_BREAK(vrdpServer, COMSETTER(Enabled) (FALSE));
+                CHECK_ERROR_BREAK(vrdpServer, COMSETTER(Enabled)(FALSE));
             }
         }
 #endif
-        Log (("VBoxHeadless: Powering up the machine...\n"));
+        Log(("VBoxHeadless: Powering up the machine...\n"));
 
         ComPtr <IProgress> progress;
-        CHECK_ERROR_BREAK(console, PowerUp (progress.asOutParam()));
+        CHECK_ERROR_BREAK(console, PowerUp(progress.asOutParam()));
 
         /* wait for result because there can be errors */
-        if (SUCCEEDED(progress->WaitForCompletion (-1)))
+        if (SUCCEEDED(progress->WaitForCompletion(-1)))
         {
             LONG progressRc;
             progress->COMGETTER(ResultCode)(&progressRc);
@@ -935,24 +1147,32 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
             }
         }
 
+        /* VirtualBox callback registration. */
+        vboxCallback = new VirtualBoxCallback();
+        vboxCallback->AddRef();
+        CHECK_ERROR(virtualBox, RegisterCallback(vboxCallback));
+        vboxCallback->Release();
+        if (FAILED(rc))
+            break;
+
 #ifdef VBOX_WITH_SAVESTATE_ON_SIGNAL
         signal(SIGINT, SaveState);
         signal(SIGTERM, SaveState);
 #endif
 
-        Log (("VBoxHeadless: Waiting for PowerDown...\n"));
+        Log(("VBoxHeadless: Waiting for PowerDown...\n"));
 
         Event *e;
 
-        while (gEventQ->waitForEvent (&e) && e)
-          gEventQ->handleEvent (e);
+        while (gEventQ->waitForEvent(&e) && e)
+          gEventQ->handleEvent(e);
 
-        Log (("VBoxHeadless: event loop has terminated...\n"));
+        Log(("VBoxHeadless: event loop has terminated...\n"));
 
 #ifdef VBOX_FFMPEG
         if (pFramebuffer)
         {
-            pFramebuffer->Release ();
+            pFramebuffer->Release();
             Log(("Released framebuffer\n"));
             pFramebuffer = NULL;
         }
@@ -962,13 +1182,24 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
     }
     while (0);
 
+    /* VirtualBox callback unregistration. */
+    if (vboxCallback)
+    {
+        vboxCallback->AddRef();
+        CHECK_ERROR(virtualBox, UnregisterCallback(vboxCallback));
+        vboxCallback->Release();
+    }
+
+    /* No more access to the 'console' object, which will be uninitialized by the next session->Close call. */
+    gConsole = NULL;
+
     if (fSessionOpened)
     {
         /*
          * Close the session. This will also uninitialize the console and
          * unregister the callback we've registered before.
          */
-        Log (("VBoxHeadless: Closing the session...\n"));
+        Log(("VBoxHeadless: Closing the session...\n"));
         session->Close();
     }
 
@@ -978,9 +1209,9 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
 
     com::Shutdown();
 
-    LogFlow (("VBoxHeadless FINISHED.\n"));
+    LogFlow(("VBoxHeadless FINISHED.\n"));
 
-    return rc;
+    return FAILED(rc) ? 1 : 0;
 }
 
 
@@ -988,7 +1219,7 @@ extern "C" DECLEXPORT (int) TrustedMain (int argc, char **argv, char **envp)
 /**
  * Main entry point.
  */
-int main (int argc, char **argv, char **envp)
+int main(int argc, char **argv, char **envp)
 {
     // initialize VBox Runtime
     int rc = RTR3InitAndSUPLib();
@@ -1008,7 +1239,6 @@ int main (int argc, char **argv, char **envp)
         return 1;
     }
 
-    return TrustedMain (argc, argv, envp);
+    return TrustedMain(argc, argv, envp);
 }
 #endif /* !VBOX_WITH_HARDENING */
-
