@@ -1,3 +1,4 @@
+/* $Id: VBoxVMSettingsUSB.cpp $ */
 /** @file
  *
  * VBox frontends: Qt4 GUI ("VirtualBox"):
@@ -5,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "VBoxVMSettingsUSB.h"
@@ -132,6 +129,11 @@ VBoxVMSettingsUSB::VBoxVMSettingsUSB (FilterType aType)
     retranslateUi();
 }
 
+bool VBoxVMSettingsUSB::isOHCIEnabled() const
+{
+    return mGbUSB->isChecked();
+}
+
 void VBoxVMSettingsUSB::getFrom (const CSystemProperties &, const VBoxGlobalSettings &)
 {
     mGbUSB->setVisible (false);
@@ -228,6 +230,7 @@ void VBoxVMSettingsUSB::putBackTo()
 void VBoxVMSettingsUSB::setValidator (QIWidgetValidator *aVal)
 {
     mValidator = aVal;
+    connect (mGbUSB, SIGNAL (stateChanged (int)), mValidator, SLOT (revalidate()));
 }
 
 void VBoxVMSettingsUSB::setOrderAfter (QWidget *aWidget)
