@@ -1,3 +1,4 @@
+/* $Id: VBoxApplianceEditorWgt.cpp $ */
 /** @file
  *
  * VBox frontends: Qt4 GUI ("VirtualBox"):
@@ -5,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2009 Sun Microsystems, Inc.
+ * Copyright (C) 2009-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /* VBox includes */
@@ -191,6 +188,7 @@ QVariant HardwareItem::data (int aColumn, int aRole) const
                         case KVirtualSystemDescriptionType_HardDiskControllerIDE: v = VBoxApplianceEditorWgt::tr ("Hard Disk Controller (IDE)"); break;
                         case KVirtualSystemDescriptionType_HardDiskControllerSATA: v = VBoxApplianceEditorWgt::tr ("Hard Disk Controller (SATA)"); break;
                         case KVirtualSystemDescriptionType_HardDiskControllerSCSI: v = VBoxApplianceEditorWgt::tr ("Hard Disk Controller (SCSI)"); break;
+                        case KVirtualSystemDescriptionType_HardDiskControllerSAS: v = VBoxApplianceEditorWgt::tr ("Hard Disk Controller (SAS)"); break;
                         case KVirtualSystemDescriptionType_CDROM: v = VBoxApplianceEditorWgt::tr ("DVD"); break;
                         case KVirtualSystemDescriptionType_Floppy: v = VBoxApplianceEditorWgt::tr ("Floppy"); break;
                         case KVirtualSystemDescriptionType_NetworkAdapter: v = VBoxApplianceEditorWgt::tr ("Network Adapter"); break;
@@ -218,7 +216,7 @@ QVariant HardwareItem::data (int aColumn, int aRole) const
                                 v = tmp; break;
                             }
                         case KVirtualSystemDescriptionType_OS: v = vboxGlobal().vmGuestOSTypeDescription (mConfigValue); break;
-                        case KVirtualSystemDescriptionType_Memory: v = mConfigValue + " " + VBoxApplianceEditorWgt::tr ("MB"); break;
+                        case KVirtualSystemDescriptionType_Memory: v = mConfigValue + " " + VBoxApplianceEditorWgt::tr ("MB", "size suffix MBytes=1024 KBytes"); break;
                         case KVirtualSystemDescriptionType_SoundCard: v = vboxGlobal().toString (static_cast<KAudioControllerType> (mConfigValue.toInt())); break;
                         case KVirtualSystemDescriptionType_NetworkAdapter: v = vboxGlobal().toString (static_cast<KNetworkAdapterType> (mConfigValue.toInt())); break;
                         default: v = mConfigValue; break;
@@ -255,6 +253,7 @@ QVariant HardwareItem::data (int aColumn, int aRole) const
                         case KVirtualSystemDescriptionType_HardDiskControllerIDE: v = QIcon (":/ide_16px.png"); break;
                         case KVirtualSystemDescriptionType_HardDiskControllerSATA: v = QIcon (":/sata_16px.png"); break;
                         case KVirtualSystemDescriptionType_HardDiskControllerSCSI: v = QIcon (":/scsi_16px.png"); break;
+                        case KVirtualSystemDescriptionType_HardDiskControllerSAS: v = QIcon (":/scsi_16px.png"); break;
                         case KVirtualSystemDescriptionType_HardDiskImage: v = QIcon (":/hd_16px.png"); break;
                         case KVirtualSystemDescriptionType_CDROM: v = QIcon (":/cd_16px.png"); break;
                         case KVirtualSystemDescriptionType_Floppy: v = QIcon (":/fd_16px.png"); break;
@@ -397,7 +396,7 @@ QWidget * HardwareItem::createEditor (QWidget *aParent, const QStyleOptionViewIt
                 {
                     QSpinBox *e = new QSpinBox (aParent);
                     e->setRange (VBoxApplianceEditorWgt::minGuestRAM(), VBoxApplianceEditorWgt::maxGuestRAM());
-                    e->setSuffix (" " + VBoxApplianceEditorWgt::tr ("MB"));
+                    e->setSuffix (" " + VBoxApplianceEditorWgt::tr ("MB", "size suffix MBytes=1024 KBytes"));
                     editor = e;
                     break;
                 }
@@ -669,7 +668,8 @@ VirtualSystemModel::VirtualSystemModel (QVector<CVirtualSystemDescription>& aVSD
                 /* Save the hard disk controller types in an extra map */
                 if (types[i] == KVirtualSystemDescriptionType_HardDiskControllerIDE ||
                     types[i] == KVirtualSystemDescriptionType_HardDiskControllerSATA ||
-                    types[i] == KVirtualSystemDescriptionType_HardDiskControllerSCSI)
+                    types[i] == KVirtualSystemDescriptionType_HardDiskControllerSCSI ||
+                    types[i] == KVirtualSystemDescriptionType_HardDiskControllerSAS)
                     controllerMap[i] = hi;
             }
         }
@@ -942,7 +942,8 @@ KVirtualSystemDescriptionType VirtualSystemSortProxyModel::mSortList[] =
     KVirtualSystemDescriptionType_NetworkAdapter,
     KVirtualSystemDescriptionType_HardDiskControllerIDE,
     KVirtualSystemDescriptionType_HardDiskControllerSATA,
-    KVirtualSystemDescriptionType_HardDiskControllerSCSI
+    KVirtualSystemDescriptionType_HardDiskControllerSCSI,
+    KVirtualSystemDescriptionType_HardDiskControllerSAS
 };
 
 VirtualSystemSortProxyModel::VirtualSystemSortProxyModel (QObject *aParent)
