@@ -30,9 +30,12 @@
 #define CR_OPENGL_VERSION_1_4 1
 #define CR_OPENGL_VERSION_1_5 1
 #define CR_OPENGL_VERSION_2_0 1
+#define CR_OPENGL_VERSION_2_1 1
 
 /* Version (string) of OpenGL functionality suported by Chromium */
-#ifdef CR_OPENGL_VERSION_2_0
+#ifdef CR_OPENGL_VERSION_2_1
+# define CR_OPENGL_VERSION_STRING "2.1"
+#elif defined(CR_OPENGL_VERSION_2_0)
 # define CR_OPENGL_VERSION_STRING "2.0"
 #else
 # define CR_OPENGL_VERSION_STRING "1.5"
@@ -121,4 +124,7 @@
 #define CR_EXT_framebuffer_object 1
 #define CR_EXT_compiled_vertex_array 1
 
+#define CR_ARB_pixel_buffer_object 1
+#define CR_EXT_texture_sRGB 1
+
 #endif /* CR_VERSION_H */
