@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -262,21 +258,25 @@ static int supSvcWinDelete(int argc, char **argv)
      * Parse the arguments.
      */
     bool fVerbose = false;
-    static const RTOPTIONDEF s_aOptions[] =
+    static const RTGETOPTDEF s_aOptions[] =
     {
         { "--verbose", 'v', RTGETOPT_REQ_NOTHING }
     };
-    int iArg = 0;
     int ch;
-    RTOPTIONUNION Value;
-    while ((ch = RTGetOpt(argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), &iArg, &Value)))
+    RTGETOPTUNION Value;
+    RTGETOPTSTATE GetState;
+    RTGetOptInit(&GetState, argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
+    while ((ch = RTGetOpt(&GetState, &Value)))
         switch (ch)
         {
-            case 'v':   fVerbose = true;  break;
-            default:    return supSvcDisplayGetOptError("delete", ch, argc, argv, iArg, &Value);
+            case 'v':
+                fVerbose = true;
+                break;
+            case VINF_GETOPT_NOT_OPTION:
+                return supSvcDisplayTooManyArgsError("delete", argc, argv, iArg);
+            default:
+                return supSvcDisplayGetOptError("delete", ch, argc, argv, iArg, &Value);
         }
-    if (iArg != argc)
-        return supSvcDisplayTooManyArgsError("delete", argc, argv, iArg);
 
     /*
      * Create the service.
@@ -327,7 +327,7 @@ static int supSvcWinCreate(int argc, char **argv)
     };
     int iArg = 0;
     int ch;
-    RTOPTIONUNION Value;
+    RTGETOPTUNION Value;
     while ((ch = RTGetOpt(argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), &iArg, &Value)))
         switch (ch)
         {
@@ -547,7 +547,7 @@ static VOID WINAPI supSvcWinServiceMain(DWORD cArgs, LPSTR *papszArgs)
             int iArg = 1; /* the first arg is the service name */
             int ch;
             int rc = 0;
-            RTOPTIONUNION Value;
+            RTGETOPTUNION Value;
             while (   !rc
                    && (ch = RTGetOpt(cArgs, papszArgs, s_aOptions, RT_ELEMENTS(s_aOptions), &iArg, &Value)))
                 switch (ch)
@@ -645,7 +645,7 @@ static int supSvcWinRunIt(int argc, char **argv)
     };
     int iArg = 0;
     int ch;
-    RTOPTIONUNION Value;
+    RTGETOPTUNION Value;
     while ((ch = RTGetOpt(argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), &iArg, &Value)))
         switch (ch)
         {
@@ -700,7 +700,7 @@ static int supSvcWinShowVersion(int argc, char **argv)
     };
     int iArg = 0;
     int ch;
-    RTOPTIONUNION Value;
+    RTGETOPTUNION Value;
     while ((ch = RTGetOpt(argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), &iArg, &Value)))
         switch (ch)
         {
@@ -718,7 +718,7 @@ static int supSvcWinShowVersion(int argc, char **argv)
         RTPrintf("%s\n", VBOX_VERSION_STRING);
     else
         RTPrintf("VirtualBox System Service Version %s\n"
-                 "(C) 2008 Sun Microsystems, Inc.\n"
+                 "(C) 2008-2010 Oracle Corporation\n"
                  "All rights reserved.\n",
                  VBOX_VERSION_STRING);
     return 0;
@@ -733,7 +733,7 @@ static int supSvcWinShowVersion(int argc, char **argv)
 static int supSvcWinShowHelp(void)
 {
     RTPrintf("VirtualBox System Service Version %s\n"
-             "(C) 2008 Sun Microsystems, Inc.\n"
+             "(C) 2008-2010 Oracle Corporation\n"
              "All rights reserved.\n"
              "\n",
              VBOX_VERSION_STRING);
