@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 /*
  * Based in part on Microsoft DDK sample code for Ndis Intermediate Miniport passthru driver sample.
@@ -33,10 +29,8 @@
 /** global lock */
 extern NDIS_SPIN_LOCK     g_GlobalLock;
 
-#ifdef VBOX_LOOPBACK_USEFLAGS
 extern UINT g_fPacketDontLoopBack;
 extern UINT g_fPacketIsLoopedBack;
-#endif
 
 /*
  * Debug Print API
@@ -51,6 +45,11 @@ extern UINT g_fPacketIsLoopedBack;
 
 #endif /* if DBG */
 
+/** get the PVBOXNETFLTINS from PADAPT */
+#define PADAPT_2_PVBOXNETFLTINS(_pAdapt)  ( (PVBOXNETFLTINS)((uint8_t *)(_pAdapt) - RT_OFFSETOF(VBOXNETFLTINS, u.s.IfAdaptor)) )
+/** get the PADAPT from PVBOXNETFLTINS */
+#define PVBOXNETFLTINS_2_PADAPT(_pNetFlt) ( &(_pNetFlt)->u.s.IfAdaptor )
+
 
 DECLHIDDEN(NTSTATUS) vboxNetFltWinPtDispatch(IN PDEVICE_OBJECT pDeviceObject, IN PIRP pIrp);
 DECLHIDDEN(VOID) vboxNetFltWinUnload(IN PDRIVER_OBJECT DriverObject);
@@ -210,6 +209,9 @@ DECLINLINE(void) vboxNetFltWinPpFreePacketInfo(PPACKET_INFO pInfo)
 /** sets flags to the packet info */
 #define SET_FLAGS_TO_INFO(_pPacketInfo, _fFlags) (ASMAtomicUoWriteU32((volatile uint32_t *)&(_pPacketInfo)->fFlags, (_fFlags)))
 
+#ifdef VBOXNETFLT_NO_PACKET_QUEUE
+DECLHIDDEN(bool) vboxNetFltWinPostIntnet(PVBOXNETFLTINS pInstance, PVOID pvPacket, const UINT fFlags);
+#else
 DECLHIDDEN(NDIS_STATUS) vboxNetFltWinQuEnqueuePacket(PVBOXNETFLTINS pInstance, PVOID pPacket, const UINT fPacketFlags);
 
 #ifndef VBOX_NETFLT_ONDEMAND_BIND
@@ -218,6 +220,9 @@ DECLHIDDEN(void) vboxNetFltWinQuFiniPacketQueue(PVBOXNETFLTINS pInstance);
 DECLHIDDEN(NTSTATUS) vboxNetFltWinQuInitPacketQueue(PVBOXNETFLTINS pInstance);
 #endif
 
+#endif /* #ifndef VBOXNETFLT_NO_PACKET_QUEUE */
+
+
 #ifndef VBOXNETADP
 /**
  * searches the list entry in a single-linked list
@@ -410,7 +415,12 @@ DECLINLINE(PNDIS_PACKET) vboxNetFltWinLbSearchLoopBackBySG(PADAPT pAdapt, PINTNE
 DECLINLINE(bool) vboxNetFltWinLbRemoveSendPacket(PADAPT pAdapt, PNDIS_PACKET pPacket)
 {
     PSEND_RSVD pSrv = (PSEND_RSVD)pPacket->ProtocolReserved;
-    return vboxNetFltWinInterlockedSearchListEntry(&pAdapt->SendPacketQueue, &pSrv->ListEntry, true);
+    bool bRet = vboxNetFltWinInterlockedSearchListEntry(&pAdapt->SendPacketQueue, &pSrv->ListEntry, true);
+#ifdef DEBUG_misha
+    PVBOXNETFLTINS pNetFlt = PADAPT_2_PVBOXNETFLTINS(pAdapt);
+    Assert(bRet == (pNetFlt->enmTrunkState == INTNETTRUNKIFSTATE_ACTIVE));
+#endif
+    return bRet;
 }
 
 # endif
@@ -426,13 +436,13 @@ extern RTMAC g_vboxNetFltWinVerifyMACGuest;
 # define VBOXNETFLT_LBVERIFY(_pnf, _p) \
     do { \
         Assert(!vboxNetFltWinCheckMACs(_p, NULL, &g_vboxNetFltWinVerifyMACGuest)); \
-        Assert(!vboxNetFltWinCheckMACs(_p, NULL, &(_pnf)->u.s.Mac)); \
+        Assert(!vboxNetFltWinCheckMACs(_p, NULL, &(_pnf)->u.s.MacAddr)); \
     } while(0)
 
 # define VBOXNETFLT_LBVERIFYSG(_pnf, _p) \
     do { \
         Assert(!vboxNetFltWinCheckMACsSG(_p, NULL, &g_vboxNetFltWinVerifyMACGuest)); \
-        Assert(!vboxNetFltWinCheckMACsSG(_p, NULL, &(_pnf)->u.s.Mac)); \
+        Assert(!vboxNetFltWinCheckMACsSG(_p, NULL, &(_pnf)->u.s.MacAddr)); \
     } while(0)
 
 #else
@@ -474,53 +484,48 @@ extern RTMAC g_vboxNetFltWinVerifyMACGuest;
  * PADAPT, PVBOXNETFLTINS reference/dereference (i.e. retain/release) API *
  **************************************************************************/
 
-/** get the PVBOXNETFLTINS from PADAPT */
-#define PADAPT_2_PVBOXNETFLTINS(_pAdapt)  ( (PVBOXNETFLTINS)((uint8_t *)(_pAdapt) - RT_OFFSETOF(VBOXNETFLTINS, u.s.IfAdaptor)) )
-/** get the PADAPT from PVBOXNETFLTINS */
-#define PVBOXNETFLTINS_2_PADAPT(_pNetFlt) ( &(_pNetFlt)->u.s.IfAdaptor )
-
 DECLHIDDEN(void) vboxNetFltWinWaitDereference(PADAPT_DEVICE pState);
 
 DECLINLINE(void) vboxNetFltWinReferenceModeNetFlt(PVBOXNETFLTINS pIns)
 {
-	ASMAtomicIncU32((volatile uint32_t *)&pIns->u.s.cModeNetFltRefs);
+    ASMAtomicIncU32((volatile uint32_t *)&pIns->u.s.cModeNetFltRefs);
 }
 
 DECLINLINE(void) vboxNetFltWinReferenceModePassThru(PVBOXNETFLTINS pIns)
 {
-	ASMAtomicIncU32((volatile uint32_t *)&pIns->u.s.cModePassThruRefs);
+    ASMAtomicIncU32((volatile uint32_t *)&pIns->u.s.cModePassThruRefs);
 }
 
 DECLINLINE(void) vboxNetFltWinIncReferenceModeNetFlt(PVBOXNETFLTINS pIns, uint32_t v)
 {
-	ASMAtomicAddU32((volatile uint32_t *)&pIns->u.s.cModeNetFltRefs, v);
+    ASMAtomicAddU32((volatile uint32_t *)&pIns->u.s.cModeNetFltRefs, v);
 }
 
 DECLINLINE(void) vboxNetFltWinIncReferenceModePassThru(PVBOXNETFLTINS pIns, uint32_t v)
 {
-	ASMAtomicAddU32((volatile uint32_t *)&pIns->u.s.cModePassThruRefs, v);
+    ASMAtomicAddU32((volatile uint32_t *)&pIns->u.s.cModePassThruRefs, v);
 }
 
 DECLINLINE(void) vboxNetFltWinDereferenceModeNetFlt(PVBOXNETFLTINS pIns)
 {
-	ASMAtomicDecU32((volatile uint32_t *)&pIns->u.s.cModeNetFltRefs);
+    ASMAtomicDecU32((volatile uint32_t *)&pIns->u.s.cModeNetFltRefs);
 }
 
 DECLINLINE(void) vboxNetFltWinDereferenceModePassThru(PVBOXNETFLTINS pIns)
 {
-	ASMAtomicDecU32((volatile uint32_t *)&pIns->u.s.cModePassThruRefs);
+    ASMAtomicDecU32((volatile uint32_t *)&pIns->u.s.cModePassThruRefs);
 }
 
 DECLINLINE(void) vboxNetFltWinDecReferenceModeNetFlt(PVBOXNETFLTINS pIns, uint32_t v)
 {
-	Assert(v);
-	ASMAtomicAddU32((volatile uint32_t *)&pIns->u.s.cModeNetFltRefs, (uint32_t)(-((int32_t)v)));
+    Assert(v);
+    ASMAtomicAddU32((volatile uint32_t *)&pIns->u.s.cModeNetFltRefs, (uint32_t)(-((int32_t)v)));
 }
 
 DECLINLINE(void) vboxNetFltWinDecReferenceModePassThru(PVBOXNETFLTINS pIns, uint32_t v)
 {
-	Assert(v);
-	ASMAtomicAddU32((volatile uint32_t *)&pIns->u.s.cModePassThruRefs, (uint32_t)(-((int32_t)v)));
+    Assert(v);
+    ASMAtomicAddU32((volatile uint32_t *)&pIns->u.s.cModePassThruRefs, (uint32_t)(-((int32_t)v)));
 }
 
 DECLINLINE(void) vboxNetFltWinSetPowerState(PADAPT_DEVICE pState, NDIS_DEVICE_POWER_STATE State)
@@ -592,7 +597,7 @@ DECLINLINE(void) vboxNetFltWinDereferenceDevices(PADAPT pAdapt, PADAPT_DEVICE pS
 
 DECLINLINE(void) vboxNetFltWinDecReferenceDevice(PADAPT pAdapt, PADAPT_DEVICE pState, uint32_t v)
 {
-	Assert(v);
+    Assert(v);
     ASMAtomicAddU32((uint32_t volatile *)&pState->cReferences, (uint32_t)(-((int32_t)v)));
 }
 
@@ -606,7 +611,7 @@ DECLINLINE(void) vboxNetFltWinDecReferenceDevices(PADAPT pAdapt, PADAPT_DEVICE p
 
 DECLINLINE(bool) vboxNetFltWinDoIncReferenceDevice(PADAPT pAdapt, PADAPT_DEVICE pState, uint32_t v)
 {
-	Assert(v);
+    Assert(v);
     if (vboxNetFltWinGetPowerState(pState) == NdisDeviceStateD0 && vboxNetFltWinGetOpState(pState) == kVBoxNetDevOpState_Initialized)
     {
         ASMAtomicAddU32((uint32_t volatile *)&pState->cReferences, v);
@@ -639,22 +644,22 @@ DECLINLINE(PVBOXNETFLTINS) vboxNetFltWinReferenceAdaptNetFltFromAdapt(PADAPT pAd
 
     pNetFlt = PADAPT_2_PVBOXNETFLTINS(pAdapt);
 
-    RTSpinlockAcquire((pNetFlt)->hSpinlock, &Tmp);
-    if(!ASMAtomicUoReadBool(&(pNetFlt)->fActive))
+    RTSpinlockAcquireNoInts((pNetFlt)->hSpinlock, &Tmp);
+    if(pNetFlt->enmTrunkState != INTNETTRUNKIFSTATE_ACTIVE)
     {
-        RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
         return NULL;
     }
 
     if(!vboxNetFltWinDoReferenceDevice(pAdapt, &pAdapt->PTState))
     {
-        RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
         return NULL;
     }
 
     vboxNetFltRetain((pNetFlt), true /* fBusy */);
 
-    RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+    RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
 
     return pNetFlt;
 }
@@ -663,29 +668,29 @@ DECLINLINE(bool) vboxNetFltWinReferenceAdaptNetFlt(PVBOXNETFLTINS pNetFlt, PADAP
 {
     RTSPINLOCKTMP Tmp = RTSPINLOCKTMP_INITIALIZER;
 
-    RTSpinlockAcquire((pNetFlt)->hSpinlock, &Tmp);
+    RTSpinlockAcquireNoInts((pNetFlt)->hSpinlock, &Tmp);
 #ifndef VBOXNETADP
     if(!vboxNetFltWinDoReferenceDevices(pAdapt, &pAdapt->MPState, &pAdapt->PTState))
 #else
     if(!vboxNetFltWinDoReferenceDevice(pAdapt, &pAdapt->MPState))
 #endif
     {
-        RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
         *pbNetFltActive = false;
         return false;
     }
 
-    if(!ASMAtomicUoReadBool(&(pNetFlt)->fActive))
+    if(pNetFlt->enmTrunkState != INTNETTRUNKIFSTATE_ACTIVE)
     {
-    	vboxNetFltWinReferenceModePassThru(pNetFlt);
-        RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+        vboxNetFltWinReferenceModePassThru(pNetFlt);
+        RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
         *pbNetFltActive = false;
         return true;
     }
 
     vboxNetFltRetain((pNetFlt), true /* fBusy */);
-	vboxNetFltWinReferenceModeNetFlt(pNetFlt);
-    RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+    vboxNetFltWinReferenceModeNetFlt(pNetFlt);
+    RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
 
     *pbNetFltActive = true;
     return true;
@@ -707,22 +712,22 @@ DECLINLINE(PVBOXNETFLTINS) vboxNetFltWinIncReferenceAdaptNetFltFromAdapt(PADAPT
 
     pNetFlt = PADAPT_2_PVBOXNETFLTINS(pAdapt);
 
-    RTSpinlockAcquire((pNetFlt)->hSpinlock, &Tmp);
-    if(!ASMAtomicUoReadBool(&(pNetFlt)->fActive))
+    RTSpinlockAcquireNoInts((pNetFlt)->hSpinlock, &Tmp);
+    if(pNetFlt->enmTrunkState != INTNETTRUNKIFSTATE_ACTIVE)
     {
-        RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
         return NULL;
     }
 
     if(!vboxNetFltWinDoIncReferenceDevice(pAdapt, &pAdapt->PTState, v))
     {
-        RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
         return NULL;
     }
 
     vboxNetFltRetain((pNetFlt), true /* fBusy */);
 
-    RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+    RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
 
     /* we have marked it as busy, so can do the res references outside the lock */
     for(i = 0; i < v-1; i++)
@@ -745,32 +750,32 @@ DECLINLINE(bool) vboxNetFltWinIncReferenceAdaptNetFlt(PVBOXNETFLTINS pNetFlt, PA
         return false;
     }
 
-    RTSpinlockAcquire((pNetFlt)->hSpinlock, &Tmp);
+    RTSpinlockAcquireNoInts((pNetFlt)->hSpinlock, &Tmp);
 #ifndef VBOXNETADP
     if(!vboxNetFltWinDoIncReferenceDevices(pAdapt, &pAdapt->MPState, &pAdapt->PTState, v))
 #else
     if(!vboxNetFltWinDoIncReferenceDevice(pAdapt, &pAdapt->MPState, v))
 #endif
     {
-        RTSpinlockRelease(pNetFlt->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts(pNetFlt->hSpinlock, &Tmp);
         *pbNetFltActive = false;
         return false;
     }
 
-    if(!ASMAtomicUoReadBool(&(pNetFlt)->fActive))
+    if(pNetFlt->enmTrunkState != INTNETTRUNKIFSTATE_ACTIVE)
     {
-    	vboxNetFltWinIncReferenceModePassThru(pNetFlt, v);
+        vboxNetFltWinIncReferenceModePassThru(pNetFlt, v);
 
-        RTSpinlockRelease((pNetFlt)->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts((pNetFlt)->hSpinlock, &Tmp);
         *pbNetFltActive = false;
         return true;
     }
 
     vboxNetFltRetain(pNetFlt, true /* fBusy */);
 
-	vboxNetFltWinIncReferenceModeNetFlt(pNetFlt, v);
+    vboxNetFltWinIncReferenceModeNetFlt(pNetFlt, v);
 
-    RTSpinlockRelease(pNetFlt->hSpinlock, &Tmp);
+    RTSpinlockReleaseNoInts(pNetFlt->hSpinlock, &Tmp);
 
     /* we have marked it as busy, so can do the res references outside the lock */
     for(i = 0; i < v-1; i++)
@@ -793,14 +798,14 @@ DECLINLINE(void) vboxNetFltWinDecReferenceNetFlt(PVBOXNETFLTINS pNetFlt, uint32_
         vboxNetFltRelease(pNetFlt, true);
     }
 
-	vboxNetFltWinDecReferenceModeNetFlt(pNetFlt, n);
+    vboxNetFltWinDecReferenceModeNetFlt(pNetFlt, n);
 }
 
 DECLINLINE(void) vboxNetFltWinDereferenceNetFlt(PVBOXNETFLTINS pNetFlt)
 {
     vboxNetFltRelease(pNetFlt, true);
 
-	vboxNetFltWinDereferenceModeNetFlt(pNetFlt);
+    vboxNetFltWinDereferenceModeNetFlt(pNetFlt);
 }
 
 DECLINLINE(void) vboxNetFltWinDecReferenceAdapt(PADAPT pAdapt, uint32_t v)
@@ -836,7 +841,7 @@ DECLINLINE(bool) vboxNetFltWinIncReferenceAdapt(PADAPT pAdapt, uint32_t v)
         return false;
     }
 
-    RTSpinlockAcquire(pNetFlt->hSpinlock, &Tmp);
+    RTSpinlockAcquireNoInts(pNetFlt->hSpinlock, &Tmp);
 #ifdef VBOX_NETFLT_ONDEMAND_BIND
     if(vboxNetFltWinDoIncReferenceDevice(pAdapt, &pAdapt->PTState))
 #elif defined(VBOXNETADP)
@@ -845,11 +850,11 @@ DECLINLINE(bool) vboxNetFltWinIncReferenceAdapt(PADAPT pAdapt, uint32_t v)
     if(vboxNetFltWinDoIncReferenceDevices(pAdapt, &pAdapt->MPState, &pAdapt->PTState, v))
 #endif
     {
-        RTSpinlockRelease(pNetFlt->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts(pNetFlt->hSpinlock, &Tmp);
         return true;
     }
 
-    RTSpinlockRelease(pNetFlt->hSpinlock, &Tmp);
+    RTSpinlockReleaseNoInts(pNetFlt->hSpinlock, &Tmp);
     return false;
 }
 
@@ -857,7 +862,7 @@ DECLINLINE(bool) vboxNetFltWinReferenceAdapt(PADAPT pAdapt)
 {
     PVBOXNETFLTINS pNetFlt = PADAPT_2_PVBOXNETFLTINS(pAdapt);
     RTSPINLOCKTMP Tmp = RTSPINLOCKTMP_INITIALIZER;
-    RTSpinlockAcquire(pNetFlt->hSpinlock, &Tmp);
+    RTSpinlockAcquireNoInts(pNetFlt->hSpinlock, &Tmp);
 #ifdef VBOX_NETFLT_ONDEMAND_BIND
     if(vboxNetFltWinDoReferenceDevice(pAdapt, &pAdapt->PTState))
 #elif defined(VBOXNETADP)
@@ -866,11 +871,11 @@ DECLINLINE(bool) vboxNetFltWinReferenceAdapt(PADAPT pAdapt)
     if(vboxNetFltWinDoReferenceDevices(pAdapt, &pAdapt->MPState, &pAdapt->PTState))
 #endif
     {
-        RTSpinlockRelease(pNetFlt->hSpinlock, &Tmp);
+        RTSpinlockReleaseNoInts(pNetFlt->hSpinlock, &Tmp);
         return true;
     }
 
-    RTSpinlockRelease(pNetFlt->hSpinlock, &Tmp);
+    RTSpinlockReleaseNoInts(pNetFlt->hSpinlock, &Tmp);
     return false;
 }
 
