@@ -5,7 +5,7 @@
 #
 
 #
-# Copyright (C) 2007-2009 Sun Microsystems, Inc.
+# Copyright (C) 2007-2010 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
 # VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 # hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 #
-# Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
-# Clara, CA 95054 USA or visit http://www.sun.com if you need
-# additional information or have any questions.
-#
 
 #
 # Usage:
@@ -142,10 +138,13 @@ find . -type d | $VBOX_GGREP -E 'opt/VirtualBox' | pkgproto >> prototype
 # don't grok for class-specific files (like sed, if any)
 filelist_fixup prototype '$2 == "none"'                                                                '$5 = "root"; $6 = "bin"'
 
-# don't include autoresponse from the base folder into / of the package, it goes into .tar.gz
-# and another one already exists in /opt/VirtualBox
+# don't include autoresponse and LICENSE from the base folder into / of
+# the package, it goes into .tar.gz and another one already exists
+# in /opt/VirtualBox
 sed '/f none autoresponse/d' prototype > prototype2
 mv -f prototype2 prototype
+sed '/f none LICENSE/d' prototype > prototype2
+mv -f prototype2 prototype
 
 # HostDriver vboxdrv
 filelist_fixup prototype '$3 == "platform/i86pc/kernel/drv/vboxdrv"'                                   '$6 = "sys"'
@@ -211,10 +210,13 @@ pkgmk -p $VBOXPKG_TIMESTAMP -o -r .
 pkgtrans -s -o /var/spool/pkg "`pwd`/$VBOX_PKGFILE" "$VBOX_PKGNAME"
 
 # $5 if exist would contain the path to the VBI package to include in the .tar.gz
+if [ -f LICENSE ]; then
+    VBOX_LICENSEFILE=LICENSE
+fi
 if test -f "$5"; then
-    $VBOX_GTAR zcvf "$VBOX_ARCHIVE" "$VBOX_PKGFILE" "$5" autoresponse ReadMe.txt
+    $VBOX_GTAR zcvf "$VBOX_ARCHIVE" $VBOX_LICENSEFILE "$VBOX_PKGFILE" "$5" autoresponse ReadMe.txt
 else
-    $VBOX_GTAR zcvf "$VBOX_ARCHIVE" "$VBOX_PKGFILE" autoresponse ReadMe.txt
+    $VBOX_GTAR zcvf "$VBOX_ARCHIVE" $VBOX_LICENSEFILE "$VBOX_PKGFILE" autoresponse ReadMe.txt
 fi
 
 echo "## Packaging and transfer completed successfully!"
