@@ -4,7 +4,7 @@
 # Sun VirtualBox
 # VirtualBox Configuration Script, Solaris host.
 #
-# Copyright (C) 2009 Sun Microsystems, Inc.
+# Copyright (C) 2009 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +14,6 @@
 # VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 # hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 #
-# Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
-# Clara, CA 95054 USA or visit http://www.sun.com if you need
-# additional information or have any questions.
-#
 
 
 # Never use exit 2 or exit 20 etc., the return codes are used in
@@ -51,6 +47,7 @@ DESC_VBOXDRV="Host"
 
 MOD_VBOXNET=vboxnet
 DESC_VBOXNET="NetAdapter"
+MOD_VBOXNET_INST=32
 
 MOD_VBOXFLT=vboxflt
 DESC_VBOXFLT="NetFilter"
@@ -545,29 +542,34 @@ cleanup_install()
         fi
     fi
 
-    # unplumb vboxnet0
-    vboxnetup=`$BIN_IFCONFIG vboxnet0 >/dev/null 2>&1`
-    if test "$?" -eq 0; then
-        $BIN_IFCONFIG vboxnet0 unplumb
-        if test "$?" -ne 0; then
-            errorprint "VirtualBox NetAdapter 'vboxnet0' couldn't be unplumbed (probably in use)."
-            if test "$fatal" = "$FATALOP"; then
-                exit 1
+    # unplumb all vboxnet instances
+    inst=0
+    while test $inst -ne $MOD_VBOXNET_INST; do
+        vboxnetup=`$BIN_IFCONFIG vboxnet$inst >/dev/null 2>&1`
+        if test "$?" -eq 0; then
+            $BIN_IFCONFIG vboxnet$inst unplumb
+            if test "$?" -ne 0; then
+                errorprint "VirtualBox NetAdapter 'vboxnet$inst' couldn't be unplumbed (probably in use)."
+                if test "$fatal" = "$FATALOP"; then
+                    exit 1
+                fi
             fi
         fi
-    fi
 
-    # unplumb vboxnet0 ipv6
-    vboxnetup=`$BIN_IFCONFIG vboxnet0 inet6 >/dev/null 2>&1`
-    if test "$?" -eq 0; then
-        $BIN_IFCONFIG vboxnet0 inet6 unplumb
-        if test "$?" -ne 0; then
-            errorprint "VirtualBox NetAdapter 'vboxnet0' IPv6 couldn't be unplumbed (probably in use)."
-            if test "$fatal" = "$FATALOP"; then
-                exit 1
+        # unplumb vboxnet0 ipv6
+        vboxnetup=`$BIN_IFCONFIG vboxnet$inst inet6 >/dev/null 2>&1`
+        if test "$?" -eq 0; then
+            $BIN_IFCONFIG vboxnet$inst inet6 unplumb
+            if test "$?" -ne 0; then
+                errorprint "VirtualBox NetAdapter 'vboxnet$inst' IPv6 couldn't be unplumbed (probably in use)."
+                if test "$fatal" = "$FATALOP"; then
+                    exit 1
+                fi
             fi
         fi
-    fi
+
+        inst=`expr $inst + 1`
+    done
 }
 
 
@@ -585,7 +587,15 @@ postinstall()
             nwambackupfile=$nwamfile.vbox
             if test -f "$nwamfile"; then
                 sed -e '/vboxnet/d' $nwamfile > $nwambackupfile
-                echo "vboxnet0	static 192.168.56.1" >> $nwambackupfile
+
+                # add all vboxnet instances as static to nwam
+                inst=0
+                networkn=56
+                while test $inst -ne 1; do
+                    echo "vboxnet$inst	static 192.168.$networkn.1" >> $nwambackupfile
+                    inst=`expr $inst + 1`
+                    networkn=`expr $networkn + 1`
+                done
                 mv -f $nwambackupfile $nwamfile
             fi
 
@@ -683,7 +693,7 @@ postinstall()
 
         return 0
     else
-        errorprint "Failed to update boot-archive"
+        errorprint "Failed to install drivers"
         exit 666
     fi
     return 1
