@@ -1,12 +1,10 @@
 /* $Id: ConsoleVRDPServer.cpp $ */
-
 /** @file
- *
  * VBox Console VRDP Helper class
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "ConsoleVRDPServer.h"
@@ -27,6 +21,7 @@
 #include "KeyboardImpl.h"
 #include "MouseImpl.h"
 
+#include "AutoCaller.h"
 #include "Logging.h"
 
 #include <iprt/asm.h>
@@ -44,8 +39,8 @@ class VRDPConsoleCallback :
     VBOX_SCRIPTABLE_IMPL(IConsoleCallback)
 {
 public:
-    VRDPConsoleCallback (ConsoleVRDPServer *server) :
-        m_server(server)
+    VRDPConsoleCallback(ConsoleVRDPServer *server)
+        : m_server(server)
     {
 #ifndef VBOX_WITH_XPCOM
         refcnt = 0;
@@ -58,16 +53,16 @@ public:
 
 #ifndef VBOX_WITH_XPCOM
     STDMETHOD_(ULONG, AddRef)() {
-        return ::InterlockedIncrement (&refcnt);
+        return ::InterlockedIncrement(&refcnt);
     }
     STDMETHOD_(ULONG, Release)()
     {
-        long cnt = ::InterlockedDecrement (&refcnt);
+        long cnt = ::InterlockedDecrement(&refcnt);
         if (cnt == 0)
             delete this;
         return cnt;
     }
-    STDMETHOD(QueryInterface) (REFIID riid , void **ppObj)
+    STDMETHOD(QueryInterface)(REFIID riid , void **ppObj)
     {
         if (riid == IID_IUnknown) {
             *ppObj = this;
@@ -86,9 +81,9 @@ public:
 
 
     STDMETHOD(OnMousePointerShapeChange)(BOOL visible, BOOL alpha, ULONG xHot, ULONG yHot,
-                                         ULONG width, ULONG height, BYTE *shape);
+                                         ULONG width, ULONG height, ComSafeArrayIn(BYTE,shape));
 
-    STDMETHOD(OnMouseCapabilityChange)(BOOL supportsAbsolute, BOOL needsHostCursor)
+    STDMETHOD(OnMouseCapabilityChange)(BOOL supportsAbsolute, BOOL supportsRelative, BOOL needsHostCursor)
     {
         if (m_server)
         {
@@ -101,7 +96,7 @@ public:
     {
         if (m_server)
         {
-            m_server->NotifyKeyboardLedsChange (fNumLock, fCapsLock, fScrollLock);
+            m_server->NotifyKeyboardLedsChange(fNumLock, fCapsLock, fScrollLock);
         }
         return S_OK;
     }
@@ -121,22 +116,27 @@ public:
         return S_OK;
     }
 
-    STDMETHOD(OnNetworkAdapterChange) (INetworkAdapter *aNetworkAdapter)
+    STDMETHOD(OnCPUChange)(ULONG aCPU, BOOL aRemove)
     {
         return S_OK;
     }
 
-    STDMETHOD(OnSerialPortChange) (ISerialPort *aSerialPort)
+    STDMETHOD(OnNetworkAdapterChange)(INetworkAdapter *aNetworkAdapter)
     {
         return S_OK;
     }
 
-    STDMETHOD(OnParallelPortChange) (IParallelPort *aParallelPort)
+    STDMETHOD(OnSerialPortChange)(ISerialPort *aSerialPort)
     {
         return S_OK;
     }
 
-    STDMETHOD(OnStorageControllerChange) ()
+    STDMETHOD(OnParallelPortChange)(IParallelPort *aParallelPort)
+    {
+        return S_OK;
+    }
+
+    STDMETHOD(OnStorageControllerChange)()
     {
         return S_OK;
     }
@@ -162,7 +162,7 @@ public:
         return S_OK;
     }
 
-    STDMETHOD(OnSharedFolderChange) (Scope_T aScope)
+    STDMETHOD(OnSharedFolderChange)(Scope_T aScope)
     {
         return S_OK;
     }
@@ -181,7 +181,7 @@ public:
         return S_OK;
     }
 
-    STDMETHOD(OnShowWindow) (ULONG64 *winId)
+    STDMETHOD(OnShowWindow)(ULONG64 *winId)
     {
         if (!winId)
             return E_POINTER;
@@ -205,7 +205,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS1_CI(VRDPConsoleCallback, IConsoleCallback)
 
 #ifdef DEBUG_sunlover
 #define LOGDUMPPTR Log
-void dumpPointer (const uint8_t *pu8Shape, uint32_t width, uint32_t height, bool fXorMaskRGB32)
+void dumpPointer(const uint8_t *pu8Shape, uint32_t width, uint32_t height, bool fXorMaskRGB32)
 {
     unsigned i;
 
@@ -230,7 +230,7 @@ void dumpPointer (const uint8_t *pu8Shape, uint32_t width, uint32_t height, bool
 
     if (fXorMaskRGB32)
     {
-        uint32_t *pu32Xor = (uint32_t *)(pu8Shape + ((((width + 7) / 8) * height + 3) & ~3));
+        uint32_t *pu32Xor = (uint32_t*)(pu8Shape + ((((width + 7) / 8) * height + 3) & ~3));
 
         for (i = 0; i < height; i++)
         {
@@ -246,7 +246,7 @@ void dumpPointer (const uint8_t *pu8Shape, uint32_t width, uint32_t height, bool
     else
     {
         /* RDP 24 bit RGB mask. */
-        uint8_t *pu8Xor = (uint8_t *)(pu8Shape + ((((width + 7) / 8) * height + 3) & ~3));
+        uint8_t *pu8Xor = (uint8_t*)(pu8Shape + ((((width + 7) / 8) * height + 3) & ~3));
         for (i = 0; i < height; i++)
         {
             unsigned j;
@@ -264,7 +264,7 @@ void dumpPointer (const uint8_t *pu8Shape, uint32_t width, uint32_t height, bool
 #define dumpPointer(a, b, c, d) do {} while (0)
 #endif /* DEBUG_sunlover */
 
-static void findTopLeftBorder (const uint8_t *pu8AndMask, const uint8_t *pu8XorMask, uint32_t width, uint32_t height, uint32_t *pxSkip, uint32_t *pySkip)
+static void findTopLeftBorder(const uint8_t *pu8AndMask, const uint8_t *pu8XorMask, uint32_t width, uint32_t height, uint32_t *pxSkip, uint32_t *pySkip)
 {
     /*
      * Find the top border of the AND mask. First assign to special value.
@@ -384,8 +384,8 @@ static void findTopLeftBorder (const uint8_t *pu8AndMask, const uint8_t *pu8XorM
         xSkipXor = 0;
     }
 
-    *pxSkip = RT_MIN (xSkipAnd, xSkipXor);
-    *pySkip = RT_MIN (ySkipAnd, ySkipXor);
+    *pxSkip = RT_MIN(xSkipAnd, xSkipXor);
+    *pySkip = RT_MIN(ySkipAnd, ySkipXor);
 }
 
 /* Generate an AND mask for alpha pointers here, because
@@ -394,9 +394,9 @@ static void findTopLeftBorder (const uint8_t *pu8AndMask, const uint8_t *pu8XorM
  * for the guest driver, but then additions reinstall would be
  * necessary, which we try to avoid.
  */
-static void mousePointerGenerateANDMask (uint8_t *pu8DstAndMask, int cbDstAndMask, const uint8_t *pu8SrcAlpha, int w, int h)
+static void mousePointerGenerateANDMask(uint8_t *pu8DstAndMask, int cbDstAndMask, const uint8_t *pu8SrcAlpha, int w, int h)
 {
-    memset (pu8DstAndMask, 0xFF, cbDstAndMask);
+    memset(pu8DstAndMask, 0xFF, cbDstAndMask);
 
     int y;
     for (y = 0; y < h; y++)
@@ -424,24 +424,24 @@ static void mousePointerGenerateANDMask (uint8_t *pu8DstAndMask, int cbDstAndMas
     }
 }
 
-STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange (
-    BOOL visible,
-    BOOL alpha,
-    ULONG xHot,
-    ULONG yHot,
-    ULONG width,
-    ULONG height,
-    BYTE *shape)
+STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange(BOOL visible,
+                                                            BOOL alpha,
+                                                            ULONG xHot,
+                                                            ULONG yHot,
+                                                            ULONG width,
+                                                            ULONG height,
+                                                            ComSafeArrayIn(BYTE,inShape))
 {
     LogSunlover(("VRDPConsoleCallback::OnMousePointerShapeChange: %d, %d, %lux%lu, @%lu,%lu\n", visible, alpha, width, height, xHot, yHot));
 
     if (m_server)
     {
-        if (!shape)
+        com::SafeArray <BYTE> aShape(ComSafeArrayInArg (inShape));
+        if (aShape.size() == 0)
         {
             if (!visible)
             {
-                m_server->MousePointerHide ();
+                m_server->MousePointerHide();
             }
         }
         else if (width != 0 && height != 0)
@@ -462,8 +462,9 @@ STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange (
              * So set pointer size to 32x32. This can be done safely
              * because most pointers are 32x32.
              */
+            uint8_t* shape = aShape.raw();
 
-            dumpPointer (shape, width, height, true);
+            dumpPointer(shape, width, height, true);
 
             int cbDstAndMask = (((width + 7) / 8) * height + 3) & ~3;
 
@@ -472,9 +473,9 @@ STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange (
 
             if (alpha)
             {
-                pu8AndMask = (uint8_t *)alloca (cbDstAndMask);
+                pu8AndMask = (uint8_t*)alloca(cbDstAndMask);
 
-                mousePointerGenerateANDMask (pu8AndMask, cbDstAndMask, pu8XorMask, width, height);
+                mousePointerGenerateANDMask(pu8AndMask, cbDstAndMask, pu8XorMask, width, height);
             }
 
             /* Windows guest alpha pointers are wider than 32 pixels.
@@ -486,11 +487,11 @@ STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange (
             uint32_t ySkip = 0; /* How many rows to skip at the top. */
             uint32_t xSkip = 0; /* How many columns to skip at the left. */
 
-            findTopLeftBorder (pu8AndMask, pu8XorMask, width, height, &xSkip, &ySkip);
+            findTopLeftBorder(pu8AndMask, pu8XorMask, width, height, &xSkip, &ySkip);
 
             /* Must not skip the hot spot. */
-            xSkip = RT_MIN (xSkip, xHot);
-            ySkip = RT_MIN (ySkip, yHot);
+            xSkip = RT_MIN(xSkip, xHot);
+            ySkip = RT_MIN(ySkip, yHot);
 
             /*
              * Compute size and allocate memory for the pointer.
@@ -508,15 +509,15 @@ STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange (
             uint32_t rdpdatawidth = dstwidth * 3;
             uint32_t rdpdatalen = dstheight * rdpdatawidth;
 
-            pointer = (VRDPCOLORPOINTER *)RTMemTmpAlloc (sizeof (VRDPCOLORPOINTER) + rdpmasklen + rdpdatalen);
+            pointer = (VRDPCOLORPOINTER *)RTMemTmpAlloc(sizeof(VRDPCOLORPOINTER) + rdpmasklen + rdpdatalen);
 
             if (pointer)
             {
-                uint8_t *maskarray = (uint8_t *)pointer + sizeof (VRDPCOLORPOINTER);
+                uint8_t *maskarray = (uint8_t*)pointer + sizeof(VRDPCOLORPOINTER);
                 uint8_t *dataarray = maskarray + rdpmasklen;
 
-                memset (maskarray, 0xFF, rdpmasklen);
-                memset (dataarray, 0x00, rdpdatalen);
+                memset(maskarray, 0xFF, rdpmasklen);
+                memset(dataarray, 0x00, rdpdatalen);
 
                 uint32_t srcmaskwidth = (width + 7) / 8;
                 uint32_t srcdatawidth = width * 4;
@@ -525,8 +526,8 @@ STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange (
                 uint8_t *src = pu8AndMask + ySkip * srcmaskwidth;
                 uint8_t *dst = maskarray + (dstheight - 1) * rdpmaskwidth;
 
-                uint32_t minheight = RT_MIN (height - ySkip, dstheight);
-                uint32_t minwidth = RT_MIN (width - xSkip, dstwidth);
+                uint32_t minheight = RT_MIN(height - ySkip, dstheight);
+                uint32_t minwidth = RT_MIN(width - xSkip, dstwidth);
 
                 unsigned x, y;
 
@@ -560,7 +561,7 @@ STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange (
                 {
                     for (x = 0; x < minwidth; x++)
                     {
-                        memcpy (dst + x * 3, &src[4 * (x + xSkip)], 3);
+                        memcpy(dst + x * 3, &src[4 * (x + xSkip)], 3);
                     }
 
                     src += srcdatawidth;
@@ -576,11 +577,11 @@ STDMETHODIMP VRDPConsoleCallback::OnMousePointerShapeChange (
                 pointer->u16MaskLen = (uint16_t)rdpmasklen;
                 pointer->u16DataLen = (uint16_t)rdpdatalen;
 
-                dumpPointer ((uint8_t *)pointer + sizeof (*pointer), dstwidth, dstheight, false);
+                dumpPointer((uint8_t*)pointer + sizeof(*pointer), dstwidth, dstheight, false);
 
-                m_server->MousePointerUpdate (pointer);
+                m_server->MousePointerUpdate(pointer);
 
-                RTMemTmpFree (pointer);
+                RTMemTmpFree(pointer);
             }
         }
     }
@@ -601,7 +602,7 @@ VRDPENTRYPOINTS_1 *ConsoleVRDPServer::mpEntryPoints = NULL;
 
 VRDPCALLBACKS_1 ConsoleVRDPServer::mCallbacks =
 {
-    { VRDP_INTERFACE_VERSION_1, sizeof (VRDPCALLBACKS_1) },
+    { VRDP_INTERFACE_VERSION_1, sizeof(VRDPCALLBACKS_1) },
     ConsoleVRDPServer::VRDPCallbackQueryProperty,
     ConsoleVRDPServer::VRDPCallbackClientLogon,
     ConsoleVRDPServer::VRDPCallbackClientConnect,
@@ -616,9 +617,9 @@ VRDPCALLBACKS_1 ConsoleVRDPServer::mCallbacks =
     ConsoleVRDPServer::VRDPCallbackVideoModeHint
 };
 
-DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallback, uint32_t index, void *pvBuffer, uint32_t cbBuffer, uint32_t *pcbOut)
+DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty(void *pvCallback, uint32_t index, void *pvBuffer, uint32_t cbBuffer, uint32_t *pcbOut)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
     int rc = VERR_NOT_SUPPORTED;
 
@@ -629,7 +630,7 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
             /* This is obsolete, the VRDP server uses VRDP_QP_NETWORK_PORT_RANGE instead. */
             ULONG port = 0;
 
-            if (cbBuffer >= sizeof (uint32_t))
+            if (cbBuffer >= sizeof(uint32_t))
             {
                 *(uint32_t *)pvBuffer = (uint32_t)port;
                 rc = VINF_SUCCESS;
@@ -639,18 +640,18 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
                 rc = VINF_BUFFER_OVERFLOW;
             }
 
-            *pcbOut = sizeof (uint32_t);
+            *pcbOut = sizeof(uint32_t);
         } break;
 
         case VRDP_QP_NETWORK_ADDRESS:
         {
             com::Bstr bstr;
-            server->mConsole->getVRDPServer ()->COMGETTER(NetAddress) (bstr.asOutParam());
+            server->mConsole->getVRDPServer()->COMGETTER(NetAddress)(bstr.asOutParam());
 
             /* The server expects UTF8. */
             com::Utf8Str address = bstr;
 
-            size_t cbAddress = address.length () + 1;
+            size_t cbAddress = address.length() + 1;
 
             if (cbAddress >= 0x10000)
             {
@@ -665,7 +666,7 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
                 {
                     if (address.raw())
                     {
-                        memcpy (pvBuffer, address.raw(), cbAddress);
+                        memcpy(pvBuffer, address.raw(), cbAddress);
                     }
                     else
                     {
@@ -688,9 +689,9 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
         {
             ULONG cMonitors = 1;
 
-            server->mConsole->machine ()->COMGETTER(MonitorCount)(&cMonitors);
+            server->mConsole->machine()->COMGETTER(MonitorCount)(&cMonitors);
 
-            if (cbBuffer >= sizeof (uint32_t))
+            if (cbBuffer >= sizeof(uint32_t))
             {
                 *(uint32_t *)pvBuffer = (uint32_t)cMonitors;
                 rc = VINF_SUCCESS;
@@ -700,13 +701,13 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
                 rc = VINF_BUFFER_OVERFLOW;
             }
 
-            *pcbOut = sizeof (uint32_t);
+            *pcbOut = sizeof(uint32_t);
         } break;
 
         case VRDP_QP_NETWORK_PORT_RANGE:
         {
             com::Bstr bstr;
-            HRESULT hrc = server->mConsole->getVRDPServer ()->COMGETTER(Ports) (bstr.asOutParam());
+            HRESULT hrc = server->mConsole->getVRDPServer()->COMGETTER(Ports)(bstr.asOutParam());
 
             if (hrc != S_OK)
             {
@@ -736,7 +737,7 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
                 {
                     if (portRange.raw())
                     {
-                        memcpy (pvBuffer, portRange.raw(), cbPortRange);
+                        memcpy(pvBuffer, portRange.raw(), cbPortRange);
                     }
                     else
                     {
@@ -755,26 +756,66 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
             *pcbOut = (uint32_t)cbPortRange;
         } break;
 
+#ifdef VBOX_WITH_VRDP_VIDEO_CHANNEL
+        case VRDP_QP_VIDEO_CHANNEL:
+        {
+            BOOL fVideoEnabled = FALSE;
+
+            server->mConsole->getVRDPServer()->COMGETTER(VideoChannel)(&fVideoEnabled);
+
+            if (cbBuffer >= sizeof(uint32_t))
+            {
+                *(uint32_t *)pvBuffer = (uint32_t)fVideoEnabled;
+                rc = VINF_SUCCESS;
+            }
+            else
+            {
+                rc = VINF_BUFFER_OVERFLOW;
+            }
+
+            *pcbOut = sizeof(uint32_t);
+        } break;
+
+        case VRDP_QP_VIDEO_CHANNEL_QUALITY:
+        {
+            ULONG ulQuality = 0;
+
+            server->mConsole->getVRDPServer()->COMGETTER(VideoChannelQuality)(&ulQuality);
+
+            if (cbBuffer >= sizeof(uint32_t))
+            {
+                *(uint32_t *)pvBuffer = (uint32_t)ulQuality;
+                rc = VINF_SUCCESS;
+            }
+            else
+            {
+                rc = VINF_BUFFER_OVERFLOW;
+            }
+
+            *pcbOut = sizeof(uint32_t);
+        } break;
+#endif /* VBOX_WITH_VRDP_VIDEO_CHANNEL */
+
         case VRDP_SP_NETWORK_BIND_PORT:
         {
-            if (cbBuffer != sizeof (uint32_t))
+            if (cbBuffer != sizeof(uint32_t))
             {
                 rc = VERR_INVALID_PARAMETER;
                 break;
             }
 
             ULONG port = *(uint32_t *)pvBuffer;
-            
+
             server->mVRDPBindPort = port;
 
             rc = VINF_SUCCESS;
 
             if (pcbOut)
             {
-                *pcbOut = sizeof (uint32_t);
+                *pcbOut = sizeof(uint32_t);
             }
 
-            server->mConsole->onRemoteDisplayInfoChange ();
+            server->mConsole->onRemoteDisplayInfoChange();
         } break;
 
         default:
@@ -784,30 +825,30 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
     return rc;
 }
 
-DECLCALLBACK(int) ConsoleVRDPServer::VRDPCallbackClientLogon (void *pvCallback, uint32_t u32ClientId, const char *pszUser, const char *pszPassword, const char *pszDomain)
+DECLCALLBACK(int) ConsoleVRDPServer::VRDPCallbackClientLogon(void *pvCallback, uint32_t u32ClientId, const char *pszUser, const char *pszPassword, const char *pszDomain)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
     return server->mConsole->VRDPClientLogon (u32ClientId, pszUser, pszPassword, pszDomain);
 }
 
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackClientConnect (void *pvCallback, uint32_t u32ClientId)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
     server->mConsole->VRDPClientConnect (u32ClientId);
 }
 
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackClientDisconnect (void *pvCallback, uint32_t u32ClientId, uint32_t fu32Intercepted)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
     server->mConsole->VRDPClientDisconnect (u32ClientId, fu32Intercepted);
 }
 
 DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackIntercept (void *pvCallback, uint32_t u32ClientId, uint32_t fu32Intercept, void **ppvIntercept)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
     LogFlowFunc(("%x\n", fu32Intercept));
 
@@ -864,7 +905,7 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackClipboard (void *pvCallback, v
 
 DECLCALLBACK(bool) ConsoleVRDPServer::VRDPCallbackFramebufferQuery (void *pvCallback, unsigned uScreenId, VRDPFRAMEBUFFERINFO *pInfo)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
     bool fAvailable = false;
 
@@ -919,7 +960,7 @@ DECLCALLBACK(bool) ConsoleVRDPServer::VRDPCallbackFramebufferQuery (void *pvCall
 
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackFramebufferLock (void *pvCallback, unsigned uScreenId)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
     if (server->maFramebuffers[uScreenId])
     {
@@ -929,7 +970,7 @@ DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackFramebufferLock (void *pvCallb
 
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackFramebufferUnlock (void *pvCallback, unsigned uScreenId)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
     if (server->maFramebuffers[uScreenId])
     {
@@ -957,7 +998,7 @@ static void fixKbdLockStatus (VRDPInputSynch *pInputSynch, IKeyboard *pKeyboard)
 
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackInput (void *pvCallback, int type, const void *pvInput, unsigned cbInput)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
     Console *pConsole = server->mConsole;
 
     switch (type)
@@ -1086,9 +1127,9 @@ DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackInput (void *pvCallback, int t
 
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackVideoModeHint (void *pvCallback, unsigned cWidth, unsigned cHeight, unsigned cBitsPerPixel, unsigned uScreenId)
 {
-    ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
+    ConsoleVRDPServer *server = static_cast<ConsoleVRDPServer*>(pvCallback);
 
-    server->mConsole->getDisplay ()->SetVideoModeHint(cWidth, cHeight, cBitsPerPixel, uScreenId);
+    server->mConsole->getDisplay()->SetVideoModeHint(cWidth, cHeight, cBitsPerPixel, uScreenId);
 }
 #endif /* VBOX_WITH_VRDP */
 
@@ -1096,8 +1137,8 @@ ConsoleVRDPServer::ConsoleVRDPServer (Console *console)
 {
     mConsole = console;
 
-    int rc = RTCritSectInit (&mCritSect);
-    AssertRC (rc);
+    int rc = RTCritSectInit(&mCritSect);
+    AssertRC(rc);
 
     mcClipboardRefs = 0;
     mpfnClipboardCallback = NULL;
@@ -1173,7 +1214,7 @@ ConsoleVRDPServer::~ConsoleVRDPServer ()
 
 int ConsoleVRDPServer::Launch (void)
 {
-    LogFlowMember(("ConsoleVRDPServer::Launch\n"));
+    LogFlowThisFunc(("\n"));
 #ifdef VBOX_WITH_VRDP
     int rc = VINF_SUCCESS;
     IVRDPServer *vrdpserver = mConsole->getVRDPServer ();
@@ -1195,7 +1236,7 @@ int ConsoleVRDPServer::Launch (void)
                 remoteUSBThreadStart ();
 #endif /* VBOX_WITH_USB */
             }
-            else
+            else if (rc != VERR_NET_ADDRESS_IN_USE)
                 AssertMsgFailed(("Could not start VRDP server: rc = %Rrc\n", rc));
         }
         else
@@ -1331,7 +1372,7 @@ void ConsoleVRDPServer::notifyRemoteUSBThreadRunning (RTTHREAD thread)
     mUSBBackends.thread = thread;
     mUSBBackends.fThreadRunning = true;
     int rc = RTThreadUserSignal (thread);
-    AssertRC (rc);
+    AssertRC(rc);
 }
 
 bool ConsoleVRDPServer::isRemoteUSBThreadRunning (void)
@@ -1339,7 +1380,7 @@ bool ConsoleVRDPServer::isRemoteUSBThreadRunning (void)
     return mUSBBackends.fThreadRunning;
 }
 
-void ConsoleVRDPServer::waitRemoteUSBThreadEvent (unsigned cMillies)
+void ConsoleVRDPServer::waitRemoteUSBThreadEvent (RTMSINTERVAL cMillies)
 {
     int rc = RTSemEventWait (mUSBBackends.event, cMillies);
     Assert (RT_SUCCESS(rc) || rc == VERR_TIMEOUT);
@@ -1371,7 +1412,7 @@ void ConsoleVRDPServer::remoteUSBThreadStart (void)
     {
         /* Wait until the thread is ready. */
         rc = RTThreadUserWait (mUSBBackends.thread, 60000);
-        AssertRC (rc);
+        AssertRC(rc);
         Assert (mUSBBackends.fThreadRunning || RT_FAILURE(rc));
     }
 }
@@ -1387,7 +1428,7 @@ void ConsoleVRDPServer::remoteUSBThreadStop (void)
         RTSemEventSignal (mUSBBackends.event);
 
         int rc = RTThreadWait (mUSBBackends.thread, 60000, NULL);
-        AssertRC (rc);
+        AssertRC(rc);
 
         mUSBBackends.thread = NIL_RTTHREAD;
     }
@@ -1435,7 +1476,12 @@ VRDPAuthResult ConsoleVRDPServer::Authenticate (const Guid &uuid, VRDPAuthGuestJ
 
         LogRel(("VRDPAUTH: ConsoleVRDPServer::Authenticate: loading external authentication library '%ls'\n", authLibrary.raw()));
 
-        int rc = RTLdrLoad (filename.raw(), &mAuthLibrary);
+        int rc;
+        if (RTPathHavePath(filename.raw()))
+            rc = RTLdrLoad(filename.raw(), &mAuthLibrary);
+        else
+            rc = RTLdrLoadAppPriv(filename.raw(), &mAuthLibrary);
+
         if (RT_FAILURE(rc))
             LogRel(("VRDPAUTH: Failed to load external authentication library. Error code: %Rrc\n", rc));
 
@@ -1474,7 +1520,7 @@ VRDPAuthResult ConsoleVRDPServer::Authenticate (const Guid &uuid, VRDPAuthGuestJ
 
         if (RT_FAILURE(rc))
         {
-            mConsole->reportAuthLibraryError (filename.raw(), rc);
+            mConsole->reportAuthLibraryError(filename.raw(), rc);
 
             mpfnAuthEntry = NULL;
             mpfnAuthEntry2 = NULL;
@@ -1534,7 +1580,7 @@ void ConsoleVRDPServer::AuthDisconnect (const Guid &uuid, uint32_t u32ClientId)
 int ConsoleVRDPServer::lockConsoleVRDPServer (void)
 {
     int rc = RTCritSectEnter (&mCritSect);
-    AssertRC (rc);
+    AssertRC(rc);
     return rc;
 }
 
@@ -1929,7 +1975,7 @@ void ConsoleVRDPServer::usbBackendRemoveFromList (RemoteUSBBackend *pRemoteUSBBa
 {
 #ifdef VBOX_WITH_USB
     int rc = lockConsoleVRDPServer ();
-    AssertRC (rc);
+    AssertRC(rc);
 
     /* Exclude the found instance from the list. */
     if (pRemoteUSBBackend->pNext)
@@ -2110,7 +2156,15 @@ bool ConsoleVRDPServer::loadVRDPLibrary (void)
 // constructor / destructor
 /////////////////////////////////////////////////////////////////////////////
 
-DEFINE_EMPTY_CTOR_DTOR (RemoteDisplayInfo)
+RemoteDisplayInfo::RemoteDisplayInfo()
+    : mParent(NULL)
+{
+}
+
+RemoteDisplayInfo::~RemoteDisplayInfo()
+{
+}
+
 
 HRESULT RemoteDisplayInfo::FinalConstruct()
 {
@@ -2132,7 +2186,7 @@ HRESULT RemoteDisplayInfo::init (Console *aParent)
 {
     LogFlowThisFunc(("aParent=%p\n", aParent));
 
-    ComAssertRet (aParent, E_INVALIDARG);
+    ComAssertRet(aParent, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
@@ -2159,7 +2213,7 @@ void RemoteDisplayInfo::uninit()
     if (autoUninitSpan.uninitDone())
         return;
 
-    unconst(mParent).setNull();
+    unconst(mParent) = NULL;
 }
 
 // IRemoteDisplayInfo properties
@@ -2171,11 +2225,11 @@ void RemoteDisplayInfo::uninit()
         if (!a##_aName)                                                   \
             return E_POINTER;                                             \
                                                                           \
-        AutoCaller autoCaller(this);                                     \
-        CheckComRCReturnRC(autoCaller.rc());                             \
+        AutoCaller autoCaller(this);                                      \
+        if (FAILED(autoCaller.rc())) return autoCaller.rc();              \
                                                                           \
         /* todo: Not sure if a AutoReadLock would be sufficient. */       \
-        AutoWriteLock alock(this);                                       \
+        AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);                  \
                                                                           \
         uint32_t value;                                                   \
         uint32_t cbOut = 0;                                               \
@@ -2186,7 +2240,8 @@ void RemoteDisplayInfo::uninit()
         *a##_aName = cbOut? !!value: FALSE;                               \
                                                                           \
         return S_OK;                                                      \
-    }
+    }                                                                     \
+    extern void IMPL_GETTER_BOOL_DUMMY(void)
 
 #define IMPL_GETTER_SCALAR(_aType, _aName, _aIndex)                       \
     STDMETHODIMP RemoteDisplayInfo::COMGETTER(_aName) (_aType *a##_aName) \
@@ -2194,11 +2249,11 @@ void RemoteDisplayInfo::uninit()
         if (!a##_aName)                                                   \
             return E_POINTER;                                             \
                                                                           \
-        AutoCaller autoCaller(this);                                     \
-        CheckComRCReturnRC(autoCaller.rc());                             \
+        AutoCaller autoCaller(this);                                      \
+        if (FAILED(autoCaller.rc())) return autoCaller.rc();              \
                                                                           \
         /* todo: Not sure if a AutoReadLock would be sufficient. */       \
-        AutoWriteLock alock(this);                                       \
+        AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);                  \
                                                                           \
         _aType value;                                                     \
         uint32_t cbOut = 0;                                               \
@@ -2209,7 +2264,8 @@ void RemoteDisplayInfo::uninit()
         *a##_aName = cbOut? value: 0;                                     \
                                                                           \
         return S_OK;                                                      \
-    }
+    }                                                                     \
+    extern void IMPL_GETTER_SCALAR_DUMMY(void)
 
 #define IMPL_GETTER_BSTR(_aType, _aName, _aIndex)                         \
     STDMETHODIMP RemoteDisplayInfo::COMGETTER(_aName) (_aType *a##_aName) \
@@ -2217,11 +2273,11 @@ void RemoteDisplayInfo::uninit()
         if (!a##_aName)                                                   \
             return E_POINTER;                                             \
                                                                           \
-        AutoCaller autoCaller(this);                                     \
-        CheckComRCReturnRC(autoCaller.rc());                             \
+        AutoCaller autoCaller(this);                                      \
+        if (FAILED(autoCaller.rc())) return autoCaller.rc();              \
                                                                           \
         /* todo: Not sure if a AutoReadLock would be sufficient. */       \
-        AutoWriteLock alock(this);                                       \
+        AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);                  \
                                                                           \
         uint32_t cbOut = 0;                                               \
                                                                           \
@@ -2231,7 +2287,7 @@ void RemoteDisplayInfo::uninit()
         if (cbOut == 0)                                                   \
         {                                                                 \
             Bstr str("");                                                 \
-            str.cloneTo(a##_aName);                                      \
+            str.cloneTo(a##_aName);                                       \
             return S_OK;                                                  \
         }                                                                 \
                                                                           \
@@ -2250,12 +2306,13 @@ void RemoteDisplayInfo::uninit()
                                                                           \
         Bstr str(pchBuffer);                                              \
                                                                           \
-        str.cloneTo(a##_aName);                                          \
+        str.cloneTo(a##_aName);                                           \
                                                                           \
         RTMemTmpFree (pchBuffer);                                         \
                                                                           \
         return S_OK;                                                      \
-    }
+    }                                                                     \
+    extern void IMPL_GETTER_BSTR_DUMMY(void)
 
 IMPL_GETTER_BOOL   (BOOL,    Active,             VRDP_QI_ACTIVE);
 IMPL_GETTER_SCALAR (LONG,    Port,               VRDP_QI_PORT);
@@ -2275,4 +2332,5 @@ IMPL_GETTER_SCALAR (ULONG,   EncryptionStyle,    VRDP_QI_ENCRYPTION_STYLE);
 
 #undef IMPL_GETTER_BSTR
 #undef IMPL_GETTER_SCALAR
+#undef IMPL_GETTER_BOOL
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
