@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "SharedFolderImpl.h"
@@ -24,17 +20,21 @@
 #include "MachineImpl.h"
 #include "ConsoleImpl.h"
 
+#include "AutoCaller.h"
 #include "Logging.h"
 
 #include <iprt/param.h>
-#include <iprt/cpputils.h>
+#include <iprt/cpp/utils.h>
 #include <iprt/path.h>
 
 // constructor / destructor
 /////////////////////////////////////////////////////////////////////////////
 
 SharedFolder::SharedFolder()
-    : mParent (NULL)
+    : mParent(NULL),
+      mMachine(NULL),
+      mConsole(NULL),
+      mVirtualBox(NULL)
 {
 }
 
@@ -95,7 +95,7 @@ HRESULT SharedFolder::init (Machine *aMachine,
  */
 HRESULT SharedFolder::initCopy (Machine *aMachine, SharedFolder *aThat)
 {
-    ComAssertRet (aThat, E_INVALIDARG);
+    ComAssertRet(aThat, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
@@ -160,7 +160,7 @@ HRESULT SharedFolder::init (VirtualBox *aVirtualBox,
 
     unconst(mVirtualBox) = aVirtualBox;
 
-    HRESULT rc = protectedInit (aVirtualBox, aName, aHostPath, aWritable);
+    HRESULT rc = protectedInit(aVirtualBox, aName, aHostPath, aWritable);
 
     /* Confirm a successful initialization when it's the case */
     if (SUCCEEDED(rc))
@@ -175,13 +175,15 @@ HRESULT SharedFolder::init (VirtualBox *aVirtualBox,
  *  @note
  *      Must be called from under the object's lock!
  */
-HRESULT SharedFolder::protectedInit (VirtualBoxBaseWithChildrenNEXT *aParent,
-                                     CBSTR aName, CBSTR aHostPath, BOOL aWritable)
+HRESULT SharedFolder::protectedInit(VirtualBoxBase *aParent,
+                                    CBSTR aName,
+                                    CBSTR aHostPath,
+                                    BOOL aWritable)
 {
     LogFlowThisFunc(("aName={%ls}, aHostPath={%ls}, aWritable={%d}\n",
                       aName, aHostPath, aWritable));
 
-    ComAssertRet (aParent && aName && aHostPath, E_INVALIDARG);
+    ComAssertRet(aParent && aName && aHostPath, E_INVALIDARG);
 
     Utf8Str hostPath = Utf8Str (aHostPath);
     size_t hostPathLen = hostPath.length();
@@ -205,24 +207,23 @@ HRESULT SharedFolder::protectedInit (VirtualBoxBaseWithChildrenNEXT *aParent,
         hostPath.stripTrailingSlash();
 
     /* Check whether the path is full (absolute) */
-    char hostPathFull [RTPATH_MAX];
+    char hostPathFull[RTPATH_MAX];
     int vrc = RTPathAbsEx(NULL,
                           hostPath.c_str(),
                           hostPathFull,
                           sizeof (hostPathFull));
     if (RT_FAILURE(vrc))
-        return setError (E_INVALIDARG,
-            tr ("Invalid shared folder path: '%s' (%Rrc)"), hostPath.raw(), vrc);
+        return setError(E_INVALIDARG,
+                        tr("Invalid shared folder path: '%s' (%Rrc)"),
+                        hostPath.raw(), vrc);
 
     if (RTPathCompare(hostPath.c_str(), hostPathFull) != 0)
-        return setError (E_INVALIDARG,
-            tr ("Shared folder path '%s' is not absolute"), hostPath.raw());
+        return setError(E_INVALIDARG,
+                        tr("Shared folder path '%s' is not absolute"),
+                        hostPath.raw());
 
     unconst(mParent) = aParent;
 
-    /* register with parent */
-    mParent->addDependentChild (this);
-
     unconst(m.name) = aName;
     unconst(m.hostPath) = hostPath;
     m.writable = aWritable;
@@ -243,14 +244,11 @@ void SharedFolder::uninit()
     if (autoUninitSpan.uninitDone())
         return;
 
-    if (mParent)
-        mParent->removeDependentChild (this);
-
     unconst(mParent) = NULL;
 
-    unconst(mMachine).setNull();
-    unconst(mConsole).setNull();
-    unconst(mVirtualBox).setNull();
+    unconst(mMachine) = NULL;
+    unconst(mConsole) = NULL;
+    unconst(mVirtualBox) = NULL;
 }
 
 // ISharedFolder properties
@@ -261,7 +259,7 @@ STDMETHODIMP SharedFolder::COMGETTER(Name) (BSTR *aName)
     CheckComArgOutPointerValid(aName);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* mName is constant during life time, no need to lock */
     m.name.cloneTo(aName);
@@ -274,7 +272,7 @@ STDMETHODIMP SharedFolder::COMGETTER(HostPath) (BSTR *aHostPath)
     CheckComArgOutPointerValid(aHostPath);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* mHostPath is constant during life time, no need to lock */
     m.hostPath.cloneTo(aHostPath);
@@ -287,7 +285,7 @@ STDMETHODIMP SharedFolder::COMGETTER(Accessible) (BOOL *aAccessible)
     CheckComArgOutPointerValid(aAccessible);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* mName and mHostPath are constant during life time, no need to lock */
 
@@ -304,7 +302,7 @@ STDMETHODIMP SharedFolder::COMGETTER(Accessible) (BOOL *aAccessible)
         return S_OK;
     }
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     m.lastAccessError = BstrFmt (
         tr ("'%s' is not accessible (%Rrc)"), hostPath.raw(), vrc);
@@ -329,14 +327,11 @@ STDMETHODIMP SharedFolder::COMGETTER(LastAccessError) (BSTR *aLastAccessError)
     CheckComArgOutPointerValid(aLastAccessError);
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    if (m.lastAccessError.isEmpty())
-        Bstr("").cloneTo(aLastAccessError);
-    else
-        m.lastAccessError.cloneTo(aLastAccessError);
+    m.lastAccessError.cloneTo(aLastAccessError);
 
     return S_OK;
 }
