@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +14,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #define LOG_GROUP_MAIN_OVERRIDE LOG_GROUP_HGCM
@@ -93,6 +89,7 @@ class HGCMThread: public HGCMObject
 
         /* A caller thread waits for completion of a SENT message on this event. */
         RTSEMEVENTMULTI m_eventSend;
+        int32_t volatile m_i32MessagesProcessed;
 
         /* Critical section for accessing the thread data, mostly for message queues. */
         RTCRITSECT m_critsect;
@@ -193,7 +190,7 @@ static DECLCALLBACK(int) hgcmWorkerThreadFunc (RTTHREAD ThreadSelf, void *pvUser
     pThread->m_thread = ThreadSelf;
     pThread->m_fu32ThreadFlags &= ~HGCMMSG_TF_INITIALIZING;
     rc = RTThreadUserSignal (ThreadSelf);
-    AssertRC (rc);
+    AssertRC(rc);
 
     pThread->m_pfnThread (pThread->Handle (), pThread->m_pvUser);
 
@@ -216,6 +213,7 @@ HGCMThread::HGCMThread ()
     m_thread (NIL_RTTHREAD),
     m_eventThread (0),
     m_eventSend (0),
+    m_i32MessagesProcessed (0),
     m_fu32ThreadFlags (0),
     m_pMsgInputQueueHead (NULL),
     m_pMsgInputQueueTail (NULL),
@@ -299,8 +297,8 @@ int HGCMThread::Initialize (HGCMTHREADHANDLE handle, const char *pszThreadName,
                 {
                     /* Wait until the thread is ready. */
                     rc = RTThreadUserWait (thread, 30000);
-                    AssertRC (rc);
-                    Assert (!(m_fu32ThreadFlags & HGCMMSG_TF_INITIALIZING) || RT_FAILURE(rc));
+                    AssertRC(rc);
+                    Assert(!(m_fu32ThreadFlags & HGCMMSG_TF_INITIALIZING) || RT_FAILURE(rc));
                 }
                 else
                 {
@@ -436,12 +434,30 @@ int HGCMThread::MsgPost (HGCMMsgCore *pMsg, PHGCMMSGCALLBACK pfnCallback, bool f
 
         if (fWait)
         {
+            /* Immediately check if the message has been processed. */
             while ((pMsg->m_fu32Flags & HGCM_MSG_F_PROCESSED) == 0)
             {
-                RTSemEventMultiWait (m_eventSend, RT_INDEFINITE_WAIT);
-                RTSemEventMultiReset (m_eventSend);
+                /* Poll infrequently to make sure no completed message has been missed. */
+                RTSemEventMultiWait (m_eventSend, 1000);
 
                 LogFlow(("HGCMThread::MsgPost: wait completed flags = %08X\n", pMsg->m_fu32Flags));
+
+                if ((pMsg->m_fu32Flags & HGCM_MSG_F_PROCESSED) == 0)
+                {
+                    RTThreadYield();
+                }
+            }
+
+            /* 'Our' message has been processed, so should reset the semaphore.
+             * There is still possible that another message has been processed
+             * and the semaphore has been signalled again.
+             * Reset only if there are no other messages completed.
+             */
+            int32_t c = ASMAtomicDecS32(&m_i32MessagesProcessed);
+            Assert(c >= 0);
+            if (c == 0)
+            {
+                RTSemEventMultiReset (m_eventSend);
             }
 
             rc = pMsg->m_rcSend;
@@ -483,7 +499,7 @@ int HGCMThread::MsgGet (HGCMMsgCore **ppMsg)
             HGCMMsgCore *pMsg = m_pMsgInputQueueHead;
 
             /* Remove the message from the head of Queue list. */
-            Assert (m_pMsgInputQueueHead->m_pPrev == NULL);
+            Assert(m_pMsgInputQueueHead->m_pPrev == NULL);
 
             if (m_pMsgInputQueueHead->m_pNext)
             {
@@ -492,7 +508,7 @@ int HGCMThread::MsgGet (HGCMMsgCore **ppMsg)
             }
             else
             {
-                Assert (m_pMsgInputQueueHead == m_pMsgInputQueueTail);
+                Assert(m_pMsgInputQueueHead == m_pMsgInputQueueTail);
 
                 m_pMsgInputQueueHead = NULL;
                 m_pMsgInputQueueTail = NULL;
@@ -584,6 +600,14 @@ void HGCMThread::MsgComplete (HGCMMsgCore *pMsg, int32_t result)
 
         bool fWaited = ((pMsg->m_fu32Flags & HGCM_MSG_F_WAIT) != 0);
 
+        if (fWaited)
+        {
+            ASMAtomicIncS32(&m_i32MessagesProcessed);
+
+            /* This should be done before setting the HGCM_MSG_F_PROCESSED flag. */
+            pMsg->m_rcSend = result;
+        }
+
         /* The message is now completed. */
         pMsg->m_fu32Flags &= ~HGCM_MSG_F_IN_PROCESS;
         pMsg->m_fu32Flags &= ~HGCM_MSG_F_WAIT;
@@ -595,11 +619,6 @@ void HGCMThread::MsgComplete (HGCMMsgCore *pMsg, int32_t result)
 
         if (fWaited)
         {
-            /* If message is being waited, then it is referenced by the waiter and the pointer
-             * if valid even after hgcmObjDeleteHandle.
-             */
-            pMsg->m_rcSend = result;
-
             /* Wake up all waiters. so they can decide if their message has been processed. */
             RTSemEventMultiSignal (m_eventSend);
         }
