@@ -1,12 +1,10 @@
 /* $Id: ConsoleImpl.h $ */
-
 /** @file
- *
  * VBox Console COM Class definition
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,18 +13,14 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_CONSOLEIMPL
 #define ____H_CONSOLEIMPL
 
 #include "VirtualBoxBase.h"
-#include "ProgressImpl.h"
 #include "SchemaDefs.h"
+#include "VBox/com/array.h"
 
 class Guest;
 class Keyboard;
@@ -40,7 +34,9 @@ class SharedFolder;
 class RemoteDisplayInfo;
 class AudioSniffer;
 class ConsoleVRDPServer;
+class ConsoleCallbackRegistration;      /* See ConsoleImpl.cpp. */
 class VMMDev;
+class Progress;
 
 #include <VBox/vrdpapi.h>
 #include <VBox/pdmdrv.h>
@@ -70,7 +66,7 @@ typedef struct VUSBIRHCONFIG *PVUSBIRHCONFIG;
 #define CHECK_CONSOLE_DRV(drv) \
     do { \
         if (!(drv)) \
-            return setError (E_ACCESSDENIED, tr ("The console is not powered up")); \
+            return setError(E_ACCESSDENIED, tr("The console is not powered up")); \
     } while (0)
 
 // Console
@@ -104,26 +100,26 @@ public:
     void FinalRelease();
 
     // public initializers/uninitializers for internal purposes only
-    HRESULT init (IMachine *aMachine, IInternalMachineControl *aControl);
+    HRESULT init(IMachine *aMachine, IInternalMachineControl *aControl);
     void uninit();
 
     // IConsole properties
-    STDMETHOD(COMGETTER(Machine)) (IMachine **aMachine);
-    STDMETHOD(COMGETTER(State)) (MachineState_T *aMachineState);
-    STDMETHOD(COMGETTER(Guest)) (IGuest **aGuest);
-    STDMETHOD(COMGETTER(Keyboard)) (IKeyboard **aKeyboard);
-    STDMETHOD(COMGETTER(Mouse)) (IMouse **aMouse);
-    STDMETHOD(COMGETTER(Display)) (IDisplay **aDisplay);
-    STDMETHOD(COMGETTER(Debugger)) (IMachineDebugger **aDebugger);
-    STDMETHOD(COMGETTER(USBDevices)) (ComSafeArrayOut (IUSBDevice *, aUSBDevices));
-    STDMETHOD(COMGETTER(RemoteUSBDevices)) (ComSafeArrayOut (IHostUSBDevice *, aRemoteUSBDevices));
-    STDMETHOD(COMGETTER(RemoteDisplayInfo)) (IRemoteDisplayInfo **aRemoteDisplayInfo);
-    STDMETHOD(COMGETTER(SharedFolders)) (ComSafeArrayOut (ISharedFolder *, aSharedFolders));
+    STDMETHOD(COMGETTER(Machine))(IMachine **aMachine);
+    STDMETHOD(COMGETTER(State))(MachineState_T *aMachineState);
+    STDMETHOD(COMGETTER(Guest))(IGuest **aGuest);
+    STDMETHOD(COMGETTER(Keyboard))(IKeyboard **aKeyboard);
+    STDMETHOD(COMGETTER(Mouse))(IMouse **aMouse);
+    STDMETHOD(COMGETTER(Display))(IDisplay **aDisplay);
+    STDMETHOD(COMGETTER(Debugger))(IMachineDebugger **aDebugger);
+    STDMETHOD(COMGETTER(USBDevices))(ComSafeArrayOut(IUSBDevice *, aUSBDevices));
+    STDMETHOD(COMGETTER(RemoteUSBDevices))(ComSafeArrayOut(IHostUSBDevice *, aRemoteUSBDevices));
+    STDMETHOD(COMGETTER(RemoteDisplayInfo))(IRemoteDisplayInfo **aRemoteDisplayInfo);
+    STDMETHOD(COMGETTER(SharedFolders))(ComSafeArrayOut(ISharedFolder *, aSharedFolders));
 
     // IConsole methods
-    STDMETHOD(PowerUp) (IProgress **aProgress);
-    STDMETHOD(PowerUpPaused) (IProgress **aProgress);
-    STDMETHOD(PowerDown) (IProgress **aProgress);
+    STDMETHOD(PowerUp)(IProgress **aProgress);
+    STDMETHOD(PowerUpPaused)(IProgress **aProgress);
+    STDMETHOD(PowerDown)(IProgress **aProgress);
     STDMETHOD(Reset)();
     STDMETHOD(Pause)();
     STDMETHOD(Resume)();
@@ -131,23 +127,23 @@ public:
     STDMETHOD(SleepButton)();
     STDMETHOD(GetPowerButtonHandled)(BOOL *aHandled);
     STDMETHOD(GetGuestEnteredACPIMode)(BOOL *aEntered);
-    STDMETHOD(SaveState) (IProgress **aProgress);
-    STDMETHOD(AdoptSavedState) (IN_BSTR aSavedStateFile);
+    STDMETHOD(SaveState)(IProgress **aProgress);
+    STDMETHOD(AdoptSavedState)(IN_BSTR aSavedStateFile);
     STDMETHOD(ForgetSavedState)(BOOL aRemove);
-    STDMETHOD(GetDeviceActivity) (DeviceType_T aDeviceType,
-                                 DeviceActivity_T *aDeviceActivity);
-    STDMETHOD(AttachUSBDevice) (IN_BSTR aId);
-    STDMETHOD(DetachUSBDevice) (IN_BSTR aId, IUSBDevice **aDevice);
-    STDMETHOD(FindUSBDeviceByAddress) (IN_BSTR aAddress, IUSBDevice **aDevice);
-    STDMETHOD(FindUSBDeviceById) (IN_BSTR aId, IUSBDevice **aDevice);
-    STDMETHOD(CreateSharedFolder) (IN_BSTR aName, IN_BSTR aHostPath, BOOL aWritable);
-    STDMETHOD(RemoveSharedFolder) (IN_BSTR aName);
+    STDMETHOD(GetDeviceActivity)(DeviceType_T aDeviceType,
+                                DeviceActivity_T *aDeviceActivity);
+    STDMETHOD(AttachUSBDevice)(IN_BSTR aId);
+    STDMETHOD(DetachUSBDevice)(IN_BSTR aId, IUSBDevice **aDevice);
+    STDMETHOD(FindUSBDeviceByAddress)(IN_BSTR aAddress, IUSBDevice **aDevice);
+    STDMETHOD(FindUSBDeviceById)(IN_BSTR aId, IUSBDevice **aDevice);
+    STDMETHOD(CreateSharedFolder)(IN_BSTR aName, IN_BSTR aHostPath, BOOL aWritable);
+    STDMETHOD(RemoveSharedFolder)(IN_BSTR aName);
     STDMETHOD(TakeSnapshot)(IN_BSTR aName, IN_BSTR aDescription,
                             IProgress **aProgress);
     STDMETHOD(DeleteSnapshot)(IN_BSTR aId, IProgress **aProgress);
     STDMETHOD(RestoreSnapshot)(ISnapshot *aSnapshot, IProgress **aProgress);
     STDMETHOD(Teleport)(IN_BSTR aHostname, ULONG aPort, IN_BSTR aPassword, ULONG aMaxDowntime, IProgress **aProgress);
-    STDMETHOD(RegisterCallback) (IConsoleCallback *aCallback);
+    STDMETHOD(RegisterCallback)(IConsoleCallback *aCallback);
     STDMETHOD(UnregisterCallback)(IConsoleCallback *aCallback);
 
     // public methods for internal purposes only
@@ -170,56 +166,63 @@ public:
 
     ConsoleVRDPServer *consoleVRDPServer() const { return mConsoleVRDPServer; }
 
-    HRESULT updateMachineState (MachineState_T aMachineState);
+    HRESULT updateMachineState(MachineState_T aMachineState);
 
     // events from IInternalSessionControl
-    HRESULT onNetworkAdapterChange (INetworkAdapter *aNetworkAdapter, BOOL changeAdapter);
-    HRESULT onSerialPortChange (ISerialPort *aSerialPort);
-    HRESULT onParallelPortChange (IParallelPort *aParallelPort);
-    HRESULT onStorageControllerChange ();
+    HRESULT onNetworkAdapterChange(INetworkAdapter *aNetworkAdapter, BOOL changeAdapter);
+    HRESULT onSerialPortChange(ISerialPort *aSerialPort);
+    HRESULT onParallelPortChange(IParallelPort *aParallelPort);
+    HRESULT onStorageControllerChange();
     HRESULT onMediumChange(IMediumAttachment *aMediumAttachment, BOOL aForce);
-    HRESULT onVRDPServerChange();
+    HRESULT onCPUChange(ULONG aCPU, BOOL aRemove);
+    HRESULT onVRDPServerChange(BOOL aRestart);
     HRESULT onUSBControllerChange();
-    HRESULT onSharedFolderChange (BOOL aGlobal);
-    HRESULT onUSBDeviceAttach (IUSBDevice *aDevice, IVirtualBoxErrorInfo *aError, ULONG aMaskedIfs);
-    HRESULT onUSBDeviceDetach (IN_BSTR aId, IVirtualBoxErrorInfo *aError);
-    HRESULT getGuestProperty (IN_BSTR aKey, BSTR *aValue, ULONG64 *aTimestamp, BSTR *aFlags);
-    HRESULT setGuestProperty (IN_BSTR aKey, IN_BSTR aValue, IN_BSTR aFlags);
-    HRESULT enumerateGuestProperties (IN_BSTR aPatterns, ComSafeArrayOut(BSTR, aNames), ComSafeArrayOut(BSTR, aValues), ComSafeArrayOut(ULONG64, aTimestamps), ComSafeArrayOut(BSTR, aFlags));
+    HRESULT onSharedFolderChange(BOOL aGlobal);
+    HRESULT onUSBDeviceAttach(IUSBDevice *aDevice, IVirtualBoxErrorInfo *aError, ULONG aMaskedIfs);
+    HRESULT onUSBDeviceDetach(IN_BSTR aId, IVirtualBoxErrorInfo *aError);
+    HRESULT getGuestProperty(IN_BSTR aKey, BSTR *aValue, ULONG64 *aTimestamp, BSTR *aFlags);
+    HRESULT setGuestProperty(IN_BSTR aKey, IN_BSTR aValue, IN_BSTR aFlags);
+    HRESULT enumerateGuestProperties(IN_BSTR aPatterns, ComSafeArrayOut(BSTR, aNames), ComSafeArrayOut(BSTR, aValues), ComSafeArrayOut(ULONG64, aTimestamps), ComSafeArrayOut(BSTR, aFlags));
+    HRESULT onlineMergeMedium(IMediumAttachment *aMediumAttachment,
+                              ULONG aSourceIdx, ULONG aTargetIdx,
+                              IMedium *aSource, IMedium *aTarget,
+                              BOOL aMergeForward, IMedium *aParentForTarget,
+                              ComSafeArrayIn(IMedium *, aChildrenToReparent),
+                              IProgress *aProgress);
     VMMDev *getVMMDev() { return mVMMDev; }
-    AudioSniffer *getAudioSniffer () { return mAudioSniffer; }
+    AudioSniffer *getAudioSniffer() { return mAudioSniffer; }
 
-    int VRDPClientLogon (uint32_t u32ClientId, const char *pszUser, const char *pszPassword, const char *pszDomain);
-    void VRDPClientConnect (uint32_t u32ClientId);
-    void VRDPClientDisconnect (uint32_t u32ClientId, uint32_t fu32Intercepted);
-    void VRDPInterceptAudio (uint32_t u32ClientId);
-    void VRDPInterceptUSB (uint32_t u32ClientId, void **ppvIntercept);
-    void VRDPInterceptClipboard (uint32_t u32ClientId);
+    int VRDPClientLogon(uint32_t u32ClientId, const char *pszUser, const char *pszPassword, const char *pszDomain);
+    void VRDPClientConnect(uint32_t u32ClientId);
+    void VRDPClientDisconnect(uint32_t u32ClientId, uint32_t fu32Intercepted);
+    void VRDPInterceptAudio(uint32_t u32ClientId);
+    void VRDPInterceptUSB(uint32_t u32ClientId, void **ppvIntercept);
+    void VRDPInterceptClipboard(uint32_t u32ClientId);
 
-    void processRemoteUSBDevices (uint32_t u32ClientId, VRDPUSBDEVICEDESC *pDevList, uint32_t cbDevList);
+    void processRemoteUSBDevices(uint32_t u32ClientId, VRDPUSBDEVICEDESC *pDevList, uint32_t cbDevList);
 
     // callback callers (partly; for some events console callbacks are notified
     // directly from IInternalSessionControl event handlers declared above)
     void onMousePointerShapeChange(bool fVisible, bool fAlpha,
                                    uint32_t xHot, uint32_t yHot,
                                    uint32_t width, uint32_t height,
-                                   void *pShape);
-    void onMouseCapabilityChange (BOOL supportsAbsolute, BOOL needsHostCursor);
-    void onStateChange (MachineState_T aMachineState);
+                                   ComSafeArrayIn(uint8_t, aShape));
+    void onMouseCapabilityChange(BOOL supportsAbsolute, BOOL supportsRelative, BOOL needsHostCursor);
+    void onStateChange(MachineState_T aMachineState);
     void onAdditionsStateChange();
     void onAdditionsOutdated();
-    void onKeyboardLedsChange (bool fNumLock, bool fCapsLock, bool fScrollLock);
-    void onUSBDeviceStateChange (IUSBDevice *aDevice, bool aAttached,
-                                 IVirtualBoxErrorInfo *aError);
-    void onRuntimeError (BOOL aFatal, IN_BSTR aErrorID, IN_BSTR aMessage);
-    HRESULT onShowWindow (BOOL aCheck, BOOL *aCanShow, ULONG64 *aWinId);
+    void onKeyboardLedsChange(bool fNumLock, bool fCapsLock, bool fScrollLock);
+    void onUSBDeviceStateChange(IUSBDevice *aDevice, bool aAttached,
+                                IVirtualBoxErrorInfo *aError);
+    void onRuntimeError(BOOL aFatal, IN_BSTR aErrorID, IN_BSTR aMessage);
+    HRESULT onShowWindow(BOOL aCheck, BOOL *aCanShow, ULONG64 *aWinId);
     void onRemoteDisplayInfoChange();
 
     static const PDMDRVREG DrvStatusReg;
 
-    void reportAuthLibraryError (const char *filename, int rc)
+    void reportAuthLibraryError(const char *filename, int rc)
     {
-        setError (E_FAIL, tr("Could not load the external authentication library '%s' (%Rrc)"), filename, rc);
+        setError(E_FAIL, tr("Could not load the external authentication library '%s' (%Rrc)"), filename, rc);
     }
 
     static HRESULT handleUnexpectedExceptions(RT_SRC_POS_DECL);
@@ -240,20 +243,20 @@ private:
     class AutoVMCallerBase
     {
     public:
-        AutoVMCallerBase (Console *aThat) : mThat (aThat), mRC (S_OK)
+        AutoVMCallerBase(Console *aThat) : mThat(aThat), mRC(S_OK)
         {
-            Assert (aThat);
-            mRC = aThat->addVMCaller (taQuiet, taAllowNullVM);
+            Assert(aThat);
+            mRC = aThat->addVMCaller(taQuiet, taAllowNullVM);
         }
         ~AutoVMCallerBase()
         {
-            if (SUCCEEDED (mRC))
+            if (SUCCEEDED(mRC))
                 mThat->releaseVMCaller();
         }
         /** Decreases the number of callers before the instance is destroyed. */
         void release()
         {
-            AssertReturnVoid (SUCCEEDED (mRC));
+            AssertReturnVoid(SUCCEEDED(mRC));
             mThat->releaseVMCaller();
             mRC = E_FAIL;
         }
@@ -261,19 +264,19 @@ private:
          *  rechecked to ensure the operation succeeded. */
         void add()
         {
-            AssertReturnVoid (!SUCCEEDED (mRC));
-            mRC = mThat->addVMCaller (taQuiet, taAllowNullVM);
+            AssertReturnVoid(!SUCCEEDED(mRC));
+            mRC = mThat->addVMCaller(taQuiet, taAllowNullVM);
         }
         /** Returns the result of Console::addVMCaller() */
         HRESULT rc() const { return mRC; }
-        /** Shortcut to SUCCEEDED (rc()) */
-        bool isOk() const { return SUCCEEDED (mRC); }
+        /** Shortcut to SUCCEEDED(rc()) */
+        bool isOk() const { return SUCCEEDED(mRC); }
     protected:
         Console *mThat;
         HRESULT mRC;
     private:
-        DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP (AutoVMCallerBase)
-        DECLARE_CLS_NEW_DELETE_NOOP (AutoVMCallerBase)
+        DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP(AutoVMCallerBase)
+        DECLARE_CLS_NEW_DELETE_NOOP(AutoVMCallerBase)
     };
 
     /**
@@ -282,8 +285,8 @@ private:
      *  releaseVMCaller() on destruction. Intended for Console methods dealing
      *  with mpVM. The usage pattern is:
      *  <code>
-     *      AutoVMCaller autoVMCaller (this);
-     *      CheckComRCReturnRC (autoVMCaller.rc());
+     *      AutoVMCaller autoVMCaller(this);
+     *      if (FAILED(autoVMCaller.rc())) return autoVMCaller.rc();
      *      ...
      *      VMR3ReqCall (mpVM, ...
      *  </code>
@@ -326,9 +329,9 @@ private:
     {
         typedef AutoVMCallerBase <taQuiet, true> Base;
     public:
-        SafeVMPtrBase (Console *aThat) : Base (aThat), mpVM (NULL)
+        SafeVMPtrBase(Console *aThat) : Base(aThat), mpVM(NULL)
         {
-            if (SUCCEEDED (Base::mRC))
+            if (SUCCEEDED(Base::mRC))
                 mpVM = aThat->mpVM;
         }
         /** Smart SaveVMPtr to PVM cast operator */
@@ -337,8 +340,8 @@ private:
         PVM raw() const { return mpVM; }
     private:
         PVM mpVM;
-        DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP (SafeVMPtrBase)
-        DECLARE_CLS_NEW_DELETE_NOOP (SafeVMPtrBase)
+        DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP(SafeVMPtrBase)
+        DECLARE_CLS_NEW_DELETE_NOOP(SafeVMPtrBase)
     };
 
 public:
@@ -348,12 +351,12 @@ public:
      *  by calling addVMCaller() on construction and releaseVMCaller() on
      *  destruction. Intended for Console children. The usage pattern is:
      *  <code>
-     *      Console::SaveVMPtr pVM (mParent);
-     *      CheckComRCReturnRC (pVM.rc());
+     *      Console::SaveVMPtr pVM(mParent);
+     *      if (FAILED(pVM.rc())) return pVM.rc();
      *      ...
-     *      VMR3ReqCall (pVM, ...
+     *      VMR3ReqCall(pVM, ...
      *      ...
-     *      printf ("%p\n", pVM.raw());
+     *      printf("%p\n", pVM.raw());
      *  </code>
      *
      *  @note Temporarily locks the argument for writing.
@@ -367,9 +370,9 @@ public:
      *  Intended for pieces of code that don't need to return the VM access
      *  failure to the caller. The usage pattern is:
      *  <code>
-     *      Console::SaveVMPtrQuiet pVM (mParent);
+     *      Console::SaveVMPtrQuiet pVM(mParent);
      *      if (pVM.rc())
-     *          VMR3ReqCall (pVM, ...
+     *          VMR3ReqCall(pVM, ...
      *      return S_OK;
      *  </code>
      *
@@ -384,11 +387,11 @@ public:
     public:
         SharedFolderData() {}
         SharedFolderData(Bstr aHostPath, BOOL aWritable)
-           : mHostPath (aHostPath)
-           , mWritable (aWritable) {}
+           : mHostPath(aHostPath)
+           , mWritable(aWritable) {}
         SharedFolderData(const SharedFolderData& aThat)
-           : mHostPath (aThat.mHostPath)
-           , mWritable (aThat.mWritable) {}
+           : mHostPath(aThat.mHostPath)
+           , mWritable(aThat.mWritable) {}
         Bstr mHostPath;
         BOOL mWritable;
     };
@@ -400,17 +403,17 @@ private:
     typedef std::list <ComObjPtr<OUSBDevice> > USBDeviceList;
     typedef std::list <ComObjPtr<RemoteUSBDevice> > RemoteUSBDeviceList;
 
-    HRESULT addVMCaller (bool aQuiet = false, bool aAllowNullVM = false);
+    HRESULT addVMCaller(bool aQuiet = false, bool aAllowNullVM = false);
     void releaseVMCaller();
 
-    HRESULT consoleInitReleaseLog (const ComPtr<IMachine> aMachine);
+    HRESULT consoleInitReleaseLog(const ComPtr<IMachine> aMachine);
 
-    HRESULT powerUp (IProgress **aProgress, bool aPaused);
-    HRESULT powerDown (Progress *aProgress = NULL);
+    HRESULT powerUp(IProgress **aProgress, bool aPaused);
+    HRESULT powerDown(Progress *aProgress = NULL);
 
     HRESULT callTapSetupApplication(bool isStatic, RTFILE tapFD, Bstr &tapDevice,
                                     Bstr &tapSetupApplication);
-#if (defined(RT_OS_LINUX) || defined(RT_OS_FREEBSD)) && !defined(VBOX_WITH_NETFLT)
+#if ((defined(RT_OS_LINUX) && !defined(VBOX_WITH_NETFLT)) || defined(RT_OS_FREEBSD))
     HRESULT attachToTapInterface(INetworkAdapter *networkAdapter);
     HRESULT detachFromTapInterface(INetworkAdapter *networkAdapter);
 #endif
@@ -419,78 +422,123 @@ private:
     HRESULT setMachineState(MachineState_T aMachineState, bool aUpdateServer = true);
     HRESULT setMachineStateLocally(MachineState_T aMachineState)
     {
-        return setMachineState (aMachineState, false /* aUpdateServer */);
+        return setMachineState(aMachineState, false /* aUpdateServer */);
     }
 
-    HRESULT findSharedFolder (CBSTR aName,
-                              ComObjPtr<SharedFolder> &aSharedFolder,
-                              bool aSetError = false);
+    HRESULT findSharedFolder(CBSTR aName,
+                             ComObjPtr<SharedFolder> &aSharedFolder,
+                             bool aSetError = false);
 
-    HRESULT fetchSharedFolders (BOOL aGlobal);
-    bool findOtherSharedFolder (IN_BSTR aName,
-                                SharedFolderDataMap::const_iterator &aIt);
+    HRESULT fetchSharedFolders(BOOL aGlobal);
+    bool findOtherSharedFolder(IN_BSTR aName,
+                               SharedFolderDataMap::const_iterator &aIt);
 
-    HRESULT createSharedFolder (CBSTR aName, SharedFolderData aData);
-    HRESULT removeSharedFolder (CBSTR aName);
+    HRESULT createSharedFolder(CBSTR aName, SharedFolderData aData);
+    HRESULT removeSharedFolder(CBSTR aName);
 
     static DECLCALLBACK(int) configConstructor(PVM pVM, void *pvConsole);
+
+    int configMediumAttachment(PCFGMNODE pCtlInst,
+                               const char *pcszDevice,
+                               unsigned uInstance,
+                               StorageBus_T enmBus,
+                               bool fUseHostIOCache,
+                               bool fSetupMerge,
+                               unsigned uMergeSource,
+                               unsigned uMergeTarget,
+                               IMediumAttachment *pMediumAtt,
+                               MachineState_T aMachineState,
+                               HRESULT *phrc,
+                               bool fAttachDetach,
+                               bool fForceUnmount,
+                               PVM pVM,
+                               DeviceType_T *paLedDevType);
+    int configMedium(PCFGMNODE pLunL0,
+                     bool fPassthrough,
+                     DeviceType_T enmType,
+                     bool fUseHostIOCache,
+                     bool fSetupMerge,
+                     unsigned uMergeSource,
+                     unsigned uMergeTarget,
+                     IMedium *pMedium,
+                     MachineState_T aMachineState,
+                     HRESULT *phrc);
+    static DECLCALLBACK(int) reconfigureMediumAttachment(Console *pConsole,
+                                                         PVM pVM,
+                                                         const char *pcszDevice,
+                                                         unsigned uInstance,
+                                                         StorageBus_T enmBus,
+                                                         bool fUseHostIOCache,
+                                                         bool fSetupMerge,
+                                                         unsigned uMergeSource,
+                                                         unsigned uMergeTarget,
+                                                         IMediumAttachment *aMediumAtt,
+                                                         MachineState_T aMachineState,
+                                                         HRESULT *phrc);
+    static DECLCALLBACK(int) changeRemovableMedium(Console *pThis,
+                                                   const char *pcszDevice,
+                                                   unsigned uInstance,
+                                                   StorageBus_T enmBus,
+                                                   bool fUseHostIOCache,
+                                                   IMediumAttachment *aMediumAtt,
+                                                   bool fForce);
+
+    int configNetwork(const char *pszDevice, unsigned uInstance, unsigned uLun,
+                      INetworkAdapter *aNetworkAdapter, PCFGMNODE pCfg,
+                      PCFGMNODE pLunL0, PCFGMNODE pInst, bool fAttachDetach);
+
     static DECLCALLBACK(int) configGuestProperties(void *pvConsole);
-    static int configNetwork(Console *pThis, const char *pszDevice,
-                             unsigned uInstance, unsigned uLun,
-                             INetworkAdapter *aNetworkAdapter,
-                             PCFGMNODE pCfg, PCFGMNODE pLunL0,
-                             PCFGMNODE pInst, bool fAttachDetach);
+    static DECLCALLBACK(int) configGuestControl(void *pvConsole);
     static DECLCALLBACK(void) vmstateChangeCallback(PVM aVM, VMSTATE aState,
                                                     VMSTATE aOldState, void *aUser);
-    static DECLCALLBACK(int) changeDrive (Console *pThis, const char *pszDevice,
-                                          unsigned uInstance, unsigned uLun,
-                                          bool fHostDrive, const char *pszPath,
-                                          const char *pszFormat, bool fPassthrough,
-                                          bool fForce);
+    static DECLCALLBACK(int) unplugCpu(Console *pThis, unsigned uCpu);
+    static DECLCALLBACK(int) plugCpu(Console *pThis, unsigned uCpu);
     HRESULT doMediumChange(IMediumAttachment *aMediumAttachment, bool fForce);
+    HRESULT doCPURemove(ULONG aCpu);
+    HRESULT doCPUAdd(ULONG aCpu);
 
 #ifdef VBOX_DYNAMIC_NET_ATTACH
-    HRESULT doNetworkAdapterChange (const char *pszDevice, unsigned uInstance,
-                                    unsigned uLun, INetworkAdapter *aNetworkAdapter);
-    static DECLCALLBACK(int) changeNetworkAttachment (Console *pThis, const char *pszDevice,
-                                                      unsigned uInstance, unsigned uLun,
-                                                      INetworkAdapter *aNetworkAdapter);
+    HRESULT doNetworkAdapterChange(const char *pszDevice, unsigned uInstance,
+                                   unsigned uLun, INetworkAdapter *aNetworkAdapter);
+    static DECLCALLBACK(int) changeNetworkAttachment(Console *pThis, const char *pszDevice,
+                                                     unsigned uInstance, unsigned uLun,
+                                                     INetworkAdapter *aNetworkAdapter);
 #endif /* VBOX_DYNAMIC_NET_ATTACH */
 
 #ifdef VBOX_WITH_USB
-    HRESULT attachUSBDevice (IUSBDevice *aHostDevice, ULONG aMaskedIfs);
-    HRESULT detachUSBDevice (USBDeviceList::iterator &aIt);
+    HRESULT attachUSBDevice(IUSBDevice *aHostDevice, ULONG aMaskedIfs);
+    HRESULT detachUSBDevice(USBDeviceList::iterator &aIt);
 
-    static DECLCALLBACK(int) usbAttachCallback (Console *that, IUSBDevice *aHostDevice, PCRTUUID aUuid,
+    static DECLCALLBACK(int) usbAttachCallback(Console *that, IUSBDevice *aHostDevice, PCRTUUID aUuid,
                        bool aRemote, const char *aAddress, ULONG aMaskedIfs);
-    static DECLCALLBACK(int) usbDetachCallback (Console *that, USBDeviceList::iterator *aIt, PCRTUUID aUuid);
+    static DECLCALLBACK(int) usbDetachCallback(Console *that, USBDeviceList::iterator *aIt, PCRTUUID aUuid);
 #endif
 
     static DECLCALLBACK(int)    fntTakeSnapshotWorker(RTTHREAD Thread, void *pvUser);
 
     static DECLCALLBACK(int)    stateProgressCallback(PVM pVM, unsigned uPercent, void *pvUser);
 
-    static DECLCALLBACK(void)   setVMErrorCallback(PVM pVM, void *pvUser, int rc, RT_SRC_POS_DECL,
-                                                   const char *pszFormat, va_list args);
+    static DECLCALLBACK(void)   genericVMSetErrorCallback(PVM pVM, void *pvUser, int rc, RT_SRC_POS_DECL,
+                                                          const char *pszErrorFmt, va_list va);
 
-    static DECLCALLBACK(void) setVMRuntimeErrorCallbackF (PVM pVM, void *pvUser, uint32_t fFatal,
+    static DECLCALLBACK(void) setVMRuntimeErrorCallbackF(PVM pVM, void *pvUser, uint32_t fFatal,
                                const char *pszErrorId,
                                const char *pszFormat, ...);
-    static DECLCALLBACK(void) setVMRuntimeErrorCallback (PVM pVM, void *pvUser, uint32_t fFatal,
+    static DECLCALLBACK(void) setVMRuntimeErrorCallback(PVM pVM, void *pvUser, uint32_t fFatal,
                                const char *pszErrorId,
                                const char *pszFormat, va_list va);
 
-    HRESULT                     captureUSBDevices (PVM pVM);
-    void                        detachAllUSBDevices (bool aDone);
+    HRESULT                     captureUSBDevices(PVM pVM);
+    void                        detachAllUSBDevices(bool aDone);
 
-    static DECLCALLBACK (int)   powerUpThread (RTTHREAD Thread, void *pvUser);
-    static DECLCALLBACK (int)   saveStateThread (RTTHREAD Thread, void *pvUser);
-    static DECLCALLBACK (int)   powerDownThread (RTTHREAD Thread, void *pvUser);
+    static DECLCALLBACK(int)   powerUpThread(RTTHREAD Thread, void *pvUser);
+    static DECLCALLBACK(int)   saveStateThread(RTTHREAD Thread, void *pvUser);
+    static DECLCALLBACK(int)   powerDownThread(RTTHREAD Thread, void *pvUser);
 
-    static DECLCALLBACK(void *) drvStatus_QueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface);
+    static DECLCALLBACK(void *) drvStatus_QueryInterface(PPDMIBASE pInterface, const char *pszIID);
     static DECLCALLBACK(void)   drvStatus_UnitChanged(PPDMILEDCONNECTORS pInterface, unsigned iLUN);
     static DECLCALLBACK(void)   drvStatus_Destruct(PPDMDRVINS pDrvIns);
-    static DECLCALLBACK(int)    drvStatus_Construct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, uint32_t fFlags);
+    static DECLCALLBACK(int)    drvStatus_Construct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags);
 
     int mcAudioRefs;
     volatile uint32_t mcVRDPClients;
@@ -500,23 +548,22 @@ private:
     static uint32_t sSSMConsoleVer;
 
     HRESULT loadDataFromSavedState();
-    int loadStateFileExecInternal (PSSMHANDLE pSSM, uint32_t u32Version);
+    int loadStateFileExecInternal(PSSMHANDLE pSSM, uint32_t u32Version);
 
-    static DECLCALLBACK(void)   saveStateFileExec (PSSMHANDLE pSSM, void *pvUser);
-    static DECLCALLBACK(int)    loadStateFileExec (PSSMHANDLE pSSM, void *pvUser, uint32_t uVersion, uint32_t uPass);
+    static DECLCALLBACK(void)   saveStateFileExec(PSSMHANDLE pSSM, void *pvUser);
+    static DECLCALLBACK(int)    loadStateFileExec(PSSMHANDLE pSSM, void *pvUser, uint32_t uVersion, uint32_t uPass);
 
 #ifdef VBOX_WITH_GUEST_PROPS
     static DECLCALLBACK(int)    doGuestPropNotification(void *pvExtension, uint32_t, void *pvParms, uint32_t cbParms);
-    HRESULT                     doMoveGuestPropertiesOnPowerOff(bool fSaving);
     HRESULT                     doEnumerateGuestProperties(CBSTR aPatterns,
                                                            ComSafeArrayOut(BSTR, aNames),
                                                            ComSafeArrayOut(BSTR, aValues),
                                                            ComSafeArrayOut(ULONG64, aTimestamps),
                                                            ComSafeArrayOut(BSTR, aFlags));
 
-    bool enabledGuestPropertiesVRDP (void);
-    void updateGuestPropertiesVRDPLogon (uint32_t u32ClientId, const char *pszUser, const char *pszDomain);
-    void updateGuestPropertiesVRDPDisconnect (uint32_t u32ClientId);
+    bool enabledGuestPropertiesVRDP(void);
+    void updateGuestPropertiesVRDPLogon(uint32_t u32ClientId, const char *pszUser, const char *pszDomain);
+    void updateGuestPropertiesVRDPDisconnect(uint32_t u32ClientId);
 #endif
 
     /** @name Teleporter support
@@ -525,7 +572,8 @@ private:
     HRESULT                     teleporterSrc(TeleporterStateSrc *pState);
     HRESULT                     teleporterSrcReadACK(TeleporterStateSrc *pState, const char *pszWhich, const char *pszNAckMsg = NULL);
     HRESULT                     teleporterSrcSubmitCommand(TeleporterStateSrc *pState, const char *pszCommand, bool fWaitForAck = true);
-    int                         teleporterTrg(PVM pVM, IMachine *pMachine, bool fStartPaused, Progress *pProgress);
+    HRESULT                     teleporterTrg(PVM pVM, IMachine *pMachine, Utf8Str *pErrorMsg, bool fStartPaused,
+                                              Progress *pProgress, bool *pfPowerOffOnFailure);
     static DECLCALLBACK(int)    teleporterTrgServeConnection(RTSOCKET Sock, void *pvUser);
     /** @} */
 
@@ -585,14 +633,16 @@ private:
         cLedSata    = 30,
         iLedScsi    = iLedSata + cLedSata,
         cLedScsi    = 16,
-        cLedStorage = cLedFloppy + cLedIde + cLedSata + cLedScsi,
+        iLedSas     = iLedScsi + cLedScsi,
+        cLedSas     = 8,
+        cLedStorage = cLedFloppy + cLedIde + cLedSata + cLedScsi + cLedSas
     };
     DeviceType_T maStorageDevType[cLedStorage];
     PPDMLED      mapStorageLeds[cLedStorage];
     PPDMLED      mapNetworkLeds[SchemaDefs::NetworkAdapterCount];
     PPDMLED      mapSharedFolderLed;
     PPDMLED      mapUSBLed[2];
-#if !defined(VBOX_WITH_NETFLT) && (defined(RT_OS_LINUX) || defined(RT_OS_FREEBSD))
+#if ((defined(RT_OS_LINUX) && !defined(VBOX_WITH_NETFLT)) || defined(RT_OS_FREEBSD))
     Utf8Str      maTAPDeviceName[8];
     RTFILE       maTapFD[8];
 #endif
@@ -611,7 +661,7 @@ private:
      * operation before starting. */
     ComObjPtr<Progress> mptrCancelableProgress;
 
-    typedef std::list <ComPtr<IConsoleCallback> > CallbackList;
+    typedef std::list<ConsoleCallbackRegistration> CallbackList;
     CallbackList mCallbacks;
 
     struct
@@ -626,8 +676,7 @@ private:
             uint32_t yHot;
             uint32_t width;
             uint32_t height;
-            BYTE *shape;
-            size_t shapeSize;
+            com::SafeArray<BYTE> shape;
         }
         mpsc;
 
@@ -636,6 +685,7 @@ private:
         {
             bool valid;
             BOOL supportsAbsolute;
+            BOOL supportsRelative;
             BOOL needsHostCursor;
         }
         mcc;
@@ -649,6 +699,16 @@ private:
             bool scrollLock;
         }
         klc;
+
+        void clear()
+        {
+            /* We cannot do memset() on mpsc to avoid cleaning shape's vtable */
+            mpsc.shape.setNull();
+            mpsc.valid = mpsc.visible = mpsc.alpha = false;
+            mpsc.xHot = mpsc.yHot = mpsc.width = mpsc.height = 0;
+            ::memset(&mcc, 0, sizeof mcc);
+            ::memset(&klc, 0, sizeof klc);
+        }
     }
     mCallbackData;
 
