@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_GUESTOSTYPEIMPL
@@ -57,7 +53,11 @@ public:
                   const char *aId, const char *aDescription,
                   VBOXOSTYPE aOSType, uint32_t aOSHint,
                   uint32_t aRAMSize, uint32_t aVRAMSize, uint32_t aHDDSize,
-                  NetworkAdapterType_T aNetworkAdapterType, uint32_t numSerialEnabled);
+                  NetworkAdapterType_T aNetworkAdapterType, uint32_t numSerialEnabled,
+                  StorageControllerType_T aDvdStorageControllerType,
+                  StorageBus_T aDvdStorageBusType,
+                  StorageControllerType_T aHdStorageControllerType,
+                  StorageBus_T aHdStorageBusType);
     void uninit();
 
     // IGuestOSType properties
@@ -72,12 +72,23 @@ public:
     STDMETHOD(COMGETTER(RecommendedVRAM)) (ULONG *aVRAMSize);
     STDMETHOD(COMGETTER(RecommendedHDD)) (ULONG *aHDDSize);
     STDMETHOD(COMGETTER(AdapterType)) (NetworkAdapterType_T *aNetworkAdapterType);
+    STDMETHOD(COMGETTER(RecommendedFirmware)) (FirmwareType_T *aFirmwareType);
+    STDMETHOD(COMGETTER(RecommendedDvdStorageBus)) (StorageBus_T *aStorageBusType);
+    STDMETHOD(COMGETTER(RecommendedDvdStorageController)) (StorageControllerType_T *aStorageControllerType);
+    STDMETHOD(COMGETTER(RecommendedHdStorageBus)) (StorageBus_T *aStorageBusType);
+    STDMETHOD(COMGETTER(RecommendedHdStorageController)) (StorageControllerType_T *aStorageControllerType);
+    STDMETHOD(COMGETTER(RecommendedPae)) (BOOL *aRecommendedExtHw);
+    STDMETHOD(COMGETTER(RecommendedUsbHid)) (BOOL *aRecommendedUsbHid);
+    STDMETHOD(COMGETTER(RecommendedHpet)) (BOOL *aRecommendedHpet);
+    STDMETHOD(COMGETTER(RecommendedUsbTablet)) (BOOL *aRecommendedUsbTablet);
+    STDMETHOD(COMGETTER(RecommendedRtcUseUtc)) (BOOL *aRecommendedRtcUseUtc);
 
     // public methods only for internal purposes
     const Bstr &id() const { return mID; }
     bool is64Bit() const { return !!(mOSHint & VBOXOSHINT_64BIT); }
     bool recommendedIOAPIC() const { return !!(mOSHint & VBOXOSHINT_IOAPIC); }
     bool recommendedVirtEx() const { return !!(mOSHint & VBOXOSHINT_HWVIRTEX); }
+    bool recommendedEFI() const { return !!(mOSHint & VBOXOSHINT_EFI); }
     NetworkAdapterType_T networkAdapterType() const { return mNetworkAdapterType; }
     uint32_t numSerialEnabled() const { return mNumSerialEnabled; }
 
@@ -98,6 +109,10 @@ private:
     const uint32_t mMonitorCount;
     const NetworkAdapterType_T mNetworkAdapterType;
     const uint32_t mNumSerialEnabled;
+    const StorageControllerType_T mDvdStorageControllerType;
+    const StorageBus_T mDvdStorageBusType;
+    const StorageControllerType_T mHdStorageControllerType;
+    const StorageBus_T mHdStorageBusType;
 };
 
 #endif // ____H_GUESTOSTYPEIMPL
