@@ -1,12 +1,10 @@
 /* $Id: MouseImpl.h $ */
-
 /** @file
- *
  * VirtualBox COM class implementation
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_MOUSEIMPL
@@ -29,6 +23,11 @@
 #include "ConsoleImpl.h"
 #include <VBox/pdmdrv.h>
 
+/** Maximum number of devices supported */
+enum { MOUSE_MAX_DEVICES = 3 };
+/** Mouse driver instance data. */
+typedef struct DRVMAINMOUSE DRVMAINMOUSE, *PDRVMAINMOUSE;
+
 /** Simple mouse event class. */
 class MouseEvent
 {
@@ -48,10 +47,13 @@ public:
 typedef ConsoleEventBuffer<MouseEvent> MouseEventBuffer;
 
 class ATL_NO_VTABLE Mouse :
-    public VirtualBoxBase,
+    public VirtualBoxBase
+#ifndef VBOXBFE_WITHOUT_COM
+    ,
     public VirtualBoxSupportErrorInfoImpl<Mouse, IMouse>,
     public VirtualBoxSupportTranslation<Mouse>,
     VBOX_SCRIPTABLE_IMPL(IMouse)
+#endif
 {
 public:
 
@@ -78,6 +80,7 @@ public:
 
     // IMouse properties
     STDMETHOD(COMGETTER(AbsoluteSupported)) (BOOL *absoluteSupported);
+    STDMETHOD(COMGETTER(RelativeSupported)) (BOOL *relativeSupported);
     STDMETHOD(COMGETTER(NeedsHostCursor)) (BOOL *needsHostCursor);
 
     // IMouse methods
@@ -91,20 +94,72 @@ public:
 
     static const PDMDRVREG  DrvReg;
 
+    Console *getParent() const
+    {
+        return mParent;
+    }
+
+    // for VMMDevInterface
+    void onVMMDevCanAbsChange(bool)
+    {
+        sendMouseCapsNotifications();
+    }
+
+    void onVMMDevNeedsHostChange(bool needsHost)
+    {
+        fVMMDevNeedsHostCursor = needsHost;
+        sendMouseCapsNotifications();
+    }
+
 private:
 
-    static DECLCALLBACK(void *) drvQueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface);
-    static DECLCALLBACK(int)    drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, uint32_t fFlags);
+    static DECLCALLBACK(void *) drvQueryInterface(PPDMIBASE pInterface, const char *pszIID);
+    static DECLCALLBACK(void)   mouseReportModes (PPDMIMOUSECONNECTOR pInterface, bool fRel, bool fAbs);
+    static DECLCALLBACK(int)    drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags);
     static DECLCALLBACK(void)   drvDestruct(PPDMDRVINS pDrvIns);
 
-    const ComObjPtr<Console, ComWeakRef> mParent;
+    HRESULT getVMMDevMouseCaps(uint32_t *pfCaps);
+    HRESULT setVMMDevMouseCaps(uint32_t fCaps);
+    HRESULT reportRelEventToMouseDev(int32_t dx, int32_t dy, int32_t dz,
+                                 int32_t dw, uint32_t fButtons);
+    HRESULT reportAbsEventToMouseDev(uint32_t mouseXAbs, uint32_t mouseYAbs,
+                                 int32_t dz, int32_t dw, uint32_t fButtons);
+    HRESULT reportAbsEventToVMMDev(uint32_t mouseXAbs, uint32_t mouseYAbs);
+    HRESULT convertDisplayWidth(LONG x, uint32_t *pcX);
+    HRESULT convertDisplayHeight(LONG y, uint32_t *pcY);
+
+    void sendMouseCapsNotifications(void);
+
+#ifdef VBOXBFE_WITHOUT_COM
+    Console *mParent;
+#else
+    Console * const         mParent;
+#endif
     /** Pointer to the associated mouse driver. */
-    struct DRVMAINMOUSE    *mpDrv;
+    struct DRVMAINMOUSE    *mpDrv[MOUSE_MAX_DEVICES];
 
     LONG uHostCaps;
+    bool fVMMDevCanAbs;
+    bool fVMMDevNeedsHostCursor;
     uint32_t mLastAbsX;
     uint32_t mLastAbsY;
+    uint32_t mLastButtons;
 };
 
-#endif // ____H_MOUSEIMPL
+#ifdef VBOXBFE_WITHOUT_COM
+/** @todo make this a member of Console */
+extern Mouse *gMouse;
+
+/** @todo can we get these from the API somehow? */
+enum
+{
+    MouseButtonState_LeftButton = 1,
+    MouseButtonState_RightButton = 2,
+    MouseButtonState_MiddleButton = 4,
+    MouseButtonState_XButton1 = 8,
+    MouseButtonState_XButton2 = 16,
+};
+#endif
+
+#endif // !____H_MOUSEIMPL
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
