@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_VRDPSERVER
@@ -28,8 +24,6 @@
 
 #include <VBox/VRDPAuth.h>
 
-class Machine;
-
 namespace settings
 {
     struct VRDPSettings;
@@ -45,18 +39,6 @@ public:
 
     struct Data
     {
-        bool operator== (const Data &that) const
-        {
-            return this == &that ||
-                   (mEnabled == that.mEnabled &&
-                    mVRDPPorts == that.mVRDPPorts &&
-                    mVRDPAddress == that.mVRDPAddress &&
-                    mAuthType == that.mAuthType &&
-                    mAuthTimeout == that.mAuthTimeout &&
-                    mAllowMultiConnection == that.mAllowMultiConnection &&
-                    mReuseSingleConnection == that.mReuseSingleConnection);
-        }
-
         BOOL mEnabled;
         Bstr mVRDPPorts;
         Bstr mVRDPAddress;
@@ -64,6 +46,8 @@ public:
         ULONG mAuthTimeout;
         BOOL mAllowMultiConnection;
         BOOL mReuseSingleConnection;
+        BOOL mVideoChannel;
+        ULONG mVideoChannelQuality;
     };
 
     VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT (VRDPServer)
@@ -104,6 +88,10 @@ public:
     STDMETHOD(COMSETTER(AllowMultiConnection)) (BOOL aAllowMultiConnection);
     STDMETHOD(COMGETTER(ReuseSingleConnection)) (BOOL *aReuseSingleConnection);
     STDMETHOD(COMSETTER(ReuseSingleConnection)) (BOOL aReuseSingleConnection);
+    STDMETHOD(COMGETTER(VideoChannel)) (BOOL *aVideoChannel);
+    STDMETHOD(COMSETTER(VideoChannel)) (BOOL aVideoChannel);
+    STDMETHOD(COMGETTER(VideoChannelQuality)) (ULONG *aVideoChannelQuality);
+    STDMETHOD(COMSETTER(VideoChannelQuality)) (ULONG aVideoChannelQuality);
 
     // IVRDPServer methods
 
@@ -112,9 +100,7 @@ public:
     HRESULT loadSettings(const settings::VRDPSettings &data);
     HRESULT saveSettings(settings::VRDPSettings &data);
 
-    bool isModified() { AutoWriteLock alock (this); return mData.isBackedUp(); }
-    bool isReallyModified() { AutoWriteLock alock (this); return mData.hasActualChanges(); }
-    bool rollback();
+    void rollback();
     void commit();
     void copyFrom (VRDPServer *aThat);
 
@@ -123,10 +109,10 @@ public:
 
 private:
 
-    const ComObjPtr<Machine, ComWeakRef> mParent;
+    Machine * const     mParent;
     const ComObjPtr<VRDPServer> mPeer;
 
-    Backupable<Data> mData;
+    Backupable<Data>    mData;
 };
 
 #endif // ____H_VRDPSERVER
