@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -43,12 +39,30 @@
 #include <iprt/uuid.h>
 
 
+/*******************************************************************************
+*   Global Variables                                                           *
+*******************************************************************************/
+/**
+ * Standard options that gets included unless RTGETOPTINIT_FLAGS_NO_STD_OPTS is
+ * set.
+ */
+static RTGETOPTDEF const g_aStdOptions[] =
+{
+    {  "--help",        'h',    RTGETOPT_REQ_NOTHING },
+    {  "-help",         'h',    RTGETOPT_REQ_NOTHING },
+    {  "--version",     'V',    RTGETOPT_REQ_NOTHING },
+    {  "-version",      'V',    RTGETOPT_REQ_NOTHING },
+};
+/** The index of --help in g_aStdOptions.  Used for some trickery.  */
+#define RTGETOPT_STD_OPTIONS_HELP_IDX   0
+
+
 
 RTDECL(int) RTGetOptInit(PRTGETOPTSTATE pState, int argc, char **argv,
                          PCRTGETOPTDEF paOptions, size_t cOptions,
                          int iFirst, uint32_t fFlags)
 {
-    AssertReturn(!fFlags, VERR_INVALID_PARAMETER);
+    AssertReturn(!(fFlags & ~(RTGETOPTINIT_FLAGS_OPTS_FIRST | RTGETOPTINIT_FLAGS_NO_STD_OPTS)), VERR_INVALID_PARAMETER);
 
     pState->argv         = argv;
     pState->argc         = argc;
@@ -57,6 +71,9 @@ RTDECL(int) RTGetOptInit(PRTGETOPTSTATE pState, int argc, char **argv,
     pState->iNext        = iFirst;
     pState->pszNextShort = NULL;
     pState->pDef         = NULL;
+    pState->uIndex       = UINT32_MAX;
+    pState->fFlags       = fFlags;
+    pState->cNonOptions  = 0;
 
     /* validate the options. */
     for (size_t i = 0; i < cOptions; i++)
@@ -67,8 +84,6 @@ RTDECL(int) RTGetOptInit(PRTGETOPTSTATE pState, int argc, char **argv,
         Assert(paOptions[i].iShort != '-');
     }
 
-    /** @todo Add an flag for sorting the arguments so that all the options comes
-     *        first. */
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTGetOptInit);
@@ -179,8 +194,9 @@ static int rtgetoptConvertMacAddr(const char *pszValue, PRTMAC pAddr)
  * @param   pszOption       The alleged long option.
  * @param   paOptions       Option array.
  * @param   cOptions        Number of items in the array.
+ * @param   fFlags          Init flags.
  */
-static PCRTGETOPTDEF rtGetOptSearchLong(const char *pszOption, PCRTGETOPTDEF paOptions, size_t cOptions)
+static PCRTGETOPTDEF rtGetOptSearchLong(const char *pszOption, PCRTGETOPTDEF paOptions, size_t cOptions, uint32_t fFlags)
 {
     PCRTGETOPTDEF pOpt = paOptions;
     while (cOptions-- > 0)
@@ -232,6 +248,12 @@ static PCRTGETOPTDEF rtGetOptSearchLong(const char *pszOption, PCRTGETOPTDEF paO
         }
         pOpt++;
     }
+
+    if (!(fFlags & RTGETOPTINIT_FLAGS_NO_STD_OPTS))
+        for (uint32_t i = 0; i < RT_ELEMENTS(g_aStdOptions); i++)
+            if (!strcmp(pszOption, g_aStdOptions[i].pszLong))
+                return &g_aStdOptions[i];
+
     return NULL;
 }
 
@@ -243,8 +265,9 @@ static PCRTGETOPTDEF rtGetOptSearchLong(const char *pszOption, PCRTGETOPTDEF paO
  * @param   chOption        The option char.
  * @param   paOptions       Option array.
  * @param   cOptions        Number of items in the array.
+ * @param   fFlags          Init flags.
  */
-static PCRTGETOPTDEF rtGetOptSearchShort(int chOption, PCRTGETOPTDEF paOptions, size_t cOptions)
+static PCRTGETOPTDEF rtGetOptSearchShort(int chOption, PCRTGETOPTDEF paOptions, size_t cOptions, uint32_t fFlags)
 {
     PCRTGETOPTDEF pOpt = paOptions;
     while (cOptions-- > 0)
@@ -253,6 +276,15 @@ static PCRTGETOPTDEF rtGetOptSearchShort(int chOption, PCRTGETOPTDEF paOptions,
             return pOpt;
         pOpt++;
     }
+
+    if (!(fFlags & RTGETOPTINIT_FLAGS_NO_STD_OPTS))
+    {
+        for (uint32_t i = 0; i < RT_ELEMENTS(g_aStdOptions); i++)
+            if (g_aStdOptions[i].iShort == chOption)
+                return &g_aStdOptions[i];
+        if (chOption == '?')
+            return &g_aStdOptions[RTGETOPT_STD_OPTIONS_HELP_IDX];
+    }
     return NULL;
 }
 
@@ -294,7 +326,7 @@ static int rtGetOptProcessValue(uint32_t fFlags, const char *pszValue, PRTGETOPT
             }
             break;
 
-#define MY_INT_CASE(req,type,memb,convfn) \
+#define MY_INT_CASE(req, type, memb, convfn) \
             case req: \
             { \
                 type Value; \
@@ -307,7 +339,7 @@ static int rtGetOptProcessValue(uint32_t fFlags, const char *pszValue, PRTGETOPT
                 pValueUnion->memb = Value; \
                 break; \
             }
-#define MY_BASE_INT_CASE(req,type,memb,convfn,base) \
+#define MY_BASE_INT_CASE(req, type, memb, convfn, base) \
             case req: \
             { \
                 type Value; \
@@ -317,41 +349,41 @@ static int rtGetOptProcessValue(uint32_t fFlags, const char *pszValue, PRTGETOPT
                 break; \
             }
 
-        MY_INT_CASE(RTGETOPT_REQ_INT8,   int8_t,   i,   RTStrToInt8Full)
-        MY_INT_CASE(RTGETOPT_REQ_INT16,  int16_t,  i,   RTStrToInt16Full)
-        MY_INT_CASE(RTGETOPT_REQ_INT32,  int32_t,  i,   RTStrToInt32Full)
-        MY_INT_CASE(RTGETOPT_REQ_INT64,  int64_t,  i,   RTStrToInt64Full)
-        MY_INT_CASE(RTGETOPT_REQ_UINT8,  uint8_t,  u,   RTStrToUInt8Full)
-        MY_INT_CASE(RTGETOPT_REQ_UINT16, uint16_t, u,   RTStrToUInt16Full)
-        MY_INT_CASE(RTGETOPT_REQ_UINT32, uint32_t, u,   RTStrToUInt32Full)
-        MY_INT_CASE(RTGETOPT_REQ_UINT64, uint64_t, u,   RTStrToUInt64Full)
+        MY_INT_CASE(RTGETOPT_REQ_INT8,   int8_t,   i8,  RTStrToInt8Full)
+        MY_INT_CASE(RTGETOPT_REQ_INT16,  int16_t,  i16, RTStrToInt16Full)
+        MY_INT_CASE(RTGETOPT_REQ_INT32,  int32_t,  i32, RTStrToInt32Full)
+        MY_INT_CASE(RTGETOPT_REQ_INT64,  int64_t,  i64, RTStrToInt64Full)
+        MY_INT_CASE(RTGETOPT_REQ_UINT8,  uint8_t,  u8,  RTStrToUInt8Full)
+        MY_INT_CASE(RTGETOPT_REQ_UINT16, uint16_t, u16, RTStrToUInt16Full)
+        MY_INT_CASE(RTGETOPT_REQ_UINT32, uint32_t, u32, RTStrToUInt32Full)
+        MY_INT_CASE(RTGETOPT_REQ_UINT64, uint64_t, u64, RTStrToUInt64Full)
 
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT8   | RTGETOPT_FLAG_HEX, int8_t,   i,   RTStrToInt8Full,   16)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT16  | RTGETOPT_FLAG_HEX, int16_t,  i,   RTStrToInt16Full,  16)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT32  | RTGETOPT_FLAG_HEX, int32_t,  i,   RTStrToInt32Full,  16)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT64  | RTGETOPT_FLAG_HEX, int64_t,  i,   RTStrToInt64Full,  16)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT8  | RTGETOPT_FLAG_HEX, uint8_t,  u,   RTStrToUInt8Full,  16)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT16 | RTGETOPT_FLAG_HEX, uint16_t, u,   RTStrToUInt16Full, 16)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT32 | RTGETOPT_FLAG_HEX, uint32_t, u,   RTStrToUInt32Full, 16)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT64 | RTGETOPT_FLAG_HEX, uint64_t, u,   RTStrToUInt64Full, 16)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT8   | RTGETOPT_FLAG_HEX, int8_t,   i8,  RTStrToInt8Full,   16)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT16  | RTGETOPT_FLAG_HEX, int16_t,  i16, RTStrToInt16Full,  16)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT32  | RTGETOPT_FLAG_HEX, int32_t,  i32, RTStrToInt32Full,  16)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT64  | RTGETOPT_FLAG_HEX, int64_t,  i64, RTStrToInt64Full,  16)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT8  | RTGETOPT_FLAG_HEX, uint8_t,  u8,  RTStrToUInt8Full,  16)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT16 | RTGETOPT_FLAG_HEX, uint16_t, u16, RTStrToUInt16Full, 16)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT32 | RTGETOPT_FLAG_HEX, uint32_t, u32, RTStrToUInt32Full, 16)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT64 | RTGETOPT_FLAG_HEX, uint64_t, u64, RTStrToUInt64Full, 16)
 
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT8   | RTGETOPT_FLAG_DEC, int8_t,   i,   RTStrToInt8Full,   10)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT16  | RTGETOPT_FLAG_DEC, int16_t,  i,   RTStrToInt16Full,  10)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT32  | RTGETOPT_FLAG_DEC, int32_t,  i,   RTStrToInt32Full,  10)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT64  | RTGETOPT_FLAG_DEC, int64_t,  i,   RTStrToInt64Full,  10)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT8  | RTGETOPT_FLAG_DEC, uint8_t,  u,   RTStrToUInt8Full,  10)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT16 | RTGETOPT_FLAG_DEC, uint16_t, u,   RTStrToUInt16Full, 10)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT32 | RTGETOPT_FLAG_DEC, uint32_t, u,   RTStrToUInt32Full, 10)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT64 | RTGETOPT_FLAG_DEC, uint64_t, u,   RTStrToUInt64Full, 10)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT8   | RTGETOPT_FLAG_DEC, int8_t,   i8,  RTStrToInt8Full,   10)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT16  | RTGETOPT_FLAG_DEC, int16_t,  i16, RTStrToInt16Full,  10)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT32  | RTGETOPT_FLAG_DEC, int32_t,  i32, RTStrToInt32Full,  10)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT64  | RTGETOPT_FLAG_DEC, int64_t,  i64, RTStrToInt64Full,  10)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT8  | RTGETOPT_FLAG_DEC, uint8_t,  u8,  RTStrToUInt8Full,  10)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT16 | RTGETOPT_FLAG_DEC, uint16_t, u16, RTStrToUInt16Full, 10)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT32 | RTGETOPT_FLAG_DEC, uint32_t, u32, RTStrToUInt32Full, 10)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT64 | RTGETOPT_FLAG_DEC, uint64_t, u64, RTStrToUInt64Full, 10)
 
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT8   | RTGETOPT_FLAG_OCT, int8_t,   i,   RTStrToInt8Full,   8)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT16  | RTGETOPT_FLAG_OCT, int16_t,  i,   RTStrToInt16Full,  8)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT32  | RTGETOPT_FLAG_OCT, int32_t,  i,   RTStrToInt32Full,  8)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_INT64  | RTGETOPT_FLAG_OCT, int64_t,  i,   RTStrToInt64Full,  8)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT8  | RTGETOPT_FLAG_OCT, uint8_t,  u,   RTStrToUInt8Full,  8)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT16 | RTGETOPT_FLAG_OCT, uint16_t, u,   RTStrToUInt16Full, 8)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT32 | RTGETOPT_FLAG_OCT, uint32_t, u,   RTStrToUInt32Full, 8)
-        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT64 | RTGETOPT_FLAG_OCT, uint64_t, u,   RTStrToUInt64Full, 8)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT8   | RTGETOPT_FLAG_OCT, int8_t,   i8,  RTStrToInt8Full,   8)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT16  | RTGETOPT_FLAG_OCT, int16_t,  i16, RTStrToInt16Full,  8)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT32  | RTGETOPT_FLAG_OCT, int32_t,  i32, RTStrToInt32Full,  8)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_INT64  | RTGETOPT_FLAG_OCT, int64_t,  i64, RTStrToInt64Full,  8)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT8  | RTGETOPT_FLAG_OCT, uint8_t,  u8,  RTStrToUInt8Full,  8)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT16 | RTGETOPT_FLAG_OCT, uint16_t, u16, RTStrToUInt16Full, 8)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT32 | RTGETOPT_FLAG_OCT, uint32_t, u32, RTStrToUInt32Full, 8)
+        MY_BASE_INT_CASE(RTGETOPT_REQ_UINT64 | RTGETOPT_FLAG_OCT, uint64_t, u64, RTStrToUInt64Full, 8)
 
 #undef MY_INT_CASE
 #undef MY_BASE_INT_CASE
@@ -394,13 +426,31 @@ static int rtGetOptProcessValue(uint32_t fFlags, const char *pszValue, PRTGETOPT
 }
 
 
+/**
+ * Moves one argv option entries.
+ *
+ * @param   papszTo             Destination.
+ * @param   papszFrom           Source.
+ */
+static void rtGetOptMoveArgvEntries(char **papszTo, char **papszFrom)
+{
+    if (papszTo != papszFrom)
+    {
+        Assert((uintptr_t)papszTo < (uintptr_t)papszFrom);
+        char * const pszMoved = papszFrom[0];
+        memmove(&papszTo[1], &papszTo[0], (uintptr_t)papszFrom - (uintptr_t)papszTo);
+        papszTo[0] = pszMoved;
+    }
+}
+
+
 RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
 {
     /*
      * Reset the variables kept in state.
      */
     pState->pDef = NULL;
-    pState->uIndex = UINT64_MAX;
+    pState->uIndex = UINT32_MAX;
 
     /*
      * Make sure the union is completely cleared out, whatever happens below.
@@ -408,12 +458,6 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
     pValueUnion->u64 = 0;
     pValueUnion->pDef = NULL;
 
-    /** @todo Handle '--' (end of options).*/
-    /** @todo Add a flag to RTGetOptInit for handling the various help options in
-     *        a common way. (-?,-h,-help,--help,++) */
-    /** @todo Add a flag to RTGetOptInit for handling the standard version options
-     *        in a common way. (-V,--version) */
-
     /*
      * The next option.
      */
@@ -427,7 +471,7 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
         /*
          * We've got short options left over from the previous call.
          */
-        pOpt = rtGetOptSearchShort(*pState->pszNextShort, pState->paOptions, pState->cOptions);
+        pOpt = rtGetOptSearchShort(*pState->pszNextShort, pState->paOptions, pState->cOptions, pState->fFlags);
         if (!pOpt)
         {
             pValueUnion->psz = pState->pszNextShort;
@@ -441,29 +485,83 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
     else
     {
         /*
-         * Pop off the next argument.
+         * Pop off the next argument.  Sorting options and dealing with the
+         * dash-dash makes this a little extra complicated.
          */
-        if (pState->iNext >= pState->argc)
-            return 0;
-        iThis = pState->iNext++;
-        pszArgThis = pState->argv[iThis];
-
-        /*
-         * Do a long option search first and then a short option one.
-         * This way we can make sure single dash long options doesn't
-         * get mixed up with short ones.
-         */
-        pOpt = rtGetOptSearchLong(pszArgThis, pState->paOptions, pState->cOptions);
-        if (    !pOpt
-            &&  pszArgThis[0] == '-'
-            &&  pszArgThis[1] != '-'
-            &&  pszArgThis[1] != '\0')
+        for (;;)
         {
-            pOpt = rtGetOptSearchShort(pszArgThis[1], pState->paOptions, pState->cOptions);
-            fShort = pOpt != NULL;
+            if (pState->iNext >= pState->argc)
+                return 0;
+
+            if (pState->cNonOptions)
+            {
+                if (pState->cNonOptions == INT32_MAX)
+                {
+                    pValueUnion->psz = pState->argv[pState->iNext++];
+                    return VINF_GETOPT_NOT_OPTION;
+                }
+
+                if (pState->iNext + pState->cNonOptions >= pState->argc)
+                {
+                    pState->cNonOptions = INT32_MAX;
+                    continue;
+                }
+            }
+
+            iThis = pState->iNext++;
+            pszArgThis = pState->argv[iThis + pState->cNonOptions];
+
+            /*
+             * Do a long option search first and then a short option one.
+             * This way we can make sure single dash long options doesn't
+             * get mixed up with short ones.
+             */
+            pOpt = rtGetOptSearchLong(pszArgThis, pState->paOptions, pState->cOptions, pState->fFlags);
+            if (    !pOpt
+                &&  pszArgThis[0] == '-'
+                &&  pszArgThis[1] != '-'
+                &&  pszArgThis[1] != '\0')
+            {
+                pOpt = rtGetOptSearchShort(pszArgThis[1], pState->paOptions, pState->cOptions, pState->fFlags);
+                fShort = pOpt != NULL;
+            }
+            else
+                fShort = false;
+
+            /* Look for dash-dash. */
+            if (!pOpt && !strcmp(pszArgThis, "--"))
+            {
+                rtGetOptMoveArgvEntries(&pState->argv[iThis], &pState->argv[iThis + pState->cNonOptions]);
+                pState->cNonOptions = INT32_MAX;
+                continue;
+            }
+
+            /* Options first hacks. */
+            if (pState->fFlags & RTGETOPTINIT_FLAGS_OPTS_FIRST)
+            {
+                if (pOpt)
+                    rtGetOptMoveArgvEntries(&pState->argv[iThis], &pState->argv[iThis + pState->cNonOptions]);
+                else if (*pszArgThis == '-')
+                {
+                    pValueUnion->psz = pszArgThis;
+                    return VERR_GETOPT_UNKNOWN_OPTION;
+                }
+                else
+                {
+                    /* not an option, add it to the non-options and try again. */
+                    pState->iNext--;
+                    pState->cNonOptions++;
+
+                    /* Switch to returning non-options if we've reached the end. */
+                    if (pState->iNext + pState->cNonOptions >= pState->argc)
+                        pState->cNonOptions = INT32_MAX;
+                    continue;
+                }
+            }
+
+            /* done */
+            break;
         }
-        else
-            fShort = false;
     }
 
     if (pOpt)
@@ -491,7 +589,8 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
                 {
                     if (iThis + 1 >= pState->argc)
                         return VERR_GETOPT_REQUIRED_ARGUMENT_MISSING;
-                    pszValue = pState->argv[iThis + 1];
+                    pszValue = pState->argv[iThis + pState->cNonOptions + 1];
+                    rtGetOptMoveArgvEntries(&pState->argv[iThis + 1], &pState->argv[iThis + pState->cNonOptions + 1]);
                     pState->iNext++;
                 }
                 else /* same argument. */
@@ -511,9 +610,9 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
                     if (pszArgThis[cchLong] == '\0')
                         return VERR_GETOPT_INDEX_MISSING;
 
-                    uint64_t uIndex;
+                    uint32_t uIndex;
                     char *pszRet = NULL;
-                    int rc = RTStrToUInt64Ex(&pszArgThis[cchLong], &pszRet, 10, &uIndex);
+                    int rc = RTStrToUInt32Ex(&pszArgThis[cchLong], &pszRet, 10, &uIndex);
                     if (rc == VWRN_TRAILING_CHARS)
                     {
                         if (   pszRet[0] != ':'
@@ -527,7 +626,8 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
                         if (iThis + 1 >= pState->argc)
                             return VERR_GETOPT_REQUIRED_ARGUMENT_MISSING;
                         pState->uIndex = uIndex;
-                        pszValue = pState->argv[iThis + 1];
+                        pszValue = pState->argv[iThis + pState->cNonOptions + 1];
+                        rtGetOptMoveArgvEntries(&pState->argv[iThis + 1], &pState->argv[iThis + pState->cNonOptions + 1]);
                         pState->iNext++;
                     }
                     else
@@ -540,7 +640,8 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
                     {
                         if (iThis + 1 >= pState->argc)
                             return VERR_GETOPT_REQUIRED_ARGUMENT_MISSING;
-                        pszValue = pState->argv[iThis + 1];
+                        pszValue = pState->argv[iThis + pState->cNonOptions + 1];
+                        rtGetOptMoveArgvEntries(&pState->argv[iThis + 1], &pState->argv[iThis + pState->cNonOptions + 1]);
                         pState->iNext++;
                     }
                     else /* same argument. */
@@ -583,9 +684,9 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
             if (pszArgThis[cchLong] == '\0')
                 return VERR_GETOPT_INDEX_MISSING;
 
-            uint64_t uIndex;
+            uint32_t uIndex;
             char *pszRet = NULL;
-            if (RTStrToUInt64Full(&pszArgThis[cchLong], 10, &uIndex) == VINF_SUCCESS)
+            if (RTStrToUInt32Full(&pszArgThis[cchLong], 10, &uIndex) == VINF_SUCCESS)
                 pState->uIndex = uIndex;
             else
                 AssertMsgFailedReturn(("%s\n", pszArgThis), VERR_GETOPT_INVALID_ARGUMENT_FORMAT); /* search bug */
@@ -599,7 +700,6 @@ RTDECL(int) RTGetOpt(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion)
      * Not a known option argument. If it starts with a switch char (-) we'll
      * fail with unkown option, and if it doesn't we'll return it as a non-option.
      */
-
     if (*pszArgThis == '-')
     {
         pValueUnion->psz = pszArgThis;
@@ -634,15 +734,18 @@ RTDECL(int) RTGetOptFetchValue(PRTGETOPTSTATE pState, PRTGETOPTUNION pValueUnion
     if (pState->iNext >= pState->argc)
         return VERR_GETOPT_REQUIRED_ARGUMENT_MISSING;
     int         iThis    = pState->iNext++;
-    const char *pszValue = pState->argv[iThis];
+    const char *pszValue = pState->argv[iThis + (pState->cNonOptions != INT32_MAX ? pState->cNonOptions : 0)];
     pValueUnion->pDef    = pOpt; /* in case of no value or error. */
 
+    if (pState->cNonOptions && pState->cNonOptions != INT32_MAX)
+        rtGetOptMoveArgvEntries(&pState->argv[iThis], &pState->argv[iThis + pState->cNonOptions]);
+
     return rtGetOptProcessValue(fFlags, pszValue, pValueUnion);
 }
 RT_EXPORT_SYMBOL(RTGetOptFetchValue);
 
 
-RTDECL(int) RTGetOptPrintError(int ch, PCRTGETOPTUNION pValueUnion)
+RTDECL(RTEXITCODE) RTGetOptPrintError(int ch, PCRTGETOPTUNION pValueUnion)
 {
     if (ch == VINF_GETOPT_NOT_OPTION)
         RTMsgError("Invalid parameter: %s", pValueUnion->psz);
@@ -660,7 +763,7 @@ RTDECL(int) RTGetOptPrintError(int ch, PCRTGETOPTUNION pValueUnion)
     else
         RTMsgError("%Rrs\n", ch);
 
-    return 2; /** @todo add defines for EXIT_SUCCESS, EXIT_FAILURE, EXIT_INVAL, etc... */
+    return RTEXITCODE_SYNTAX;
 }
 RT_EXPORT_SYMBOL(RTGetOptPrintError);
 
