@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -35,13 +31,27 @@
 #include <iprt/mem.h>
 #include "internal/iprt.h"
 
-#include <iprt/asm.h>
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
+# include <iprt/asm-amd64-x86.h>
+#endif
 #include <iprt/assert.h>
 #include <iprt/param.h>
 #include <iprt/string.h>
 #include <iprt/thread.h>
 #include "r0drv/alloc-r0drv.h"
 
+#undef RTMemTmpAlloc
+#undef RTMemTmpAllocZ
+#undef RTMemTmpFree
+#undef RTMemAlloc
+#undef RTMemAllocZ
+#undef RTMemAllocVar
+#undef RTMemAllocZVar
+#undef RTMemRealloc
+#undef RTMemFree
+#undef RTMemDup
+#undef RTMemDupEx
+
 
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
@@ -130,7 +140,7 @@ RTDECL(void *)  RTMemAlloc(size_t cb) RT_NO_THROW
     PRTMEMHDR pHdr;
     RT_ASSERT_INTS_ON();
 
-    pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, 0);
+    pHdr = rtR0MemAlloc(cb + RTR0MEM_FENCE_EXTRA, 0);
     if (pHdr)
     {
 #ifdef RTR0MEM_STRICT
@@ -160,7 +170,7 @@ RTDECL(void *)  RTMemAllocZ(size_t cb) RT_NO_THROW
     PRTMEMHDR pHdr;
     RT_ASSERT_INTS_ON();
 
-    pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, RTMEMHDR_FLAG_ZEROED);
+    pHdr = rtR0MemAlloc(cb + RTR0MEM_FENCE_EXTRA, RTMEMHDR_FLAG_ZEROED);
     if (pHdr)
     {
 #ifdef RTR0MEM_STRICT
@@ -176,6 +186,44 @@ RTDECL(void *)  RTMemAllocZ(size_t cb) RT_NO_THROW
 RT_EXPORT_SYMBOL(RTMemAllocZ);
 
 
+/**
+ * Wrapper around RTMemAlloc for automatically aligning variable sized
+ * allocations so that the various electric fence heaps works correctly.
+ *
+ * @returns See RTMemAlloc.
+ * @param   cbUnaligned         The unaligned size.
+ */
+RTDECL(void *) RTMemAllocVar(size_t cbUnaligned)
+{
+    size_t cbAligned;
+    if (cbUnaligned >= 16)
+        cbAligned = RT_ALIGN_Z(cbUnaligned, 16);
+    else
+        cbAligned = RT_ALIGN_Z(cbUnaligned, sizeof(void *));
+    return RTMemAlloc(cbAligned);
+}
+RT_EXPORT_SYMBOL(RTMemAllocVar);
+
+
+/**
+ * Wrapper around RTMemAllocZ for automatically aligning variable sized
+ * allocations so that the various electric fence heaps works correctly.
+ *
+ * @returns See RTMemAllocZ.
+ * @param   cbUnaligned         The unaligned size.
+ */
+RTDECL(void *) RTMemAllocZVar(size_t cbUnaligned)
+{
+    size_t cbAligned;
+    if (cbUnaligned >= 16)
+        cbAligned = RT_ALIGN_Z(cbUnaligned, 16);
+    else
+        cbAligned = RT_ALIGN_Z(cbUnaligned, sizeof(void *));
+    return RTMemAllocZ(cbAligned);
+}
+RT_EXPORT_SYMBOL(RTMemAllocZVar);
+
+
 /**
  * Reallocates memory.
  *
@@ -200,7 +248,7 @@ RTDECL(void *) RTMemRealloc(void *pvOld, size_t cbNew) RT_NO_THROW
             PRTMEMHDR pHdrNew;
             if (pHdrOld->cb >= cbNew && pHdrOld->cb - cbNew <= 128)
                 return pvOld;
-            pHdrNew = rtMemAlloc(cbNew + RTR0MEM_FENCE_EXTRA, 0);
+            pHdrNew = rtR0MemAlloc(cbNew + RTR0MEM_FENCE_EXTRA, 0);
             if (pHdrNew)
             {
                 size_t cbCopy = RT_MIN(pHdrOld->cb, pHdrNew->cb);
@@ -216,7 +264,7 @@ RTDECL(void *) RTMemRealloc(void *pvOld, size_t cbNew) RT_NO_THROW
                                   RTR0MEM_FENCE_EXTRA, (uint8_t *)(pHdrOld + 1) + pHdrOld->cb,
                                   RTR0MEM_FENCE_EXTRA, &g_abFence[0]));
 #endif
-                rtMemFree(pHdrOld);
+                rtR0MemFree(pHdrOld);
                 return pHdrNew + 1;
             }
         }
@@ -254,7 +302,7 @@ RTDECL(void) RTMemFree(void *pv) RT_NO_THROW
                           RTR0MEM_FENCE_EXTRA, (uint8_t *)(pHdr + 1) + pHdr->cb,
                           RTR0MEM_FENCE_EXTRA, &g_abFence[0]));
 #endif
-        rtMemFree(pHdr);
+        rtR0MemFree(pHdr);
     }
     else
         AssertMsgFailed(("pHdr->u32Magic=%RX32 pv=%p\n", pHdr->u32Magic, pv));
@@ -278,7 +326,7 @@ RTDECL(void *)    RTMemExecAlloc(size_t cb) RT_NO_THROW
     RT_ASSERT_PREEMPTIBLE();
 #endif
 
-    pHdr = rtMemAlloc(cb + RTR0MEM_FENCE_EXTRA, RTMEMHDR_FLAG_EXEC);
+    pHdr = rtR0MemAlloc(cb + RTR0MEM_FENCE_EXTRA, RTMEMHDR_FLAG_EXEC);
     if (pHdr)
     {
 #ifdef RTR0MEM_STRICT
@@ -316,7 +364,7 @@ RTDECL(void)      RTMemExecFree(void *pv) RT_NO_THROW
                           RTR0MEM_FENCE_EXTRA, (uint8_t *)(pHdr + 1) + pHdr->cb,
                           RTR0MEM_FENCE_EXTRA, &g_abFence[0]));
 #endif
-        rtMemFree(pHdr);
+        rtR0MemFree(pHdr);
     }
     else
         AssertMsgFailed(("pHdr->u32Magic=%RX32 pv=%p\n", pHdr->u32Magic, pv));
