@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -39,18 +35,29 @@
 #include <iprt/string.h>
 
 #include <stdlib.h>
-#include <malloc.h>
+#ifndef RT_OS_FREEBSD /* Deprecated on FreeBSD */
+# include <malloc.h>
+#endif
 #include <errno.h>
 #include <sys/mman.h>
 
-#if !defined(RT_USE_MMAP) && (defined(RT_OS_LINUX))
-# define RT_USE_MMAP
+
+/*******************************************************************************
+*   Defined Constants And Macros                                               *
+*******************************************************************************/
+#if !defined(RT_USE_MMAP_EXEC) && (defined(RT_OS_LINUX))
+# define RT_USE_MMAP_EXEC
 #endif
 
+#if !defined(RT_USE_MMAP_PAGE) && 0 /** @todo mmap is too slow for full scale EF setup. */
+# define RT_USE_MMAP_PAGE
+#endif
+
+
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
 *******************************************************************************/
-#ifdef RT_USE_MMAP
+#ifdef RT_USE_MMAP_EXEC
 /**
  * RTMemExecAlloc() header used when using mmap for allocating the memory.
  */
@@ -66,14 +73,12 @@ typedef struct RTMEMEXECHDR
 } RTMEMEXECHDR, *PRTMEMEXECHDR;
 
 /** Magic for RTMEMEXECHDR. */
-#define RTMEMEXECHDR_MAGIC (~(size_t)0xfeedbabe)
+# define RTMEMEXECHDR_MAGIC (~(size_t)0xfeedbabe)
 
-#endif  /* RT_USE_MMAP */
+#endif  /* RT_USE_MMAP_EXEC */
 
 
 
-#ifdef IN_RING3
-
 /**
  * Allocates memory which may contain code.
  *
@@ -85,15 +90,15 @@ RTDECL(void *) RTMemExecAlloc(size_t cb) RT_NO_THROW
 {
     AssertMsg(cb, ("Allocating ZERO bytes is really not a good idea! Good luck with the next assertion!\n"));
 
-#ifdef RT_USE_MMAP
+#ifdef RT_USE_MMAP_EXEC
     /*
      * Use mmap to get low memory.
      */
     size_t cbAlloc = RT_ALIGN_Z(cb + sizeof(RTMEMEXECHDR), PAGE_SIZE);
     void *pv = mmap(NULL, cbAlloc, PROT_READ | PROT_WRITE | PROT_EXEC, MAP_PRIVATE | MAP_ANONYMOUS
-#if defined(RT_ARCH_AMD64) && defined(MAP_32BIT)
+# if defined(RT_ARCH_AMD64) && defined(MAP_32BIT)
                     | MAP_32BIT
-#endif
+# endif
                     , -1, 0);
     AssertMsgReturn(pv != MAP_FAILED, ("errno=%d cb=%#zx\n", errno, cb), NULL);
     PRTMEMEXECHDR pHdr = (PRTMEMEXECHDR)pv;
@@ -143,7 +148,7 @@ RTDECL(void)    RTMemExecFree(void *pv) RT_NO_THROW
 {
     if (pv)
     {
-#ifdef RT_USE_MMAP
+#ifdef RT_USE_MMAP_EXEC
         PRTMEMEXECHDR pHdr = (PRTMEMEXECHDR)pv - 1;
         AssertMsgReturnVoid(RT_ALIGN_P(pHdr, PAGE_SIZE) == pHdr, ("pHdr=%p pv=%p\n", pHdr, pv));
         AssertMsgReturnVoid(pHdr->uMagic == RTMEMEXECHDR_MAGIC, ("pHdr=%p(uMagic=%#zx) pv=%p\n", pHdr, pHdr->uMagic, pv));
@@ -165,14 +170,22 @@ RTDECL(void)    RTMemExecFree(void *pv) RT_NO_THROW
  */
 RTDECL(void *) RTMemPageAlloc(size_t cb) RT_NO_THROW
 {
-#if 0 /** @todo huh? we're using posix_memalign in the next function... */
+#ifdef RT_USE_MMAP_PAGE
+    size_t  cbAligned = RT_ALIGN_Z(cb, PAGE_SIZE);
+    void   *pv = mmap(NULL, cbAligned, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+    AssertMsgReturn(pv != MAP_FAILED, ("errno=%d cb=%#zx\n", errno, cb), NULL);
+    return pv;
+
+#else
+# if defined(RT_OS_FREEBSD) /** @todo huh? we're using posix_memalign in the next function... */
     void *pv;
     int rc = posix_memalign(&pv, PAGE_SIZE, RT_ALIGN_Z(cb, PAGE_SIZE));
     if (!rc)
         return pv;
     return NULL;
-#else
+# else /* !RT_OS_FREEBSD */
     return memalign(PAGE_SIZE, cb);
+# endif
 #endif
 }
 
@@ -186,14 +199,22 @@ RTDECL(void *) RTMemPageAlloc(size_t cb) RT_NO_THROW
  */
 RTDECL(void *) RTMemPageAllocZ(size_t cb) RT_NO_THROW
 {
+#ifdef RT_USE_MMAP_PAGE
+    size_t  cbAligned = RT_ALIGN_Z(cb, PAGE_SIZE);
+    void   *pv = mmap(NULL, cbAligned, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+    AssertMsgReturn(pv != MAP_FAILED, ("errno=%d cb=%#zx\n", errno, cb), NULL);
+    return pv;
+
+#else
     void *pv;
     int rc = posix_memalign(&pv, PAGE_SIZE, RT_ALIGN_Z(cb, PAGE_SIZE));
     if (!rc)
     {
-        bzero(pv, RT_ALIGN_Z(cb, PAGE_SIZE));
+        RT_BZERO(pv, RT_ALIGN_Z(cb, PAGE_SIZE));
         return pv;
     }
     return NULL;
+#endif
 }
 
 
@@ -203,10 +224,20 @@ RTDECL(void *) RTMemPageAllocZ(size_t cb) RT_NO_THROW
  * @param   pv      Pointer to the block as it was returned by the allocation function.
  *                  NULL will be ignored.
  */
-RTDECL(void) RTMemPageFree(void *pv) RT_NO_THROW
+RTDECL(void) RTMemPageFree(void *pv, size_t cb) RT_NO_THROW
 {
     if (pv)
+    {
+        Assert(!((uintptr_t)pv & PAGE_OFFSET_MASK));
+
+#ifdef RT_USE_MMAP_PAGE
+        size_t cbAligned = RT_ALIGN_Z(cb, PAGE_SIZE);
+        int rc = munmap(pv, cbAligned);
+        AssertMsg(!rc, ("munmap(%p, %#zx) -> %d errno=%d\n", pv, cbAligned, rc, errno)); NOREF(rc);
+#else
         free(pv);
+#endif
+    }
 }
 
 
@@ -273,5 +304,3 @@ RTDECL(int) RTMemProtect(void *pv, size_t cb, unsigned fProtect) RT_NO_THROW
         return rc;
     return RTErrConvertFromErrno(errno);
 }
-
-#endif
