@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -67,7 +63,7 @@ int mmR3PagePoolInit(PVM pVM)
     /** @todo @bufref{1865},@bufref{3202}: mapping the page pool page into
      *        ring-0. Need to change the wasy we allocate it... */
     AssertReleaseReturn(sizeof(*pVM->mm.s.pPagePoolR3) + sizeof(*pVM->mm.s.pPagePoolLowR3) < PAGE_SIZE, VERR_INTERNAL_ERROR);
-    int rc = SUPPageAllocLockedEx(1, (void **)&pVM->mm.s.pPagePoolR3, NULL);
+    int rc = SUPR3PageAllocEx(1, 0 /*fFlags*/, (void **)&pVM->mm.s.pPagePoolR3, NULL /*pR0Ptr*/, NULL /*paPages*/);
     if (RT_FAILURE(rc))
         return rc;
     memset(pVM->mm.s.pPagePoolR3, 0, PAGE_SIZE);
@@ -122,7 +118,7 @@ void mmR3PagePoolTerm(PVM pVM)
         while (pSubPool)
         {
             int rc = SUPR3PageFreeEx(pSubPool->pvPages, pSubPool->cPages);
-            AssertMsgRC(rc, ("SUPPageFree(%p) failed with rc=%Rrc\n", pSubPool->pvPages, rc));
+            AssertMsgRC(rc, ("SUPR3PageFreeEx(%p) failed with rc=%Rrc\n", pSubPool->pvPages, rc));
             pSubPool->pvPages = NULL;
 
             /* next */
@@ -143,8 +139,8 @@ void mmR3PagePoolTerm(PVM pVM)
         PMMPAGESUBPOOL  pSubPool = pVM->mm.s.pPagePoolLowR3->pHead;
         while (pSubPool)
         {
-            int rc = SUPLowFree(pSubPool->pvPages, pSubPool->cPages);
-            AssertMsgRC(rc, ("SUPPageFree(%p) failed with rc=%d\n", pSubPool->pvPages, rc));
+            int rc = SUPR3LowFree(pSubPool->pvPages, pSubPool->cPages);
+            AssertMsgRC(rc, ("SUPR3LowFree(%p) failed with rc=%d\n", pSubPool->pvPages, rc));
             pSubPool->pvPages = NULL;
 
             /* next */
@@ -257,7 +253,7 @@ DECLINLINE(void *) mmR3PagePoolAlloc(PMMPAGEPOOL pPool)
                             N_("Failed to lock host %zd bytes of memory (out of memory)"), (size_t)cPages << PAGE_SHIFT);
     }
     else
-        rc = SUPLowAlloc(cPages, &pSub->pvPages, NULL, paPhysPages);
+        rc = SUPR3LowAlloc(cPages, &pSub->pvPages, NULL, paPhysPages);
     if (RT_SUCCESS(rc))
     {
         /*
@@ -317,11 +313,11 @@ DECLINLINE(void *) mmR3PagePoolAlloc(PMMPAGEPOOL pPool)
         return pSub->pvPages;
     }
 
-    MMR3HeapFree(pSub);
+    MMHyperFree(pPool->pVM, pSub);
     STAM_COUNTER_INC(&pPool->cErrors);
     if (pPool->fLow)
         VMSetError(pPool->pVM, rc, RT_SRC_POS,
-                   N_("Failed to expand page pool for memory below 4GB. current size: %d pages"),
+                   N_("Failed to expand page pool for memory below 4GB. Current size: %d pages"),
                    pPool->cPages);
     AssertMsgFailed(("Failed to expand pool%s. rc=%Rrc poolsize=%d\n",
                      pPool->fLow ? " (<4GB)" : "", rc, pPool->cPages));
@@ -396,6 +392,7 @@ DECLINLINE(void) mmR3PagePoolFree(PMMPAGEPOOL pPool, void *pv)
  */
 VMMR3DECL(void *) MMR3PageAlloc(PVM pVM)
 {
+    /* Note: unprotected by locks; currently fine as it's used during init or under the PGM lock */
     return mmR3PagePoolAlloc(pVM->mm.s.pPagePoolR3);
 }
 
@@ -414,6 +411,7 @@ VMMR3DECL(void *) MMR3PageAlloc(PVM pVM)
  */
 VMMR3DECL(RTHCPHYS) MMR3PageAllocPhys(PVM pVM)
 {
+    /* Note: unprotected by locks; currently fine as it's used during init or under the PGM lock */
     /** @todo optimize this, it's the most common case now. */
     void *pv = mmR3PagePoolAlloc(pVM->mm.s.pPagePoolR3);
     if (pv)
