@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,35 +13,22 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
-#ifndef __PDMAsyncCompletionInternal_h
-#define __PDMAsyncCompletionInternal_h
 
-#include "PDMInternal.h"
-#include <iprt/cdefs.h>
+#ifndef ___PDMAsyncCompletionInternal_h
+#define ___PDMAsyncCompletionInternal_h
+
 #include <iprt/critsect.h>
+#include <iprt/memcache.h>
+#include <iprt/sg.h>
 #include <VBox/types.h>
 #include <VBox/cfgm.h>
+#include <VBox/stam.h>
 #include <VBox/pdmasynccompletion.h>
+#include "PDMInternal.h"
 
 RT_C_DECLS_BEGIN
 
-/**
- * Supported endpoint classes.
- */
-typedef enum PDMASYNCCOMPLETIONEPCLASSTYPE
-{
-    /** File class. */
-    PDMASYNCCOMPLETIONEPCLASSTYPE_FILE = 0,
-    /** Number of supported classes. */
-    PDMASYNCCOMPLETIONEPCLASSTYPE_MAX,
-    /** 32bit hack. */
-    PDMASYNCCOMPLETIONEPCLASSTYPE_32BIT_HACK = 0x7fffffff
-} PDMASYNCCOMPLETIONEPCLASSTYPE;
 
 /**
  * PDM Async completion endpoint operations.
@@ -111,7 +98,7 @@ typedef struct PDMASYNCCOMPLETIONEPCLASSOPS
      */
     DECLR3CALLBACKMEMBER(int, pfnEpRead, (PPDMASYNCCOMPLETIONTASK pTask,
                                           PPDMASYNCCOMPLETIONENDPOINT pEndpoint, RTFOFF off,
-                                          PCPDMDATASEG paSegments, size_t cSegments,
+                                          PCRTSGSEG paSegments, size_t cSegments,
                                          size_t cbRead));
 
     /**
@@ -127,7 +114,7 @@ typedef struct PDMASYNCCOMPLETIONEPCLASSOPS
      */
     DECLR3CALLBACKMEMBER(int, pfnEpWrite, (PPDMASYNCCOMPLETIONTASK pTask,
                                            PPDMASYNCCOMPLETIONENDPOINT pEndpoint, RTFOFF off,
-                                           PCPDMDATASEG paSegments, size_t cSegments,
+                                           PCRTSGSEG paSegments, size_t cSegments,
                                            size_t cbWrite));
 
     /**
@@ -141,7 +128,7 @@ typedef struct PDMASYNCCOMPLETIONEPCLASSOPS
                                            PPDMASYNCCOMPLETIONENDPOINT pEndpoint));
 
     /**
-     * Initiates a flush request on the given endpoint.
+     * Queries the size of the endpoint. Optional.
      *
      * @returns VBox status code.
      * @param   pEndpoint     Endpoint the request is for.
@@ -150,6 +137,18 @@ typedef struct PDMASYNCCOMPLETIONEPCLASSOPS
     DECLR3CALLBACKMEMBER(int, pfnEpGetSize, (PPDMASYNCCOMPLETIONENDPOINT pEndpoint,
                                              uint64_t *pcbSize));
 
+    /**
+     * Sets the size of the endpoint. Optional.
+     * This is a synchronous operation.
+     *
+     *
+     * @returns VBox status code.
+     * @param   pEndpoint     Endpoint the request is for.
+     * @param   cbSize        New size for the endpoint.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnEpSetSize, (PPDMASYNCCOMPLETIONENDPOINT pEndpoint,
+                                             uint64_t cbSize));
+
     /** Initialization safety marker. */
     uint32_t    u32VersionEnd;
 } PDMASYNCCOMPLETIONEPCLASSOPS;
@@ -169,13 +168,6 @@ typedef struct PDMASYNCCOMPLETIONEPCLASS
 {
     /** Pointer to the shared VM structure. */
     PVM                                         pVM;
-    /** @name   Things configurable through CFGM
-     * @{ */
-    /** Size of the per endpoint cache. */
-    uint32_t                                    cEndpointCacheSize;
-    /** Size of the per class cache. */
-    uint32_t                                    cEpClassCacheSize;
-    /** @} */
     /** Critical section protecting the endpoint list. */
     RTCRITSECT                                  CritSect;
     /** Number of endpoints in the list. */
@@ -184,11 +176,8 @@ typedef struct PDMASYNCCOMPLETIONEPCLASS
     R3PTRTYPE(PPDMASYNCCOMPLETIONENDPOINT)      pEndpointsHead;
     /** Pointer to the callback table. */
     R3PTRTYPE(PCPDMASYNCCOMPLETIONEPCLASSOPS)   pEndpointOps;
-    /** Bigger cache for free task items used by all endpoints
-     * of this class. */
-    R3PTRTYPE(volatile PPDMASYNCCOMPLETIONTASK) apTaskCache[10];
-    /** Number of tasks cached */
-    volatile uint32_t                           cTasksCached;
+    /** Task cache. */
+    RTMEMCACHE                                  hMemCacheTasks;
 } PDMASYNCCOMPLETIONEPCLASS;
 /** Pointer to the PDM async completion endpoint class data. */
 typedef PDMASYNCCOMPLETIONEPCLASS *PPDMASYNCCOMPLETIONEPCLASS;
@@ -205,16 +194,6 @@ typedef struct PDMASYNCCOMPLETIONENDPOINT
     R3PTRTYPE(PPDMASYNCCOMPLETIONENDPOINT)      pPrev;
     /** Pointer to the class this endpoint belongs to. */
     R3PTRTYPE(PPDMASYNCCOMPLETIONEPCLASS)       pEpClass;
-    /** Head of the small cache for allocated task structures for exclusive
-     * use by this endpoint. */
-    R3PTRTYPE(volatile PPDMASYNCCOMPLETIONTASK) pTasksFreeHead;
-    /** Tail of the small cache for allocated task structures for exclusive
-     * use by this endpoint. */
-    R3PTRTYPE(volatile PPDMASYNCCOMPLETIONTASK) pTasksFreeTail;
-    /** Number of elements in the cache. */
-    volatile uint32_t                           cTasksCached;
-    /** Start slot for the global task cache. */
-    unsigned                                    iSlotStart;
     /** ID of the next task to ensure consistency. */
     volatile uint32_t                           uTaskIdNext;
     /** Flag whether a wraparound occurred for the ID counter. */
@@ -225,6 +204,18 @@ typedef struct PDMASYNCCOMPLETIONENDPOINT
     unsigned                                    cUsers;
     /** URI describing the endpoint */
     char                                       *pszUri;
+#ifdef VBOX_WITH_STATISTICS
+    STAMCOUNTER                                 StatTaskRunTimesNs[10];
+    STAMCOUNTER                                 StatTaskRunTimesMicroSec[10];
+    STAMCOUNTER                                 StatTaskRunTimesMs[10];
+    STAMCOUNTER                                 StatTaskRunTimesSec[10];
+    STAMCOUNTER                                 StatTaskRunOver100Sec;
+    STAMCOUNTER                                 StatIoOpsPerSec;
+    STAMCOUNTER                                 StatIoOpsStarted;
+    STAMCOUNTER                                 StatIoOpsCompleted;
+    uint64_t                                    tsIntervalStartMs;
+    uint64_t                                    cIoOpsCompleted;
+#endif
 } PDMASYNCCOMPLETIONENDPOINT;
 
 /**
@@ -245,18 +236,24 @@ typedef struct PDMASYNCCOMPLETIONTASK
     void                                   *pvUser;
     /** Task id. */
     uint32_t                                uTaskId;
+    /** Start timestamp. */
+    uint64_t                                tsNsStart;
 } PDMASYNCCOMPLETIONTASK;
 
 /**
  * Called by the endpoint if a task has finished.
  *
  * @returns nothing
- * @param   pTask    Pointer to the finished task.
+ * @param   pTask                     Pointer to the finished task.
+ * @param   rc                        Status code of the completed request.
+ * @param   fCallCompletionHandler    Flag whether the completion handler should be called to
+ *                                    inform the owner of the task that it has completed.
  */
-void pdmR3AsyncCompletionCompleteTask(PPDMASYNCCOMPLETIONTASK pTask);
+void pdmR3AsyncCompletionCompleteTask(PPDMASYNCCOMPLETIONTASK pTask, int rc, bool fCallCompletionHandler);
 
 RT_C_DECLS_END
 
 extern const PDMASYNCCOMPLETIONEPCLASSOPS g_PDMAsyncCompletionEndpointClassFile;
 
-#endif /* __PDMAsyncCompletionInternal_h */
+#endif /* !___PDMAsyncCompletionInternal_h */
+
