@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,16 +13,13 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___GMMR0Internal_h
 #define ___GMMR0Internal_h
 
 #include <VBox/gmm.h>
+#include <iprt/avl.h>
 
 /**
  * The allocation sizes.
@@ -37,49 +34,82 @@ typedef struct GMMVMSIZES
     /** The number of pages for fixed allocations like MMIO2 and the hyper heap. */
     uint32_t        cFixedPages;
 } GMMVMSIZES;
+/** Pointer to a GMMVMSIZES. */
 typedef GMMVMSIZES *PGMMVMSIZES;
 
 
+/**
+ * Shared module registration info (per VM)
+ */
+typedef struct GMMSHAREDMODULEPERVM
+{
+    /** Tree node. */
+    AVLGCPTRNODECORE            Core;
+
+    /** Pointer to global shared module info. */
+    PGMMSHAREDMODULE            pGlobalModule;
+
+    /** Set if another VM registered a different shared module at the same base address. */
+    bool                        fCollision;
+    /** Alignment. */
+    bool                        bAlignment[3];
+
+    /** Number of included region descriptors */
+    uint32_t                    cRegions;
+
+    /** Shared region descriptor(s). */
+    GMMSHAREDREGIONDESC         aRegions[1];
+} GMMSHAREDMODULEPERVM;
+/** Pointer to a GMMSHAREDMODULEPERVM. */
+typedef GMMSHAREDMODULEPERVM *PGMMSHAREDMODULEPERVM;
+
 /**
  * The per-VM GMM data.
  */
 typedef struct GMMPERVM
 {
     /** The reservations. */
-    GMMVMSIZES      Reserved;
+    GMMVMSIZES          Reserved;
     /** The actual allocations.
      * This includes both private and shared page allocations. */
-    GMMVMSIZES      Allocated;
+    GMMVMSIZES          Allocated;
 
     /** The current number of private pages. */
-    uint64_t        cPrivatePages;
+    uint64_t            cPrivatePages;
     /** The current number of shared pages. */
-    uint64_t        cSharedPages;
+    uint64_t            cSharedPages;
     /** The current over-comitment policy. */
-    GMMOCPOLICY     enmPolicy;
+    GMMOCPOLICY         enmPolicy;
     /** The VM priority for arbitrating VMs in low and out of memory situation.
      * Like which VMs to start sequeezing first. */
-    GMMPRIORITY     enmPriority;
+    GMMPRIORITY         enmPriority;
 
     /** The current number of ballooned pages. */
-    uint64_t        cBalloonedPages;
+    uint64_t            cBalloonedPages;
     /** The max number of pages that can be ballooned. */
-    uint64_t        cMaxBalloonedPages;
+    uint64_t            cMaxBalloonedPages;
     /** The number of pages we've currently requested the guest to give us.
      * This is 0 if no pages currently requested. */
-    uint64_t        cReqBalloonedPages;
+    uint64_t            cReqBalloonedPages;
     /** The number of pages the guest has given us in response to the request.
      * This is not reset on request completed and may be used in later decisions. */
-    uint64_t        cReqActuallyBalloonedPages;
+    uint64_t            cReqActuallyBalloonedPages;
     /** The number of pages we've currently requested the guest to take back. */
-    uint64_t        cReqDeflatePages;
+    uint64_t            cReqDeflatePages;
+
+    /** Shared module tree (per-vm). */
+    PAVLGCPTRNODECORE   pSharedModuleTree;
+
     /** Whether ballooning is enabled or not. */
-    bool            fBallooningEnabled;
+    bool                fBallooningEnabled;
+
+    /** Whether shared paging is enabled or not. */
+    bool                fSharedPagingEnabled;
 
     /** Whether the VM is allowed to allocate memory or not.
      * This is used when the reservation update request fails or when the VM has
      * been told to suspend/save/die in an out-of-memory case. */
-    bool            fMayAllocate;
+    bool                fMayAllocate;
 } GMMPERVM;
 /** Pointer to the per-VM GMM data. */
 typedef GMMPERVM *PGMMPERVM;
