@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -27,6 +23,7 @@
 #include <VBox/sup.h>
 #include <VBox/trpm.h>
 #include <VBox/cpum.h>
+#include <VBox/pdmapi.h>
 #include <VBox/pgm.h>
 #include <VBox/stam.h>
 #include <VBox/tm.h>
@@ -42,8 +39,11 @@
 #include <VBox/version.h>
 #include <VBox/log.h>
 
+#include <iprt/asm-amd64-x86.h>
 #include <iprt/assert.h>
+#include <iprt/crc32.h>
 #include <iprt/mp.h>
+#include <iprt/once.h>
 #include <iprt/stdarg.h>
 #include <iprt/string.h>
 #ifdef VBOX_WITH_VMMR0_DISABLE_PREEMPTION
@@ -67,8 +67,13 @@ RT_C_DECLS_END
 /*******************************************************************************
 *   Global Variables                                                           *
 *******************************************************************************/
-/** Pointer to the internal networking service instance. */
-PINTNET g_pIntNet = 0;
+/** Drag in necessary library bits.
+ * The runtime lives here (in VMMR0.r0) and VBoxDD*R0.r0 links against us. */
+PFNRT g_VMMGCDeps[] =
+{
+    (PFNRT)RTCrc32,
+    (PFNRT)RTOnce
+};
 
 
 /**
@@ -102,18 +107,14 @@ VMMR0DECL(int) ModuleInit(void)
 #endif
                     if (RT_SUCCESS(rc))
                     {
-                        LogFlow(("ModuleInit: g_pIntNet=%p\n", g_pIntNet));
-                        g_pIntNet = NULL;
-                        LogFlow(("ModuleInit: g_pIntNet=%p should be NULL now...\n", g_pIntNet));
-                        rc = INTNETR0Create(&g_pIntNet);
+                        rc = IntNetR0Init();
                         if (RT_SUCCESS(rc))
                         {
-                            LogFlow(("ModuleInit: returns success. g_pIntNet=%p\n", g_pIntNet));
+                            LogFlow(("ModuleInit: returns success.\n"));
                             return VINF_SUCCESS;
                         }
 
                         /* bail out */
-                        g_pIntNet = NULL;
                         LogFlow(("ModuleTerm: returns %Rrc\n", rc));
 #ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
                         PGMR0DynMapTerm();
@@ -142,17 +143,12 @@ VMMR0DECL(void) ModuleTerm(void)
     LogFlow(("ModuleTerm:\n"));
 
     /*
-     * Destroy the internal networking instance.
+     * Terminate the internal network service.
      */
-    if (g_pIntNet)
-    {
-        INTNETR0Destroy(g_pIntNet);
-        g_pIntNet = NULL;
-    }
+    IntNetR0Term();
 
     /*
      * PGM (Darwin) and HWACCM global cleanup.
-     * Destroy the GMM and GVMM instances.
      */
 #ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
     PGMR0DynMapTerm();
@@ -160,6 +156,9 @@ VMMR0DECL(void) ModuleTerm(void)
     PGMDeregisterStringFormatTypes();
     HWACCMR0Term();
 
+    /*
+     * Destroy the GMM and GVMM instances.
+     */
     GMMR0Term();
     GVMMR0Term();
 
@@ -425,9 +424,6 @@ static void vmmR0RecordRC(PVM pVM, PVMCPU pVCpu, int rc)
                 case VMMCALLRING3_PDM_LOCK:
                     STAM_COUNTER_INC(&pVM->vmm.s.StatRZCallPDMLock);
                     break;
-                case VMMCALLRING3_PDM_QUEUE_FLUSH:
-                    STAM_COUNTER_INC(&pVM->vmm.s.StatRZCallPDMQueueFlush);
-                    break;
                 case VMMCALLRING3_PGM_POOL_GROW:
                     STAM_COUNTER_INC(&pVM->vmm.s.StatRZCallPGMPoolGrow);
                     break;
@@ -817,7 +813,6 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
 
         /*
          * Attempt to enable hwacc mode and check the current setting.
-         *
          */
         case VMMR0_DO_HWACC_ENABLE:
             return HWACCMR0EnableAllCpus(pVM);
@@ -876,6 +871,11 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
                 return VERR_INVALID_CPU_ID;
             return PGMR0PhysAllocateHandyPages(pVM, &pVM->aCpus[idCpu]);
 
+        case VMMR0_DO_PGM_ALLOCATE_LARGE_HANDY_PAGE:
+            if (idCpu == NIL_VMCPUID)
+                return VERR_INVALID_CPU_ID;
+            return PGMR0PhysAllocateLargeHandyPage(pVM, &pVM->aCpus[idCpu]);
+
         /*
          * GMM wrappers.
          */
@@ -899,16 +899,28 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
                 return VERR_INVALID_PARAMETER;
             return GMMR0FreePagesReq(pVM, idCpu, (PGMMFREEPAGESREQ)pReqHdr);
 
+        case VMMR0_DO_GMM_FREE_LARGE_PAGE:
+            if (u64Arg)
+                return VERR_INVALID_PARAMETER;
+            return GMMR0FreeLargePageReq(pVM, idCpu, (PGMMFREELARGEPAGEREQ)pReqHdr);
+
+        case VMMR0_DO_GMM_QUERY_HYPERVISOR_MEM_STATS:
+            if (u64Arg)
+                return VERR_INVALID_PARAMETER;
+            return GMMR0QueryHypervisorMemoryStatsReq(pVM, (PGMMMEMSTATSREQ)pReqHdr);
+
+        case VMMR0_DO_GMM_QUERY_MEM_STATS:
+            if (idCpu == NIL_VMCPUID)
+                return VERR_INVALID_CPU_ID;
+            if (u64Arg)
+                return VERR_INVALID_PARAMETER;
+            return GMMR0QueryMemoryStatsReq(pVM, idCpu, (PGMMMEMSTATSREQ)pReqHdr);
+
         case VMMR0_DO_GMM_BALLOONED_PAGES:
             if (u64Arg)
                 return VERR_INVALID_PARAMETER;
             return GMMR0BalloonedPagesReq(pVM, idCpu, (PGMMBALLOONEDPAGESREQ)pReqHdr);
 
-        case VMMR0_DO_GMM_DEFLATED_BALLOON:
-            if (pReqHdr)
-                return VERR_INVALID_PARAMETER;
-            return GMMR0DeflatedBalloon(pVM, idCpu, (uint32_t)u64Arg);
-
         case VMMR0_DO_GMM_MAP_UNMAP_CHUNK:
             if (u64Arg)
                 return VERR_INVALID_PARAMETER;
@@ -919,6 +931,60 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
                 return VERR_INVALID_PARAMETER;
             return GMMR0SeedChunk(pVM, idCpu, (RTR3PTR)u64Arg);
 
+        case VMMR0_DO_GMM_REGISTER_SHARED_MODULE:
+            if (idCpu == NIL_VMCPUID)
+                return VERR_INVALID_CPU_ID;
+            if (u64Arg)
+                return VERR_INVALID_PARAMETER;
+            return GMMR0RegisterSharedModuleReq(pVM, idCpu, (PGMMREGISTERSHAREDMODULEREQ)pReqHdr);
+
+        case VMMR0_DO_GMM_UNREGISTER_SHARED_MODULE:
+            if (idCpu == NIL_VMCPUID)
+                return VERR_INVALID_CPU_ID;
+            if (u64Arg)
+                return VERR_INVALID_PARAMETER;
+            return GMMR0UnregisterSharedModuleReq(pVM, idCpu, (PGMMUNREGISTERSHAREDMODULEREQ)pReqHdr);
+
+        case VMMR0_DO_GMM_RESET_SHARED_MODULES:
+            if (idCpu == NIL_VMCPUID)
+                return VERR_INVALID_CPU_ID;
+            if (    u64Arg
+                ||  pReqHdr)
+                return VERR_INVALID_PARAMETER;
+            return GMMR0ResetSharedModules(pVM, idCpu);
+
+#ifdef VBOX_WITH_PAGE_SHARING
+        case VMMR0_DO_GMM_CHECK_SHARED_MODULES:
+        {
+            if (idCpu == NIL_VMCPUID)
+                return VERR_INVALID_CPU_ID;
+            if (    u64Arg
+                ||  pReqHdr)
+                return VERR_INVALID_PARAMETER;
+
+            PVMCPU pVCpu = &pVM->aCpus[idCpu];
+
+            /* Select a valid VCPU context. */
+            ASMAtomicWriteU32(&pVCpu->idHostCpu, RTMpCpuId());
+
+# ifdef DEBUG_sandervl
+            /* Make sure that log flushes can jump back to ring-3; annoying to get an incomplete log (this is risky though as the code doesn't take this into account). */
+            int rc = GMMR0CheckSharedModulesStart(pVM);
+            if (rc == VINF_SUCCESS)
+            {
+                rc = vmmR0CallRing3SetJmp(&pVCpu->vmm.s.CallRing3JmpBufR0, GMMR0CheckSharedModules, pVM, pVCpu); /* this may resume code. */
+                GMMR0CheckSharedModulesEnd(pVM);
+            }
+# else
+            int rc = GMMR0CheckSharedModules(pVM, pVCpu);
+# endif
+
+            /* Clear the VCPU context. */
+            ASMAtomicWriteU32(&pVCpu->idHostCpu, NIL_RTCPUID);
+            return rc;
+        }
+#endif
+
         /*
          * A quick GCFGM mock-up.
          */
@@ -947,6 +1013,22 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
             return rc;
         }
 
+        /*
+         * PDM Wrappers.
+         */
+        case VMMR0_DO_PDM_DRIVER_CALL_REQ_HANDLER:
+        {
+            if (!pVM || !pReqHdr || u64Arg || idCpu != NIL_VMCPUID)
+                return VERR_INVALID_PARAMETER;
+            return PDMR0DriverCallReqHandler(pVM, (PPDMDRIVERCALLREQHANDLERREQ)pReqHdr);
+        }
+
+        case VMMR0_DO_PDM_DEVICE_CALL_REQ_HANDLER:
+        {
+            if (!pVM || !pReqHdr || u64Arg || idCpu != NIL_VMCPUID)
+                return VERR_INVALID_PARAMETER;
+            return PDMR0DeviceCallReqHandler(pVM, (PPDMDEVICECALLREQHANDLERREQ)pReqHdr);
+        }
 
         /*
          * Requests to the internal networking service.
@@ -956,59 +1038,48 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
             PINTNETOPENREQ pReq = (PINTNETOPENREQ)pReqHdr;
             if (u64Arg || !pReq || !vmmR0IsValidSession(pVM, pReq->pSession, pSession) || idCpu != NIL_VMCPUID)
                 return VERR_INVALID_PARAMETER;
-            if (!g_pIntNet)
-                return VERR_NOT_SUPPORTED;
-            return INTNETR0OpenReq(g_pIntNet, pSession, pReq);
+            return IntNetR0OpenReq(pSession, pReq);
         }
 
         case VMMR0_DO_INTNET_IF_CLOSE:
             if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFCLOSEREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
                 return VERR_INVALID_PARAMETER;
-            if (!g_pIntNet)
-                return VERR_NOT_SUPPORTED;
-            return INTNETR0IfCloseReq(g_pIntNet, pSession, (PINTNETIFCLOSEREQ)pReqHdr);
+            return IntNetR0IfCloseReq(pSession, (PINTNETIFCLOSEREQ)pReqHdr);
 
-        case VMMR0_DO_INTNET_IF_GET_RING3_BUFFER:
-            if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFGETRING3BUFFERREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
+        case VMMR0_DO_INTNET_IF_GET_BUFFER_PTRS:
+            if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFGETBUFFERPTRSREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
                 return VERR_INVALID_PARAMETER;
-            if (!g_pIntNet)
-                return VERR_NOT_SUPPORTED;
-            return INTNETR0IfGetRing3BufferReq(g_pIntNet, pSession, (PINTNETIFGETRING3BUFFERREQ)pReqHdr);
+            return IntNetR0IfGetBufferPtrsReq(pSession, (PINTNETIFGETBUFFERPTRSREQ)pReqHdr);
 
         case VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE:
             if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFSETPROMISCUOUSMODEREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
                 return VERR_INVALID_PARAMETER;
-            if (!g_pIntNet)
-                return VERR_NOT_SUPPORTED;
-            return INTNETR0IfSetPromiscuousModeReq(g_pIntNet, pSession, (PINTNETIFSETPROMISCUOUSMODEREQ)pReqHdr);
+            return IntNetR0IfSetPromiscuousModeReq(pSession, (PINTNETIFSETPROMISCUOUSMODEREQ)pReqHdr);
 
         case VMMR0_DO_INTNET_IF_SET_MAC_ADDRESS:
             if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFSETMACADDRESSREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
                 return VERR_INVALID_PARAMETER;
-            if (!g_pIntNet)
-                return VERR_NOT_SUPPORTED;
-            return INTNETR0IfSetMacAddressReq(g_pIntNet, pSession, (PINTNETIFSETMACADDRESSREQ)pReqHdr);
+            return IntNetR0IfSetMacAddressReq(pSession, (PINTNETIFSETMACADDRESSREQ)pReqHdr);
 
         case VMMR0_DO_INTNET_IF_SET_ACTIVE:
             if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFSETACTIVEREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
                 return VERR_INVALID_PARAMETER;
-            if (!g_pIntNet)
-                return VERR_NOT_SUPPORTED;
-            return INTNETR0IfSetActiveReq(g_pIntNet, pSession, (PINTNETIFSETACTIVEREQ)pReqHdr);
+            return IntNetR0IfSetActiveReq(pSession, (PINTNETIFSETACTIVEREQ)pReqHdr);
 
         case VMMR0_DO_INTNET_IF_SEND:
             if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFSENDREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
                 return VERR_INVALID_PARAMETER;
-            if (!g_pIntNet)
-                return VERR_NOT_SUPPORTED;
-            return INTNETR0IfSendReq(g_pIntNet, pSession, (PINTNETIFSENDREQ)pReqHdr);
+            return IntNetR0IfSendReq(pSession, (PINTNETIFSENDREQ)pReqHdr);
 
         case VMMR0_DO_INTNET_IF_WAIT:
             if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFWAITREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
                 return VERR_INVALID_PARAMETER;
-            if (!g_pIntNet)
-                return VERR_NOT_SUPPORTED;
-            return INTNETR0IfWaitReq(g_pIntNet, pSession, (PINTNETIFWAITREQ)pReqHdr);
+            return IntNetR0IfWaitReq(pSession, (PINTNETIFWAITREQ)pReqHdr);
+
+        case VMMR0_DO_INTNET_IF_ABORT_WAIT:
+            if (u64Arg || !pReqHdr || !vmmR0IsValidSession(pVM, ((PINTNETIFWAITREQ)pReqHdr)->pSession, pSession) || idCpu != NIL_VMCPUID)
+                return VERR_INVALID_PARAMETER;
+            return IntNetR0IfAbortWaitReq(pSession, (PINTNETIFABORTWAITREQ)pReqHdr);
 
         /*
          * For profiling.
@@ -1105,7 +1176,6 @@ VMMR0DECL(int) VMMR0EntryEx(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperation,
             case VMMR0_DO_GMM_ALLOCATE_PAGES:
             case VMMR0_DO_GMM_FREE_PAGES:
             case VMMR0_DO_GMM_BALLOONED_PAGES:
-            case VMMR0_DO_GMM_DEFLATED_BALLOON:
             /* On the mac we might not have a valid jmp buf, so check these as well. */
             case VMMR0_DO_VMMR0_INIT:
             case VMMR0_DO_VMMR0_TERM:
@@ -1298,19 +1368,19 @@ DECLEXPORT(bool) RTCALL RTAssertShouldPanic(void)
  * @param   pszFile     Location file name.
  * @param   pszFunction Location function name.
  */
-DECLEXPORT(void) RTCALL AssertMsg1(const char *pszExpr, unsigned uLine, const char *pszFile, const char *pszFunction)
+DECLEXPORT(void) RTCALL RTAssertMsg1Weak(const char *pszExpr, unsigned uLine, const char *pszFile, const char *pszFunction)
 {
-#if !defined(DEBUG_sandervl) && !defined(RT_OS_DARWIN)
-    SUPR0Printf("\n!!R0-Assertion Failed!!\n"
-                "Expression: %s\n"
-                "Location  : %s(%d) %s\n",
-                pszExpr, pszFile, uLine, pszFunction);
-#endif
+    /*
+     * To the log.
+     */
     LogAlways(("\n!!R0-Assertion Failed!!\n"
                "Expression: %s\n"
                "Location  : %s(%d) %s\n",
                pszExpr, pszFile, uLine, pszFunction));
 
+    /*
+     * To the global VMM buffer.
+     */
     PVM pVM = GVMMR0GetVMByEMT(NIL_RTNATIVETHREAD);
     if (pVM)
         RTStrPrintf(pVM->vmm.s.szRing0AssertMsg1, sizeof(pVM->vmm.s.szRing0AssertMsg1),
@@ -1318,9 +1388,11 @@ DECLEXPORT(void) RTCALL AssertMsg1(const char *pszExpr, unsigned uLine, const ch
                     "Expression: %s\n"
                     "Location  : %s(%d) %s\n",
                     pszExpr, pszFile, uLine, pszFunction);
-#ifdef RT_OS_DARWIN
+
+    /*
+     * Continue the normal way.
+     */
     RTAssertMsg1(pszExpr, uLine, pszFile, pszFunction);
-#endif
 }
 
 
@@ -1331,41 +1403,47 @@ DECLEXPORT(void) RTCALL AssertMsg1(const char *pszExpr, unsigned uLine, const ch
 static DECLCALLBACK(size_t) rtLogOutput(void *pv, const char *pachChars, size_t cbChars)
 {
     for (size_t i = 0; i < cbChars; i++)
-    {
-#if !defined(DEBUG_sandervl) && !defined(RT_OS_DARWIN)
-        SUPR0Printf("%c", pachChars[i]);
-#endif
         LogAlways(("%c", pachChars[i]));
-    }
 
     return cbChars;
 }
 
 
-DECLEXPORT(void) RTCALL AssertMsg2(const char *pszFormat, ...)
+/**
+ * Override this so we can push it up to ring-3.
+ *
+ * @param   pszFormat   The format string.
+ * @param   va          Arguments.
+ */
+DECLEXPORT(void) RTCALL RTAssertMsg2WeakV(const char *pszFormat, va_list va)
 {
-    va_list va;
+    va_list vaCopy;
 
+    /*
+     * Push the message to the logger.
+     */
     PRTLOGGER pLog = RTLogDefaultInstance(); /** @todo we want this for release as well! */
     if (pLog)
     {
-        va_start(va, pszFormat);
-        RTLogFormatV(rtLogOutput, pLog, pszFormat, va);
-        va_end(va);
-
-        PVM pVM = GVMMR0GetVMByEMT(NIL_RTNATIVETHREAD);
-        if (pVM)
-        {
-            va_start(va, pszFormat);
-            RTStrPrintfV(pVM->vmm.s.szRing0AssertMsg2, sizeof(pVM->vmm.s.szRing0AssertMsg2), pszFormat, va);
-            va_end(va);
-        }
+        va_copy(vaCopy, va);
+        RTLogFormatV(rtLogOutput, pLog, pszFormat, vaCopy);
+        va_end(vaCopy);
     }
 
-#ifdef RT_OS_DARWIN
-    va_start(va, pszFormat);
+    /*
+     * Push it to the global VMM buffer.
+     */
+    PVM pVM = GVMMR0GetVMByEMT(NIL_RTNATIVETHREAD);
+    if (pVM)
+    {
+        va_copy(vaCopy, va);
+        RTStrPrintfV(pVM->vmm.s.szRing0AssertMsg2, sizeof(pVM->vmm.s.szRing0AssertMsg2), pszFormat, vaCopy);
+        va_end(vaCopy);
+    }
+
+    /*
+     * Continue the normal way.
+     */
     RTAssertMsg2V(pszFormat, va);
-    va_end(va);
-#endif
 }
 
