@@ -1,27 +1,31 @@
-/* $Id: kLdr.h 2 2007-11-16 16:07:14Z bird $ */
+/* $Id: kLdr.h 29 2009-07-01 20:30:29Z bird $ */
 /** @file
- *
  * kLdr - The Dynamic Loader.
+ */
+
+/*
+ * Copyright (c) 2006-2007 Knut St. Osmundsen <bird-kStuff-spamix@anduin.net>
  *
- * Copyright (c) 2006 knut st. osmundsen <bird@anduin.net>
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
  *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
  *
- * This file is part of kLdr.
- *
- * kLdr is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * kLdr is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with kLdr; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
  */
 
 #ifndef ___k_kLdr_h___
@@ -240,7 +244,7 @@ typedef struct KLDRSEG
     /** Segment flags. */
     KU32            fFlags;
     /** The segment protection. */
-    KPROT        enmProt;
+    KPROT           enmProt;
     /** The size of the segment. */
     KLDRSIZE        cb;
     /** The required segment alignment.
@@ -581,8 +585,8 @@ typedef FNKLDRENUMRSRC *PFNKLDRENUMRSRC;
 /** @} */
 
 
-int     kLdrModOpen(const char *pszFilename, PPKLDRMOD ppMod);
-int     kLdrModOpenFromRdr(PKRDR pRdr, PPKLDRMOD ppMod);
+int     kLdrModOpen(const char *pszFilename, KU32 fFlags, KCPUARCH enmCpuArch, PPKLDRMOD ppMod);
+int     kLdrModOpenFromRdr(PKRDR pRdr, KU32 fFlags, KCPUARCH enmCpuArch, PPKLDRMOD ppMod);
 int     kLdrModOpenNative(const char *pszFilename, PPKLDRMOD ppMod);
 int     kLdrModOpenNativeByHandle(KUPTR uHandle, PPKLDRMOD ppMod);
 int     kLdrModClose(PKLDRMOD pMod);
@@ -646,10 +650,14 @@ typedef struct KLDRMODOPS
      *          On failure, a non-zero OS specific error code is returned.
      * @param   pOps            Pointer to the registered method table.
      * @param   pRdr            The file provider instance to use.
+     * @param   fFlags          Flags, MBZ.
+     * @param   enmCpuArch      The desired CPU architecture. KCPUARCH_UNKNOWN means
+     *                          anything goes, but with a preference for the current
+     *                          host architecture.
      * @param   offNewHdr       The offset of the new header in MZ files. -1 if not found.
      * @param   ppMod           Where to store the module instance pointer.
      */
-    int (* pfnCreate)(PCKLDRMODOPS pOps, PKRDR pRdr, KLDRFOFF offNewHdr, PPKLDRMOD ppMod);
+    int (* pfnCreate)(PCKLDRMODOPS pOps, PKRDR pRdr, KU32 fFlags, KCPUARCH enmCpuArch, KLDRFOFF offNewHdr, PPKLDRMOD ppMod);
     /**
      * Destroys an loader module instance.
      *
